const http = require('http');
const vm = require('vm');
const utilsMethod = require('./utils');
const app = require('./app');
const config = require('./config');
const ApiClient = require('@airiot/sdk-nodejs/api');
const Rabbit = require('@airiot/sdk-nodejs/conn/rabbit');
const KafkaJS = require('@airiot/sdk-nodejs/conn/kafka');
const Mqtt = require('@airiot/sdk-nodejs/conn/mqtt');

var jsondiffpatch = require("jsondiffpatch")
var _ = require('lodash');


const {Etcd3} = require('etcd3');
const client = new Etcd3({
    hosts: config.etcd.endpoints,
    auth: {
        username: config.etcd.username,
        password: config.etcd.password
    }
});

const keyToRead = '/airiot/config/pro.json';
// const getSchema = require('./utils');
// let schemaCache = require('./schema.js');
class SchemaCacheGenerate {
    constructor(config) {
        this.config = config
    }


    mqttInit() {
        let _this = this
        _this.mqClient.receive({topic: 'logHandle/#', qos: 0}, (err,topic, message) => {
            const ts = topic.split('/')
            console.log("消息", topic)
            if (ts.length === 3 && ts[0] === 'logHandle') {
                let msg = {}
                try {
                    msg = JSON.parse(message)
                } catch (err) {
                    console.error(`解析class topic:${topic},错误:${err}`)
                }
                if (msg.logHandleType) {
                    console.log('msg:', msg)
                    let projectID = ts[1]
                    switch (msg.logHandleType) {
                        case "dashboard":
                            switch (msg.logHandleAction) {
                                case "add":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">添加</span>` + "画面";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "edit":
                                    _this.dashboardDiff(msg, projectID);
                                    // let diffWordDashboard = ''
                                    // try {
                                    //     diffWordDashboard = await _this.testDashboardSchema(msg)
                                    // } catch (err) {
                                    //     console.error('testDashboardSchema:', err)
                                    // }
                                    // _this.testDashboardCellsSchema(msg, diffWordDashboard);
                                    break;
                                case "delete":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: red">删除</span>` + "画面";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                                case "share":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">分享</span>` + "画面";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                            }
                            break;
                        case "report":
                            switch (msg.logHandleAction) {
                                case "add":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">添加</span>` + "报表";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "edit":
                                    _this.testReportSchema(msg, projectID);
                                    break;
                                case "delete":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: red">删除</span>` + "报表";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                            }
                            break;
                        case "reportcopy":
                            switch (msg.logHandleAction) {
                                case "add":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">添加</span>` + "报表副本";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "edit":
                                    _this.testReportSchema(msg, projectID);
                                    break;
                                case "delete":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: red">删除</span>` + "报表副本";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                            }
                            break;
                        case "event":
                            switch (msg.logHandleAction) {
                                case "add":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">添加</span>` + "事件";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "edit":
                                    _this.testEventSchema(msg, projectID);
                                    break;
                                case "delete":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: red">删除</span>` + "事件";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                            }
                            break;
                        case "tableData":
                            switch (msg.logHandleAction) {
                                case "add":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">添加</span>` + "表记录";
                                    if (msg.extTableName) {
                                        data = data + "(" + '表名称为' + msg.extTableName + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "edit":
                                    _this.testExtSchema(msg, projectID);
                                    break;
                                case "delete":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: red">删除</span>` + "表记录";
                                    if (msg.extTableName) {
                                        data = data + "(" + '表名称为' + msg.extTableName + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                            }
                            break;
                        case "table":
                            switch (msg.logHandleAction) {
                                case "add":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">添加</span>` + "表";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "edit":
                                    _this.testTableSchema(msg, projectID);
                                    break;
                                case "delete":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: red">删除</span>` + "表";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                            }
                            break;
                        case "eventHandler":
                            switch (msg.logHandleAction) {
                                case "add":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">添加</span>` + "事件handle";
                                    if (msg.data && msg.data.handlername) {
                                        data = data + "(" + '名称为' + msg.data.handlername + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "edit":
                                    _this.testEventHandlerSchema(msg, projectID);
                                    break;
                                case "delete":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: red">删除</span>` + "事件handle";
                                    if (msg.data && msg.data.handlername) {
                                        data = data + "(" + '名称为' + msg.data.handlername + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                            }
                            break;
                        case "warning":
                            switch (msg.logHandleAction) {
                                case "edit":
                                    _this.testWarnRemarkSchema(msg, projectID);
                                    break;
                                case "process":
                                    _this.testWarnProcess(msg, projectID);
                                    break;
                                case "confirm":
                                    _this.testWarnConfirm(msg, projectID);
                                    break;
                            }
                            break;
                        case "user":
                            switch (msg.logHandleAction) {
                                case "pwd":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: blue">修改</span>` + "了" + msg.userops.name + "的密码";
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "add":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">添加</span>` + "用户";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "edit":
                                    _this.testUserSchema(msg, projectID);
                                    break;
                                case "delete":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: red">删除</span>` + "用户";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                            }
                            break;
                        case "app":
                            switch (msg.logHandleAction) {
                                case "add":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">添加</span>` + "扩展应用";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "edit":
                                    _this.testUserSchema(msg, projectID);
                                    break;
                                case "delete":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: red">删除</span>` + "扩展应用";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                            }
                            break;
                        case "oauthapp":
                            switch (msg.logHandleAction) {
                                case "add":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">添加</span>` + "oauthapp";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "edit":
                                    _this.testOauthAppSchema(msg, projectID);
                                    break;
                                case "delete":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: red">删除</span>` + "oauthapp";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                            }
                            break;
                        case "login":
                            switch (msg.logHandleAction) {
                                case "login":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = (msg.user && msg.user.name ? msg.user.name : msg.userName) + '于' + formatTime + '登录';
                                    //var data = msg.user.name + '于' + formatTime + '登录';
                                    if ((msg.data && msg.data.name && msg.data.name !== '')) {
                                        data = data + `<span style="color: red">失败</span>` + '(' + msg.data.name + ')'
                                    } else {
                                        data = data + `<span style="color: green">成功</span>`
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                            }
                            break;
                        case "dept":
                            switch (msg.logHandleAction) {
                                case "add":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">添加</span>` + "部门";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "edit":
                                    _this.testDeptSchema(msg, projectID);
                                    break;
                                case "delete":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: red">删除</span>` + "部门";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                            }
                            break;
                        case "sync":
                            switch (msg.logHandleAction) {
                                case "add":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">添加</span>` + "同步配置";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "edit":
                                    _this.testSyncSchema(msg, projectID);
                                    break;
                                case "delete":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: red">删除</span>` + "同步配置";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                                case "instant":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">执行</span>` + "一次性同步";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    if ((msg.execError && msg.execError !== '')) {
                                        data = data + `<span style="color: red">失败</span>` + '(' + msg.execError + ')'
                                    } else {
                                        data = data + `<span style="color: green">成功</span>`
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "interval":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">执行</span>` + "定时同步";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    if ((msg.execError && msg.execError !== '')) {
                                        data = data + `<span style="color: red">失败</span>` + '(' + msg.execError + ')'
                                    } else {
                                        data = data + `<span style="color: green">成功</span>`
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                            }
                            break;
                        case "warningarchive":
                            switch (msg.logHandleAction) {
                                case "add":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">添加</span>` + "报警归档配置";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "edit":
                                    _this.testWarningArchiveSchema(msg, projectID);
                                    break;
                                case "delete":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: red">删除</span>` + "报警归档配置";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                                case "instant":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">执行</span>` + "一次性报警归档";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    if ((msg.execError && msg.execError !== '')) {
                                        data = data + `<span style="color: red">失败</span>` + '(' + msg.execError + ')'
                                    } else {
                                        data = data + `<span style="color: green">成功</span>`
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "interval":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">执行</span>` + "定时报警归档";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    if ((msg.execError && msg.execError !== '')) {
                                        data = data + `<span style="color: red">失败</span>` + '(' + msg.execError + ')'
                                    } else {
                                        data = data + `<span style="color: green">成功</span>`
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                            }
                            break;
                        case "warningclean":
                            switch (msg.logHandleAction) {
                                case "add":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">添加</span>` + "报警清除配置";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "edit":
                                    _this.testWarningCleanSchema(msg, projectID);
                                    break;
                                case "delete":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: red">删除</span>` + "报警清除配置";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                                case "instant":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">执行</span>` + "一次性报警清除";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    if ((msg.execError && msg.execError !== '')) {
                                        data = data + `<span style="color: red">失败</span>` + '(' + msg.execError + ')'
                                    } else {
                                        data = data + `<span style="color: green">成功</span>`
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "interval":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">执行</span>` + "定时报警清除";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    if ((msg.execError && msg.execError !== '')) {
                                        data = data + `<span style="color: red">失败</span>` + '(' + msg.execError + ')'
                                    } else {
                                        data = data + `<span style="color: green">成功</span>`
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                            }
                            break;
                        case "flow":
                            switch (msg.logHandleAction) {
                                case "add":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">添加</span>` + "流程";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "edit":
                                    _this.testFlowSchema(msg, projectID);
                                    break;
                                case "delete":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: red">删除</span>` + "流程";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                            }
                            break;
                        case "flowtask":
                            switch (msg.logHandleAction) {
                                case "add":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">添加</span>` + "流程任务";
                                    if (msg.data && msg.data.type) {
                                        if (msg.data.type === 'fillin') {
                                            data = data + "(类型为填写)"
                                        } else if (msg.data.type === 'approve') {
                                            data = data + "(类型为审批)"
                                        } else if (msg.data.type === 'cc') {
                                            data = data + "(类型为抄送)"
                                        }
                                    }
                                    if (msg.flow && msg.flow.name) {
                                        data = data + "(" + '所属流程为' + msg.flow.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "edit":
                                    _this.testFlowTaskSchema(msg, projectID);
                                    break;
                                case "delete":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: red">删除</span>` + "流程任务";
                                    if (msg.data && msg.data.type) {
                                        if (msg.data.type === 'fillin') {
                                            data = data + "(类型为填写)"
                                        } else if (msg.data.type === 'approve') {
                                            data = data + "(类型为审批)"
                                        } else if (msg.data.type === 'cc') {
                                            data = data + "(类型为抄送)"
                                        }
                                    }
                                    if (msg.flow && msg.flow.name) {
                                        data = data + "(" + '所属流程为' + msg.flow.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                                case "agree":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">通过</span>` + "流程任务";
                                    if (msg.data && msg.data.jobKey) {
                                        data = data + "(" + '标识jobKey为' + msg.data.jobKey + ")"
                                    }
                                    if (msg.flow && msg.flow.name) {
                                        data = data + "(" + '所属流程为' + msg.flow.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                                case "disagree":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: red">拒绝</span>` + "流程任务";
                                    if (msg.data && msg.data.jobKey) {
                                        data = data + "(" + '标识jobKey为' + msg.data.jobKey + ")"
                                    }
                                    if (msg.flow && msg.flow.name) {
                                        data = data + "(" + '所属流程为' + msg.flow.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                                case "referral":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">转审</span>` + "流程任务";
                                    if (msg.data && msg.data.jobKey) {
                                        data = data + "(" + '标识jobKey为' + msg.data.jobKey + ")"
                                    }
                                    if (msg.flow && msg.flow.name) {
                                        data = data + "(" + '所属流程为' + msg.flow.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                            }
                            break;
                        case "role":
                            switch (msg.logHandleAction) {
                                case "add":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">添加</span>` + "角色";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "edit":
                                    _this.testRoleSchema(msg, projectID);
                                    break;
                                case "delete":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: red">删除</span>` + "角色";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                            }
                            break;
                        case "systemVariable":
                            switch (msg.logHandleAction) {
                                case "add":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: green">添加</span>` + "系统变量";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break;
                                case "edit":
                                    _this.testSystemVariableSchema(msg, projectID);
                                    break;
                                case "delete":
                                    var formatTime = msg.time.replace(new RegExp('T', 'g'), " ");
                                    formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                                    formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                                    var data = msg.user.name + '于' + formatTime + `<span style="color: red">删除</span>` + "系统变量";
                                    if (msg.data && msg.data.name) {
                                        data = data + "(" + '名称为' + msg.data.name + ")"
                                    }
                                    var updateMap = {'diff': data}
                                    _this.updateLog(_this.tokenStr, _this.config, updateMap, msg.id, projectID)
                                    break
                            }
                            break;
                    }
                }
            }
        })
    }

    // 获取token
    token(projectID) {
        // return global.tokenCache[projectID];
        return '';
    }

    async getValue(key) {
        try {
            console.log('getValue in:')
            const value = await client.get(key).string();
            console.log(`The value for ${key} is: ${value}`);
            return value;
        } catch (error) {
            console.error('Error reading from etcd:', error);
            throw error; // 抛出错误可能需要上层处理
        } finally {
            client.close(); // 确保清理资源
        }
    }

    // 更新日志
    async updateLog(token, config, updateMap, logId, projectID) {
        //let content = JSON.stringify(updateMap);
        // if (token === undefined) {
        //     token = await this.token(projectID)
        // }
        // if (token === undefined){
        //     token = ''
        // }
        // console.log(projectID,'log token:',token)
        // return new Promise((resolve, reject) => {
        //     // let filter = encodeURIComponent(JSON.stringify({
        //     //     // "filter": {"modelId": modelId},
        //     //     "project": {"custom": 1}
        //     // }))
        //     let options = {
        //         hostname: config.gateway.host || 'traefik',
        //         port: config.gateway.port || 80,
        //         path: `/core/log/${logId}`,
        //         headers: {
        //             accept: "application/json",
        //             Authorization: token,
        //             "Request-Type": "service",
        //             "x-request-project": projectID,
        //         },
        //         method: "PATCH"
        //     };
        //     if (token == '') {
        //         options = {
        //             hostname: config.gateway.host || 'traefik',
        //             port: config.gateway.port || 80,
        //             path: `/core/log/${logId}`,
        //             headers: {
        //                 accept: "application/json",
        //                 "Request-Type": "service",
        //                 "x-request-project": projectID,
        //             },
        //             method: "PATCH"
        //         };
        //     }
        //     var req = http.request(options, function (res, err) {
        //         // console.log('STATUS: ' + res.statusCode);
        //         // console.log('HEADERS: ' + JSON.stringify(res.headers));
        //         if (res.statusCode !== 200) {
        //             return reject("日志修改失败",res.statusCode);
        //         }
        //         res.setEncoding("utf8");
        //         let rawData = '';
        //         res.on('data', (chunk) => {
        //             rawData += chunk;
        //         });
        //         res.on('end', () => {
        //             try {
        //                 const parsedData = JSON.parse(rawData);
        //                 console.log('日志修改 rawData:', rawData, 'content:', content)
        //                 return resolve(parsedData);
        //             } catch (e) {
        //                 return reject(e);
        //             }
        //         });
        //     });
        //
        //     req.on("error", function (e) {
        //         console.log("problem with request: " + e.message);
        //         return reject(e);
        //     });
        //
        //     req.write(content)
        //     req.end();
        // }).catch((error) => {
        //     throw new Error(error);
        // });
        return this.apiCli.updateLog(projectID, logId, updateMap)
    };

    //保持方法调用按照顺序，使用async/await 内部方法使用Promise
    async dashboardDiff(msg, projectID) {
        let _this = this;
        let diffWordDashboard = null;
        try {
            diffWordDashboard = await _this.testDashboardSchema(msg, projectID)
        } catch (err) {
            console.error('语义化画面Schema失败:', err, "==projectID:", projectID, "msg:", msg)
            return
        }
        _this.testDashboardCellsSchema(msg, diffWordDashboard, projectID);
    };

    async testNodeSchema(wholeLog, projectID) {
        let _this = this
        //生成addLogMap，editLogMap，removeLogMap的key，val
        const recursionMergePath = (obj, walkJsonPath) => {
            return Object.keys(obj).map(k => {
                // console.log("k:", k, "v:", obj[k])

                if (isNaN(k) && k !== '_t') {
                    let testNumber = k.replace('_', "");
                    if (isNaN(testNumber)) {
                        if (walkJsonPath === '') {
                            walkJsonPath = k
                        } else {
                            walkJsonPath = walkJsonPath + '>' + k
                        }
                    }
                }

                if (_.isPlainObject(obj[k])) {
                    recursionMergePath(obj[k], walkJsonPath)
                }

                if (obj[k] instanceof Array) {
                    switch (obj[k].length) {
                        case 1:
                            addLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                        case 2:
                            editLogMap[walkJsonPath] = obj[k][0] + ">" + obj[k][1];
                            splitWalk = walkJsonPath.split('>');
                            // console.log('splitWalk:',splitWalk)
                            splitWalkLength = splitWalk.length - 1;
                            // console.log('splitWalkLength:',splitWalkLength)
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                    // console.log('splitWalkKey:',k,'splitWalkVal:',splitWalk[k],'k !== splitWalkLength:',k !== splitWalkLength)
                                    // console.log(Object.prototype.toString.call(k))
                                    // console.log(Object.prototype.toString.call(splitWalkLength))
                                }
                            });
                            // console.log('splitwalkJsonPath:', walkJsonPath)
                            break;
                        case 3:
                            removeLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                    }
                }


                // const value = obj[k]
                // let title = properties[k] && properties[k].title
                // // console.log(value,title,properties)
                // if (!title && k === 'settings') title = '驱动设置'
                // if (!title || !properties) return
                // if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                //     const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                //     return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                // }
                // if (_.isPlainObject(value) && k !== 'logic') {
                //     let objProperties = properties && properties[k] && properties[k].properties
                //     if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                //         objProperties = properties && properties[k] && properties[k].items.properties
                //     }
                //     return {[title]: convertObjectKey(value, objProperties)}
                // } else {
                //     return {[title]: value}
                // }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // originToAvailable中的递归方法
        const convertObjectKey = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                let title = properties[k] && properties[k].title
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) return
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertObjectKey(value, objProperties)}
                } else {
                    return {[title]: value}
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // originToAvailable中的递归方法
        const convertObjectKeyCustom = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                let title = properties[k] && properties[k].title
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) {
                    return {[k]: value}
                }
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    return {[title]: value.map(item => convertObjectKeyCustom(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertObjectKeyCustom(value, objProperties)}
                } else {
                    if (properties[k].enum && properties[k].enum_title && _.isArray(properties[k].enum) && _.isArray(properties[k].enum_title)) {
                        let selectorEnumMap = {};
                        properties[k].enum.forEach(function (obj, index) {
                            if (properties[k].enum_title[index]) {
                                selectorEnumMap[obj] = properties[k].enum_title[index]
                            }
                        });
                        return {[title]: selectorEnumMap[value]}
                    } else {
                        return {[title]: value}
                    }
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // 原始数据转换为可读性数据(转中文map)
        const originToAvailable = (value, model, projectID) => {
            let driversSchema = {}

            let data = {}
            try {
                // driversSchema = app.get('drivers')
                if (!global.schemaCache[projectID]) {
                    console.log('global.schemaCache[projectID]未加载完成')
                    return {}
                }
                driversSchema = global.schemaCache[projectID]["device"]
            } catch (error) {
                console.log('加载device模块失败！')
            }
            // const nodeSchema = app.get('models')['Node']
            // const m = convertObjectKey(value, nodeSchema.properties)
            if (value && _.isPlainObject(value)) {
                Object.keys(value).forEach(objKey => {
                    // 资产
                    if (objKey === 'department') {
                        let u = []
                        if (value.department && value.department.length > 0) {
                            u = value.department.map(item => item.name).join(',')
                        }
                        data['所属部门'] = u
                    } else if (objKey === 'parent') {
                        let p = []
                        if (value.parent && value.parent.length > 0) {
                            p = value.parent.map(item => item.name).join(',')
                        }
                        data['所属资产'] = p
                    } else if (objKey === 'name') {
                        data['名称'] = value.name
                    } else if (objKey === 'remark') {
                        data['备注'] = value.remark
                    } else if (objKey === 'creator') {
                        data['创建人'] = value.creator
                    } else if (objKey === 'focus') {
                        data['关注'] = value.focus ? '是' : '否'
                    } else if (objKey === 'online') {
                        data['在线'] = value.online ? '在线' : '掉线'
                    } else if (objKey === 'off') {
                        data['断电'] = value.off ? '上电' : '断电'
                    } else if (objKey === 'disable') {
                        data['禁用'] = value.disable ? '是' : '否'
                    } else if (objKey === 'uid') {
                        data['编号'] = value.uid
                    } else if (objKey === 'gis') {
                        data['地理信息'] = value.gis
                    } else if (objKey === 'model') {
                        data['模型'] = value.model.name
                    } else if (objKey === 'dashboard') {
                        data['画面'] = value.dashboard.did
                    } else if (objKey === 'video') {
                        const u = {}
                        if (value.video.url && value.video.url.length > 0) {
                            u['视频地址'] = value.video.url.map(item => ({id: item.key, '地址': item.value}))
                        }
                        data['视频'] = u
                    } else if (objKey === 'device' && !_.isEmpty(value.device)) {
                        let d = {}
                        const driver = model && model.model && model.model.device && model.model.device.driver
                        if (driver && driversSchema[driver]) {
                            const properties = driversSchema[driver].device.properties
                            d = convertObjectKey(value.device, properties)
                        }
                        data['设备配置'] = d
                    } else if (objKey === 'computed') {
                        let c = {}
                        // const computedSchema = getNodeSchema({ name: 'computed' })

                        const computedSchema = global.schemaCache[projectID]['computed']
                        if (computedSchema) {
                            const properties = computedSchema.properties
                            c = convertObjectKey(value.computed, properties)
                        }
                        data['计算节点'] = c
                    } else if (objKey === 'warning') {
                        let w = {}
                        // const warningSchema = getNodeSchema({ name: 'warning' })
                        const warningSchema = global.schemaCache[projectID]['warning']
                        if (warningSchema) {
                            const properties = warningSchema.properties
                            w = convertObjectKey(value.warning, properties)
                        }
                        data['报警信息'] = w
                    } else if (objKey === 'hasChild'
                        || objKey === 'modifyTime'
                        || objKey === 'timeoutTime'
                        || objKey === 'connectTime'
                        || objKey === 'createTime'
                        || objKey === 'offlined' // ？？？未知字段
                        || objKey === 'drivers'  // ？？？未知字段
                    ) {
                        //  忽略属性
                    } else if (objKey === "warning.restrainWarning") {
                        data["报警信息"] = {"抑制报警": value[objKey]}
                    } else {
                        data[objKey] = value[objKey]
                    }
                })
            } else {
                data = value
            }

            return data
        };

        //格式化为一句话
        const formatDiffToWord = (wholeLog, addLogMap, editLogMap, removeLogMap) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + ":";
            data = data + "\n"
            if (addLogMap.length !== 0) {
                Object.keys(addLogMap).forEach(objKey => {
                    if (!addLogMap[objKey]) {
                        data = data + `<span style="color: green">添加</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!addLogMap[objKey].名称 && _.isPlainObject(addLogMap[objKey]) || addLogMap[objKey] instanceof Array) {
                    //     addLogMap[objKey] = JSON.stringify(addLogMap[objKey])
                    // }
                    if (addLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = ''
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (editLogMap.length !== 0) {
                Object.keys(editLogMap).forEach(objKey => {
                    if (!editLogMap[objKey]) {
                        return
                    }
                    let lastWordList = editLogMap[objKey].split('>');
                    let lastWord = '';
                    let originWord = '';
                    if (lastWordList.length > 1) {
                        originWord = lastWordList[0]
                        lastWord = lastWordList[lastWordList.length - 1]
                    }
                    data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ");"
                    data = data + "\n"
                })
            }
            if (removeLogMap.length !== 0) {
                Object.keys(removeLogMap).forEach(objKey => {
                    if (!removeLogMap[objKey]) {
                        data = data + `<span style="color: red">删除</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!removeLogMap[objKey].名称 && _.isPlainObject(removeLogMap[objKey]) || removeLogMap[objKey] instanceof Array) {
                    //     removeLogMap[objKey] = JSON.stringify(removeLogMap[objKey])
                    // }
                    if (removeLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = '';
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };

        let addLogMap = {};
        let editLogMap = {};
        let removeLogMap = {};

        let walkJsonPath = '';
        let splitWalk = [];
        let splitWalkLength = 0;


        // let wholeLog = {
        //     user: {
        //         "id": "22222",
        //         "name": "张三"
        //     },
        //     time: "2020-07-06T09:57:40+08:00",
        //     model: {"id": "5c934d3b839a5547624eaebf", "name": "采油井#"},
        //     oldData: {
        //         "computed": {
        //             "tags": [
        //                 {
        //                     "id": "YXZT",
        //                     "mapping": {
        //                         "id": "YXZT",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "运行状态",
        //                     "policy": "save",
        //                     "type": "mapping",
        //                 },
        //                 {
        //                     "id": "YW",
        //                     "mapping": {
        //                         "id": "YW",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "油温",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //                 {
        //                     "id": "YW2",
        //                     "mapping": {
        //                         "id": "YW2",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "油温2",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //             ]
        //         },
        //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站"},
        //         "custom": {"record": "测试记录"},
        //         "warning": {
        //             "rules": [
        //                 {
        //                     "alert": true,
        //                     "description": "注气焖井油温>200",
        //                     "disable": true,
        //                     "extraTags": [
        //                         {
        //                             "id": "lat"
        //                         }
        //                     ],
        //                     "handle": false,
        //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
        //                     "interval": 30,
        //                     "level": "中",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "==": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     200
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
        //                     "warningname": "注气报警"
        //                 },
        //                 {
        //                     "alert": true,
        //                     "description": "非注气状态报警",
        //                     "disable": true,
        //                     "handle": true,
        //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "!=": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     180
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "超限报警",
        //                     "warningname": "非注气状态"
        //                 }
        //             ]
        //         }
        //     },
        //     data: {
        //         "computed": {
        //             "tags": [
        //                 {
        //                     "id": "YXZT",
        //                     "mapping": {
        //                         "id": "YXZT",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "运行状态1",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //                 {
        //                     "id": "IA",
        //                     "mapping": {
        //                         "id": "IA",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "A相电流",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 }
        //             ],
        //         },
        //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站1"},
        //         "custom": {"record": "测试记录1", "select-4F10": "测试选择器111"},
        //         "warning": {
        //             "rules": [
        //                 {
        //                     "alert": true,
        //                     "description": "注气焖井油温>200",
        //                     "disable": true,
        //                     "extraTags": [
        //                         {
        //                             "id": "lat"
        //                         }
        //                     ],
        //                     "handle": false,
        //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
        //                     "interval": 30,
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "==": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     200
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
        //                     "warningname": "注气报警"
        //                 },
        //                 {
        //                     "alert": true,
        //                     "description": "非注气状态报警",
        //                     "disable": true,
        //                     "handle": true,
        //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "!=": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     180
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "超限报警",
        //                     "warningname": "非注气状态"
        //                 }
        //             ]
        //         }
        //     }
        // };

        let originStr = wholeLog.oldData
        let newStr = wholeLog.data

        let newChineseStr = originToAvailable(newStr, wholeLog, projectID);

        if (Object.keys(newChineseStr).length === 0) {
            console.log("newChineseStr global缓存pending")
            return
        }

        let originChineseStr = originToAvailable(originStr, wholeLog, projectID);

        console.log("newChineseStr:", JSON.stringify(newChineseStr))

        console.log("originChineseStr:", JSON.stringify(originChineseStr))

        //提取出两个json中不同的部分
        let deltChinese = jsondiffpatch.diff(originChineseStr, newChineseStr);

        if (!deltChinese) {
            let updateMap = {'diff': ''}
            try {
                let tokenProject = await _this.token(projectID)
                _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
            } catch (err) {
                console.error('updateLog err:', err)
            }
            return
        }
        console.log("deltChinese:", JSON.stringify(deltChinese))

        recursionMergePath(deltChinese, walkJsonPath)
        // console.log(recursionMergePath(deltChinese, walkJsonPath));

        // console.log('walkJsonPath Chinese:', walkJsonPath);

        console.log('addLogMap Chinese:', addLogMap);

        console.log('editLogMap Chinese:', editLogMap);

        console.log('removeLogMap Chinese:', removeLogMap);

        console.log('formatDiffToWord:');
        let diffWord = formatDiffToWord(wholeLog, addLogMap, editLogMap, removeLogMap)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };

    async testModelSchema(wholeLog, projectID) {
        let _this = this
        //生成addLogMap，editLogMap，removeLogMap的key，val
        const recursionMergePath = (obj, walkJsonPath) => {
            return Object.keys(obj).map(k => {
                // console.log("k:", k, "v:", obj[k])

                if (isNaN(k) && k !== '_t') {
                    let testNumber = k.replace('_', "");
                    if (isNaN(testNumber)) {
                        if (walkJsonPath === '') {
                            walkJsonPath = k
                        } else {
                            walkJsonPath = walkJsonPath + '>' + k
                        }
                    }
                }

                if (_.isPlainObject(obj[k])) {
                    // if (obj[k]["_t"]){
                    //     let newWalkPath = walkJsonPath + ""
                    //     recursionMergePathAdd(obj[k], newWalkPath)
                    // }else{
                    recursionMergePath(obj[k], walkJsonPath)
                    // }
                }

                if (obj[k] instanceof Array) {
                    switch (obj[k].length) {
                        case 1:
                            if (!isNaN(k) && k !== '_t') {
                                if (walkJsonPath === '') {
                                    walkJsonPath = k
                                } else {
                                    walkJsonPath = walkJsonPath + '>' + k
                                }
                            }
                            addLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                        case 2:
                            if (!isNaN(k) && k !== '_t') {
                                if (walkJsonPath === '') {
                                    walkJsonPath = k
                                } else {
                                    walkJsonPath = walkJsonPath + '>' + k
                                }
                            }
                            editLogMap[walkJsonPath] = obj[k][0] + ">" + obj[k][1];
                            splitWalk = walkJsonPath.split('>');
                            // console.log('splitWalk:',splitWalk)
                            splitWalkLength = splitWalk.length - 1;
                            // console.log('splitWalkLength:',splitWalkLength)
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                    // console.log('splitWalkKey:',k,'splitWalkVal:',splitWalk[k],'k !== splitWalkLength:',k !== splitWalkLength)
                                    // console.log(Object.prototype.toString.call(k))
                                    // console.log(Object.prototype.toString.call(splitWalkLength))
                                }
                            });
                            // console.log('splitwalkJsonPath:', walkJsonPath)
                            break;
                        case 3:
                            if (!isNaN(k) && k !== '_t') {
                                if (walkJsonPath === '') {
                                    walkJsonPath = k
                                } else {
                                    walkJsonPath = walkJsonPath + '>' + k
                                }
                            }
                            removeLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                    }
                }


                // const value = obj[k]
                // let title = properties[k] && properties[k].title
                // // console.log(value,title,properties)
                // if (!title && k === 'settings') title = '驱动设置'
                // if (!title || !properties) return
                // if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                //     const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                //     return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                // }
                // if (_.isPlainObject(value) && k !== 'logic') {
                //     let objProperties = properties && properties[k] && properties[k].properties
                //     if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                //         objProperties = properties && properties[k] && properties[k].items.properties
                //     }
                //     return {[title]: convertObjectKey(value, objProperties)}
                // } else {
                //     return {[title]: value}
                // }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // originToAvailable中的递归方法
        const convertObjectKey = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                let title = properties[k] && properties[k].title
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) return
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertObjectKey(value, objProperties)}
                } else {
                    return {[title]: value}
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // 原始数据转换为可读性数据(转中文map)
        const originToAvailable = (value, model, projectID) => {
            let driversSchema = {}

            let data = {}
            try {
                // driversSchema = app.get('drivers')
                if (!global.schemaCache[projectID]) {
                    console.log('global.schemaCache[projectID]未加载完成', projectID)
                    return {}
                }
                driversSchema = global.schemaCache[projectID]["device"]
            } catch (error) {
                console.log('加载device模块失败！')
            }
            // const nodeSchema = app.get('models')['Node']
            // const m = convertObjectKey(value, nodeSchema.properties)
            if (value && _.isPlainObject(value)) {
                Object.keys(value).forEach(objKey => {
                    // 资产
                    if (objKey === 'department') {
                        let u = []
                        if (value.department && value.department.length > 0) {
                            u = value.department.map(item => item.name).join(',')
                        }
                        data['所属部门'] = u
                    } else if (objKey === 'parent') {
                        let p = []
                        if (value.parent && value.parent.length > 0) {
                            p = value.parent.map(item => item.name).join(',')
                        }
                        data['所属资产'] = p
                    } else if (objKey === 'name') {
                        data['名称'] = value.name
                    } else if (objKey === 'remark') {
                        data['备注'] = value.remark
                    } else if (objKey === 'creator') {
                        data['创建人'] = value.creator
                    } else if (objKey === 'focus') {
                        data['关注'] = value.focus ? '是' : '否'
                    } else if (objKey === 'online') {
                        data['在线'] = value.online ? '在线' : '掉线'
                    } else if (objKey === 'off') {
                        data['断电'] = value.off ? '上电' : '断电'
                    } else if (objKey === 'disable') {
                        data['禁用'] = value.disable ? '是' : '否'
                    } else if (objKey === 'uid') {
                        data['编号'] = value.uid
                    } else if (objKey === 'gis') {
                        data['地理信息'] = value.gis
                    } else if (objKey === 'model') {
                        data['模型'] = value.model.name
                    } else if (objKey === 'dashboard') {
                        data['画面'] = value.dashboard.did
                    } else if (objKey === 'video') {
                        const u = {}
                        if (value.video.url && value.video.url.length > 0) {
                            u['视频地址'] = value.video.url.map(item => ({id: item.key, '地址': item.value}))
                        }
                        data['视频'] = u
                    } else if (objKey === 'device' && !_.isEmpty(value.device)) {
                        let d = {}
                        // const driver = model && model.model && model.model.device && model.model.device.driver
                        // if (driver && driversSchema[driver]) {
                        //     const properties = driversSchema[driver].device.properties
                        //     d = convertObjectKey(value.device, properties)
                        // }
                        if (value.device) {
                            d = value.device
                        }
                        data['设备配置'] = d
                    } else if (objKey === 'computed') {
                        let c = {}
                        // const computedSchema = getNodeSchema({ name: 'computed' })

                        const computedSchema = global.schemaCache[projectID]['computed']
                        if (computedSchema) {
                            const properties = computedSchema.properties
                            c = convertObjectKey(value.computed, properties)
                        }
                        data['计算节点'] = c
                    } else if (objKey === 'warning') {
                        let w = {}
                        // const warningSchema = getNodeSchema({ name: 'warning' })
                        const warningSchema = global.schemaCache[projectID]['warning']
                        if (warningSchema) {
                            const properties = warningSchema.properties
                            w = convertObjectKey(value.warning, properties)
                        }
                        data['报警信息'] = w
                    } else if (objKey === 'custom') {
                        let c = {}
                        if (value.custom && value.custom.schema) {
                            c = value.custom.schema
                        }
                        data['属性设置'] = c
                    } else if (objKey === 'hasChild'
                        || objKey === 'modifyTime'
                        || objKey === 'timeoutTime'
                        || objKey === 'connectTime'
                        || objKey === 'createTime'
                        || objKey === 'offlined' // ？？？未知字段
                        || objKey === 'drivers'  // ？？？未知字段
                    ) {
                        //  忽略属性
                    } else {
                        data[objKey] = value[objKey]
                    }
                })
            } else {
                data = value
            }

            return data
        }

        //格式化为一句话
        const formatDiffToWord = (wholeLog, addLogMap, editLogMap, removeLogMap) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + ":";
            data = data + "\n"
            if (addLogMap.length !== 0) {
                Object.keys(addLogMap).forEach(objKey => {
                    if (!addLogMap[objKey]) {
                        data = data + `<span style="color: green">添加</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!addLogMap[objKey].name && !addLogMap[objKey].名称 && _.isPlainObject(addLogMap[objKey]) || addLogMap[objKey] instanceof Array) {
                    //     addLogMap[objKey] = JSON.stringify(addLogMap[objKey])
                    // }
                    if (addLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = ''
                        if (lastWordList.length > 2) {
                            lastWord = lastWordList[lastWordList.length - 2]
                        } else if (lastWordList.length <= 2) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else if (addLogMap[objKey].name) {
                        let lastWordList = objKey.split('>');
                        let lastWord = ''
                        if (lastWordList.length > 2) {
                            lastWord = lastWordList[lastWordList.length - 2]
                        } else if (lastWordList.length <= 2) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        if (lastWord === 'tags') {
                            lastWord = '数据点'
                        }
                        data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].name + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (editLogMap.length !== 0) {
                Object.keys(editLogMap).forEach(objKey => {
                    if (!editLogMap[objKey]) {
                        return
                    }
                    let lastWordList = editLogMap[objKey].split('>');
                    let lastWord = '';
                    let originWord = '';
                    if (lastWordList.length > 1) {
                        originWord = lastWordList[0]
                        lastWord = lastWordList[lastWordList.length - 1]
                    }
                    data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ");"
                    data = data + "\n"
                })
            }
            if (removeLogMap.length !== 0) {
                Object.keys(removeLogMap).forEach(objKey => {
                    if (!removeLogMap[objKey]) {
                        data = data + `<span style="color: red">删除</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!removeLogMap[objKey].名称 && _.isPlainObject(removeLogMap[objKey]) || removeLogMap[objKey] instanceof Array) {
                    //     removeLogMap[objKey] = JSON.stringify(removeLogMap[objKey])
                    // }
                    if (removeLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = '';
                        if (lastWordList.length > 2) {
                            lastWord = lastWordList[lastWordList.length - 2]
                        } else if (lastWordList.length <= 2) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };

        let addLogMap = {};
        let editLogMap = {};
        let removeLogMap = {};

        let walkJsonPath = '';
        let splitWalk = [];
        let splitWalkLength = 0;


        // let wholeLog = {
        //     user: {
        //         "id": "22222",
        //         "name": "张三"
        //     },
        //     time: "2020-07-06T09:57:40+08:00",
        //     model: {"id": "5c934d3b839a5547624eaebf", "name": "采油井#"},
        //     oldData: {
        //         "computed": {
        //             "tags": [
        //                 {
        //                     "id": "YXZT",
        //                     "mapping": {
        //                         "id": "YXZT",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "运行状态",
        //                     "policy": "save",
        //                     "type": "mapping",
        //                 },
        //                 {
        //                     "id": "YW",
        //                     "mapping": {
        //                         "id": "YW",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "油温",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //                 {
        //                     "id": "YW2",
        //                     "mapping": {
        //                         "id": "YW2",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "油温2",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //             ]
        //         },
        //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站"},
        //         "custom": {"record": "测试记录"},
        //         "warning": {
        //             "rules": [
        //                 {
        //                     "alert": true,
        //                     "description": "注气焖井油温>200",
        //                     "disable": true,
        //                     "extraTags": [
        //                         {
        //                             "id": "lat"
        //                         }
        //                     ],
        //                     "handle": false,
        //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
        //                     "interval": 30,
        //                     "level": "中",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "==": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     200
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
        //                     "warningname": "注气报警"
        //                 },
        //                 {
        //                     "alert": true,
        //                     "description": "非注气状态报警",
        //                     "disable": true,
        //                     "handle": true,
        //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "!=": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     180
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "超限报警",
        //                     "warningname": "非注气状态"
        //                 }
        //             ]
        //         }
        //     },
        //     data: {
        //         "computed": {
        //             "tags": [
        //                 {
        //                     "id": "YXZT",
        //                     "mapping": {
        //                         "id": "YXZT",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "运行状态1",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //                 {
        //                     "id": "IA",
        //                     "mapping": {
        //                         "id": "IA",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "A相电流",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 }
        //             ],
        //         },
        //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站1"},
        //         "custom": {"record": "测试记录1", "select-4F10": "测试选择器111"},
        //         "warning": {
        //             "rules": [
        //                 {
        //                     "alert": true,
        //                     "description": "注气焖井油温>200",
        //                     "disable": true,
        //                     "extraTags": [
        //                         {
        //                             "id": "lat"
        //                         }
        //                     ],
        //                     "handle": false,
        //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
        //                     "interval": 30,
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "==": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     200
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
        //                     "warningname": "注气报警"
        //                 },
        //                 {
        //                     "alert": true,
        //                     "description": "非注气状态报警",
        //                     "disable": true,
        //                     "handle": true,
        //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "!=": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     180
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "超限报警",
        //                     "warningname": "非注气状态"
        //                 }
        //             ]
        //         }
        //     }
        // };

        let originStr = wholeLog.oldData
        let newStr = wholeLog.data

        let newChineseStr = originToAvailable(newStr, wholeLog, projectID);

        if (Object.keys(newChineseStr).length === 0) {
            console.log("newChineseStr global缓存pending")
            return
        }

        let originChineseStr = originToAvailable(originStr, wholeLog, projectID);

        console.log("newChineseStr:", JSON.stringify(newChineseStr))

        console.log("originChineseStr:", JSON.stringify(originChineseStr))

        //提取出两个json中不同的部分
        let deltChinese = jsondiffpatch.diff(originChineseStr, newChineseStr);

        if (!deltChinese) {
            let updateMap = {'diff': ''}
            try {
                let tokenProject = await _this.token(projectID)
                _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
            } catch (err) {
                console.error('updateLog err:', err)
            }
            return
        }
        console.log("deltChinese:", JSON.stringify(deltChinese))

        recursionMergePath(deltChinese, walkJsonPath)
        // console.log(recursionMergePath(deltChinese, walkJsonPath));

        // console.log('walkJsonPath Chinese:', walkJsonPath);

        console.log('addLogMap Chinese:', addLogMap);

        console.log('editLogMap Chinese:', editLogMap);

        console.log('removeLogMap Chinese:', removeLogMap);

        console.log('formatDiffToWord:');
        let diffWord = formatDiffToWord(wholeLog, addLogMap, editLogMap, removeLogMap)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };

    async testDashboardSchema(wholeLog, projectID) {
        return new Promise((resolve, reject) => {
            let _this = this
            //生成addLogMap，editLogMap，removeLogMap的key，val
            const recursionMergePath = (obj, walkJsonPath) => {
                return Object.keys(obj).map(k => {
                    // console.log("k:", k, "v:", obj[k])

                    if (isNaN(k) && k !== '_t') {
                        let testNumber = k.replace('_', "");
                        if (isNaN(testNumber)) {
                            if (walkJsonPath === '') {
                                walkJsonPath = k
                            } else {
                                walkJsonPath = walkJsonPath + '>' + k
                            }
                        }
                    }

                    if (_.isPlainObject(obj[k])) {
                        recursionMergePath(obj[k], walkJsonPath)
                    }

                    if (obj[k] instanceof Array) {
                        switch (obj[k].length) {
                            case 1:
                                addLogMap[walkJsonPath] = obj[k][0];
                                splitWalk = walkJsonPath.split('>');
                                splitWalkLength = splitWalk.length - 1;
                                walkJsonPath = '';
                                Object.keys(splitWalk).map(k => {
                                    if (k !== splitWalkLength.toString()) {
                                        if (walkJsonPath === '') {
                                            walkJsonPath = splitWalk[k]
                                        } else {
                                            walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                        }
                                    }
                                });
                                break;
                            case 2:
                                editLogMap[walkJsonPath] = obj[k][0] + ">" + obj[k][1];
                                splitWalk = walkJsonPath.split('>');
                                // console.log('splitWalk:',splitWalk)
                                splitWalkLength = splitWalk.length - 1;
                                // console.log('splitWalkLength:',splitWalkLength)
                                walkJsonPath = '';
                                Object.keys(splitWalk).map(k => {
                                    if (k !== splitWalkLength.toString()) {
                                        if (walkJsonPath === '') {
                                            walkJsonPath = splitWalk[k]
                                        } else {
                                            walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                        }
                                        // console.log('splitWalkKey:',k,'splitWalkVal:',splitWalk[k],'k !== splitWalkLength:',k !== splitWalkLength)
                                        // console.log(Object.prototype.toString.call(k))
                                        // console.log(Object.prototype.toString.call(splitWalkLength))
                                    }
                                });
                                // console.log('splitwalkJsonPath:', walkJsonPath)
                                break;
                            case 3:
                                removeLogMap[walkJsonPath] = obj[k][0];
                                splitWalk = walkJsonPath.split('>');
                                splitWalkLength = splitWalk.length - 1;
                                walkJsonPath = '';
                                Object.keys(splitWalk).map(k => {
                                    if (k !== splitWalkLength.toString()) {
                                        if (walkJsonPath === '') {
                                            walkJsonPath = splitWalk[k]
                                        } else {
                                            walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                        }
                                    }
                                });
                                break;
                        }
                    }


                    // const value = obj[k]
                    // let title = properties[k] && properties[k].title
                    // // console.log(value,title,properties)
                    // if (!title && k === 'settings') title = '驱动设置'
                    // if (!title || !properties) return
                    // if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    //     const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    //     return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                    // }
                    // if (_.isPlainObject(value) && k !== 'logic') {
                    //     let objProperties = properties && properties[k] && properties[k].properties
                    //     if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                    //         objProperties = properties && properties[k] && properties[k].items.properties
                    //     }
                    //     return {[title]: convertObjectKey(value, objProperties)}
                    // } else {
                    //     return {[title]: value}
                    // }
                }).reduce((prev, curr) => ({...prev, ...curr}), {})
            };

            // originToAvailable中的递归方法
            const convertObjectKey = (obj, properties) => {
                return Object.keys(obj).map(k => {
                    const value = obj[k]
                    let title = properties[k] && properties[k].title
                    // console.log(value,title,properties)
                    if (!title && k === 'settings') title = '驱动设置'
                    if (!title || !properties) return
                    if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                        const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                        return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                    }
                    if (_.isPlainObject(value) && k !== 'logic') {
                        let objProperties = properties && properties[k] && properties[k].properties
                        if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                            objProperties = properties && properties[k] && properties[k].items.properties
                        }
                        return {[title]: convertObjectKey(value, objProperties)}
                    } else {
                        return {[title]: value}
                    }
                }).reduce((prev, curr) => ({...prev, ...curr}), {})
            };

            // 原始数据转换为可读性数据(转中文map)
            const originToAvailable = (value, model, projectID) => {
                let driversSchema = {}
                let cellSchema = {}

                let data = {}
                try {
                    // driversSchema = app.get('drivers')
                    if (!global.schemaCache[projectID]) {
                        console.log('global.schemaCache[projectID]未加载完成')
                        return {}
                    }
                    driversSchema = global.schemaCache[projectID]["device"]
                    cellSchema = global.schemaCache[projectID]["cell"]
                } catch (error) {
                    console.log('加载device模块失败！')
                }
                // const nodeSchema = app.get('models')['Node']
                // const m = convertObjectKey(value, nodeSchema.properties)
                if (value && _.isPlainObject(value)) {
                    Object.keys(value).forEach(objKey => {
                        // 资产
                        if (objKey === 'department') {
                            let u = []
                            if (value.department && value.department.length > 0) {
                                u = value.department.map(item => item.name).join(',')
                            }
                            data['所属部门'] = u
                        } else if (objKey === 'parent') {
                            let p = []
                            if (value.parent && value.parent.length > 0) {
                                p = value.parent.map(item => item.name).join(',')
                            }
                            data['所属资产'] = p
                        } else if (objKey === 'data') {
                            // continue
                        } else if (objKey === 'name') {
                            data['名称'] = value.name
                        } else if (objKey === 'remark') {
                            data['备注'] = value.remark
                        } else if (objKey === 'creator') {
                            data['创建人'] = value.creator
                        } else if (objKey === 'focus') {
                            data['关注'] = value.focus ? '是' : '否'
                        } else if (objKey === 'isDelete') {
                            data['已回收'] = value.isDelete ? '是' : '否'
                        } else if (objKey === 'online') {
                            data['在线'] = value.online ? '在线' : '掉线'
                        } else if (objKey === 'off') {
                            data['断电'] = value.off ? '上电' : '断电'
                        } else if (objKey === 'disable') {
                            data['禁用'] = value.disable ? '是' : '否'
                        } else if (objKey === 'uid') {
                            data['编号'] = value.uid
                        } else if (objKey === 'gis') {
                            data['地理信息'] = value.gis
                        } else if (objKey === 'model') {
                            if (value.model.name) {
                                data['模型'] = value.model.name
                            }
                        } else if (objKey === 'node') {
                            if (value.node.name) {
                                data['资产'] = value.node.name
                            }
                        } else if (objKey === 'catalog') {
                            if (value.catalog.name) {
                                data['目录'] = value.catalog.name
                            }
                        } else if (objKey === 'dashboard') {
                            data['画面'] = value.dashboard.did
                        } else if (objKey === 'video') {
                            const u = {}
                            if (value.video.url && value.video.url.length > 0) {
                                u['视频地址'] = value.video.url.map(item => ({id: item.key, '地址': item.value}))
                            }
                            data['视频'] = u
                        } else if (objKey === 'device' && !_.isEmpty(value.device)) {
                            let d = {}
                            const driver = model && model.model && model.model.device && model.model.device.driver
                            if (driver && driversSchema[driver]) {
                                const properties = driversSchema[driver].device.properties
                                d = convertObjectKey(value.device, properties)
                            }
                            data['设备配置'] = d
                        } else if (objKey === 'computed') {
                            let c = {}
                            // const computedSchema = getNodeSchema({ name: 'computed' })

                            const computedSchema = global.schemaCache[projectID]['computed']
                            if (computedSchema) {
                                const properties = computedSchema.properties
                                c = convertObjectKey(value.computed, properties)
                            }
                            data['计算节点'] = c
                        } else if (objKey === 'warning') {
                            let w = {}
                            // const warningSchema = getNodeSchema({ name: 'warning' })
                            const warningSchema = global.schemaCache[projectID]['warning']
                            if (warningSchema) {
                                const properties = warningSchema.properties
                                w = convertObjectKey(value.warning, properties)
                            }
                            data['报警信息'] = w
                        } else if (objKey === 'custom') {
                            let c = {}
                            if (value.custom && value.custom.schema) {
                                c = value.custom.schema
                            }
                            data['属性设置'] = c
                        } else if (objKey === 'hasChild'
                            || objKey === 'modifyTime'
                            || objKey === 'timeoutTime'
                            || objKey === 'connectTime'
                            || objKey === 'createTime'
                            || objKey === 'offlined' // ？？？未知字段
                            || objKey === 'drivers'  // ？？？未知字段
                            || objKey === "cells"
                        ) {
                            //  忽略属性
                        } else {
                            data[objKey] = value[objKey]
                        }
                    })
                } else {
                    data = value
                }

                return data
            }

            //格式化为一句话
            const formatDiffToWord = (wholeLog, addLogMap, editLogMap, removeLogMap) => {
                let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
                formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
                formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
                let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + ":";
                data = data + "\n"
                if (addLogMap.length !== 0) {
                    Object.keys(addLogMap).forEach(objKey => {
                        if (!addLogMap[objKey]) {
                            data = data + `<span style="color: green">添加</span>:` + "值null" + "(位置:" + objKey + ");";
                            data = data + "\n";
                            return
                        }
                        // if (!addLogMap[objKey].名称 && _.isPlainObject(addLogMap[objKey]) || addLogMap[objKey] instanceof Array) {
                        //     addLogMap[objKey] = JSON.stringify(addLogMap[objKey])
                        // }
                        if (addLogMap[objKey].名称) {
                            let lastWordList = objKey.split('>');
                            let lastWord = ''
                            if (lastWordList.length !== 0) {
                                lastWord = lastWordList[lastWordList.length - 1]
                            }
                            data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                            data = data + "\n"
                        } else {
                            data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ");"
                            data = data + "\n"
                        }
                    })
                }
                if (editLogMap.length !== 0) {
                    Object.keys(editLogMap).forEach(objKey => {
                        if (!editLogMap[objKey]) {
                            return
                        }
                        let lastWordList = editLogMap[objKey].split('>');
                        let lastWord = '';
                        let originWord = '';
                        if (lastWordList.length > 1) {
                            originWord = lastWordList[0]
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    })
                }
                if (removeLogMap.length !== 0) {
                    Object.keys(removeLogMap).forEach(objKey => {
                        if (!removeLogMap[objKey]) {
                            data = data + `<span style="color: red">删除</span>:` + "值null" + "(位置:" + objKey + ");";
                            data = data + "\n";
                            return
                        }
                        // if (!removeLogMap[objKey].名称 && _.isPlainObject(removeLogMap[objKey]) || removeLogMap[objKey] instanceof Array) {
                        //     removeLogMap[objKey] = JSON.stringify(removeLogMap[objKey])
                        // }
                        if (removeLogMap[objKey].名称) {
                            let lastWordList = objKey.split('>');
                            let lastWord = '';
                            if (lastWordList.length !== 0) {
                                lastWord = lastWordList[lastWordList.length - 1]
                            }
                            data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                            data = data + "\n"
                        } else {
                            data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ");"
                            data = data + "\n"
                        }
                    })
                }
                if (data.endsWith('\n')) {
                    data = data.substring(0, data.length - 2);
                }
                return data
            };

            let addLogMap = {};
            let editLogMap = {};
            let removeLogMap = {};

            let walkJsonPath = '';
            let splitWalk = [];
            let splitWalkLength = 0;


            // let wholeLog = {
            //     user: {
            //         "id": "22222",
            //         "name": "张三"
            //     },
            //     time: "2020-07-06T09:57:40+08:00",
            //     model: {"id": "5c934d3b839a5547624eaebf", "name": "采油井#"},
            //     oldData: {
            //         "computed": {
            //             "tags": [
            //                 {
            //                     "id": "YXZT",
            //                     "mapping": {
            //                         "id": "YXZT",
            //                         "model": {
            //                             "id": "5d0215ef978032d7c9acaf62",
            //                             "name": "采油井4G"
            //                         },
            //                         "name": "运行状态",
            //                         "type": ""
            //                     },
            //                     "name": "运行状态",
            //                     "policy": "save",
            //                     "type": "mapping",
            //                 },
            //                 {
            //                     "id": "YW",
            //                     "mapping": {
            //                         "id": "YW",
            //                         "model": {
            //                             "id": "5d0215ef978032d7c9acaf62",
            //                             "name": "采油井4G"
            //                         },
            //                         "name": "运行状态",
            //                         "type": ""
            //                     },
            //                     "name": "油温",
            //                     "policy": "save",
            //                     "unit": "12656",
            //                 },
            //                 {
            //                     "id": "YW2",
            //                     "mapping": {
            //                         "id": "YW2",
            //                         "model": {
            //                             "id": "5d0215ef978032d7c9acaf62",
            //                             "name": "采油井4G"
            //                         },
            //                         "name": "运行状态",
            //                         "type": ""
            //                     },
            //                     "name": "油温2",
            //                     "policy": "save",
            //                     "unit": "12656",
            //                 },
            //             ]
            //         },
            //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站"},
            //         "custom": {"record": "测试记录"},
            //         "warning": {
            //             "rules": [
            //                 {
            //                     "alert": true,
            //                     "description": "注气焖井油温>200",
            //                     "disable": true,
            //                     "extraTags": [
            //                         {
            //                             "id": "lat"
            //                         }
            //                     ],
            //                     "handle": false,
            //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
            //                     "interval": 30,
            //                     "level": "中",
            //                     "logic": {
            //                         "and": [
            //                             {
            //                                 "==": [
            //                                     {
            //                                         "var": [
            //                                             "node.custom.select-CF3B"
            //                                         ]
            //                                     },
            //                                     "2"
            //                                 ]
            //                             },
            //                             {
            //                                 ">": [
            //                                     {
            //                                         "var": [
            //                                             "var.YW"
            //                                         ]
            //                                     },
            //                                     200
            //                                 ]
            //                             }
            //                         ]
            //                     },
            //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
            //                     "warningname": "注气报警"
            //                 },
            //                 {
            //                     "alert": true,
            //                     "description": "非注气状态报警",
            //                     "disable": true,
            //                     "handle": true,
            //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
            //                     "level": "高",
            //                     "logic": {
            //                         "and": [
            //                             {
            //                                 "!=": [
            //                                     {
            //                                         "var": [
            //                                             "node.custom.select-CF3B"
            //                                         ]
            //                                     },
            //                                     "2"
            //                                 ]
            //                             },
            //                             {
            //                                 ">": [
            //                                     {
            //                                         "var": [
            //                                             "var.YW"
            //                                         ]
            //                                     },
            //                                     180
            //                                 ]
            //                             }
            //                         ]
            //                     },
            //                     "type": "超限报警",
            //                     "warningname": "非注气状态"
            //                 }
            //             ]
            //         }
            //     },
            //     data: {
            //         "computed": {
            //             "tags": [
            //                 {
            //                     "id": "YXZT",
            //                     "mapping": {
            //                         "id": "YXZT",
            //                         "model": {
            //                             "id": "5d0215ef978032d7c9acaf62",
            //                             "name": "采油井4G"
            //                         },
            //                         "name": "运行状态",
            //                         "type": ""
            //                     },
            //                     "name": "运行状态1",
            //                     "policy": "save",
            //                     "unit": "12656",
            //                 },
            //                 {
            //                     "id": "IA",
            //                     "mapping": {
            //                         "id": "IA",
            //                         "model": {
            //                             "id": "5d0215ef978032d7c9acaf62",
            //                             "name": "采油井4G"
            //                         },
            //                         "name": "运行状态",
            //                         "type": ""
            //                     },
            //                     "name": "A相电流",
            //                     "policy": "save",
            //                     "unit": "12656",
            //                 }
            //             ],
            //         },
            //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站1"},
            //         "custom": {"record": "测试记录1", "select-4F10": "测试选择器111"},
            //         "warning": {
            //             "rules": [
            //                 {
            //                     "alert": true,
            //                     "description": "注气焖井油温>200",
            //                     "disable": true,
            //                     "extraTags": [
            //                         {
            //                             "id": "lat"
            //                         }
            //                     ],
            //                     "handle": false,
            //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
            //                     "interval": 30,
            //                     "level": "高",
            //                     "logic": {
            //                         "and": [
            //                             {
            //                                 "==": [
            //                                     {
            //                                         "var": [
            //                                             "node.custom.select-CF3B"
            //                                         ]
            //                                     },
            //                                     "2"
            //                                 ]
            //                             },
            //                             {
            //                                 ">": [
            //                                     {
            //                                         "var": [
            //                                             "var.YW"
            //                                         ]
            //                                     },
            //                                     200
            //                                 ]
            //                             }
            //                         ]
            //                     },
            //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
            //                     "warningname": "注气报警"
            //                 },
            //                 {
            //                     "alert": true,
            //                     "description": "非注气状态报警",
            //                     "disable": true,
            //                     "handle": true,
            //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
            //                     "level": "高",
            //                     "logic": {
            //                         "and": [
            //                             {
            //                                 "!=": [
            //                                     {
            //                                         "var": [
            //                                             "node.custom.select-CF3B"
            //                                         ]
            //                                     },
            //                                     "2"
            //                                 ]
            //                             },
            //                             {
            //                                 ">": [
            //                                     {
            //                                         "var": [
            //                                             "var.YW"
            //                                         ]
            //                                     },
            //                                     180
            //                                 ]
            //                             }
            //                         ]
            //                     },
            //                     "type": "超限报警",
            //                     "warningname": "非注气状态"
            //                 }
            //             ]
            //         }
            //     }
            // };

            delete (wholeLog.oldData['statusList'])
            delete (wholeLog.data['statusList'])
            let originStr = wholeLog.oldData
            let newStr = wholeLog.data

            let newChineseStr = originToAvailable(newStr, wholeLog, projectID);

            if (Object.keys(newChineseStr).length === 0) {
                // console.log("newChineseStr global缓存pending testDashboardSchema")
                // return
            }

            let originChineseStr = originToAvailable(originStr, wholeLog, projectID);

            console.log("newChineseStr:", JSON.stringify(newChineseStr))

            console.log("originChineseStr:", JSON.stringify(originChineseStr))

            //提取出两个json中不同的部分
            let deltChinese = jsondiffpatch.diff(originChineseStr, newChineseStr);

            if (!deltChinese) {
                // let updateMap = {'diff': ''}
                // _this.updateLog(_this.tokenStr, _this.config, updateMap, wholeLog.id)
                return resolve(null)
            }
            console.log("deltChinese:", JSON.stringify(deltChinese))

            recursionMergePath(deltChinese, walkJsonPath)
            // console.log(recursionMergePath(deltChinese, walkJsonPath));

            // console.log('walkJsonPath Chinese:', walkJsonPath);

            console.log('addLogMap Chinese:', addLogMap);

            console.log('editLogMap Chinese:', editLogMap);

            console.log('removeLogMap Chinese:', removeLogMap);

            console.log('formatDiffToWord:');
            let diffWord = formatDiffToWord(wholeLog, addLogMap, editLogMap, removeLogMap)
            console.log(diffWord);
            return resolve(diffWord);
        });
        // let updateMap = {'diff': diffWord}
        // _this.updateLog(_this.tokenStr, _this.config, updateMap, wholeLog.id)
    };

    async testDashboardCellsSchema(wholeLog, diffWordInDashboard, projectID) {
        let _this = this
        let componentKeyNameMap = {};
        // let hasArray = false;
        // let hasArrayKey = '';
        //生成addLogMap，editLogMap，removeLogMap的key，val
        const recursionMergePath = (obj, walkJsonPath) => {
            return Object.keys(obj).map(k => {
                // console.log("k:", k, "v:", obj[k])

                if (isNaN(k) && k !== '_t') {
                    let testNumber = k.replace('_', "");
                    if (isNaN(testNumber)) {
                        if (walkJsonPath === '') {
                            walkJsonPath = k
                        } else {
                            walkJsonPath = walkJsonPath + '>' + k
                        }
                        console.log('all walkJsonPath:', walkJsonPath)
                    }
                }
                // if (hasArray === true){
                //     if (walkJsonPath.includes(hasArrayKey)){
                //         return
                //     }
                // }

                if (_.isPlainObject(obj[k])) {
                    recursionMergePath(obj[k], walkJsonPath)
                }

                if (obj[k] instanceof Array) {
                    switch (obj[k].length) {
                        case 1:
                            console.log('addLogMap walkJsonPath:', walkJsonPath)
                            addLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (parseInt(k) < splitWalkLength) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            console.log('addLogMap walkJsonPath after:', walkJsonPath)
                            break;
                        case 2:
                            console.log('editLogMap walkJsonPath:', walkJsonPath)
                            editLogMap[walkJsonPath] = obj[k][0] + ">" + obj[k][1];
                            splitWalk = walkJsonPath.split('>');
                            // console.log('splitWalk:',splitWalk)
                            splitWalkLength = splitWalk.length - 1;
                            // console.log('splitWalkLength:',splitWalkLength)
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (parseInt(k) < splitWalkLength) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                    // console.log('splitWalkKey:',k,'splitWalkVal:',splitWalk[k],'k !== splitWalkLength:',k !== splitWalkLength)
                                    // console.log(Object.prototype.toString.call(k))
                                    // console.log(Object.prototype.toString.call(splitWalkLength))
                                }
                            });
                            console.log('editLogMap walkJsonPath after:', walkJsonPath)
                            // console.log('splitwalkJsonPath:', walkJsonPath)
                            break;
                        case 3:
                            // hasArray = true
                            // hasArrayKey = k;
                            console.log('removeLogMap walkJsonPath:', walkJsonPath)
                            removeLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (parseInt(k) < splitWalkLength) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            console.log('removeLogMap walkJsonPath after:', walkJsonPath)
                            break;
                    }
                }


                // const value = obj[k]
                // let title = properties[k] && properties[k].title
                // // console.log(value,title,properties)
                // if (!title && k === 'settings') title = '驱动设置'
                // if (!title || !properties) return
                // if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                //     const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                //     return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                // }
                // if (_.isPlainObject(value) && k !== 'logic') {
                //     let objProperties = properties && properties[k] && properties[k].properties
                //     if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                //         objProperties = properties && properties[k] && properties[k].items.properties
                //     }
                //     return {[title]: convertObjectKey(value, objProperties)}
                // } else {
                //     return {[title]: value}
                // }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };


        // originToAvailable中的递归方法
        const convertObjectKey = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                if (!properties || !properties[k] || !properties[k].title) {
                    return
                }
                let title = properties[k] && properties[k].title
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) return
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertObjectKey(value, objProperties)}
                } else {
                    return {[title]: value}
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        const convertCellsObjectKey = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                if (!properties || !properties[k]) {
                    if (k === 'name') {
                        return {['名称']: value}
                    } else {
                        return
                    }
                }
                let title = properties[k] && properties[k].title
                //console.log('properties[k]:', properties[k], 'title:', title, 'k:', k, 'obj[k]:', obj[k])
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) return
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    //console.log('arrProperties:', arrProperties)
                    return {[title]: value.map(item => convertCellsObjectKey(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    //console.log('objProperties:', objProperties)
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertCellsObjectKey(value, objProperties)}
                } else {
                    return {[title]: value}
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // 原始数据转换为可读性数据(转中文map)
        const originToAvailable = (value, model, projectID) => {
            let driversSchema = {};
            let cellSchema = {};

            let data = {}
            try {
                // driversSchema = app.get('drivers')
                if (!global.schemaCache[projectID]) {
                    console.log('global.schemaCache[projectID]未加载完成')
                    return {}
                }
                driversSchema = global.schemaCache[projectID]["device"]
                cellSchema = global.schemaCache[projectID]["cell"]
            } catch (error) {
                console.log('加载device模块失败！')
            }
            // const nodeSchema = app.get('models')['Node']
            // const m = convertObjectKey(value, nodeSchema.properties)
            if (value && _.isPlainObject(value)) {
                Object.keys(value).forEach(objKey => {
                    // 资产
                    if (objKey === 'department') {
                        let u = [];
                        if (value.department && value.department.length > 0) {
                            u = value.department.map(item => item.name).join(',')
                        }
                        data['所属部门'] = u
                    } else if (objKey === 'parent') {
                        let p = []
                        if (value.parent && value.parent.length > 0) {
                            p = value.parent.map(item => item.name).join(',')
                        }
                        data['所属资产'] = p
                    } else if (objKey === 'name') {
                        data['名称'] = value.name
                    } else if (objKey === 'remark') {
                        data['备注'] = value.remark
                    } else if (objKey === 'creator') {
                        data['创建人'] = value.creator
                    } else if (objKey === 'focus') {
                        data['关注'] = value.focus ? '是' : '否'
                    } else if (objKey === 'online') {
                        data['在线'] = value.online ? '在线' : '掉线'
                    } else if (objKey === 'off') {
                        data['断电'] = value.off ? '上电' : '断电'
                    } else if (objKey === 'disable') {
                        data['禁用'] = value.disable ? '是' : '否'
                    } else if (objKey === 'uid') {
                        data['编号'] = value.uid
                    } else if (objKey === 'gis') {
                        data['地理信息'] = value.gis
                    } else if (objKey === 'model') {
                        data['模型'] = value.model.name
                    } else if (objKey === 'dashboard') {
                        data['画面'] = value.dashboard.did
                    } else if (objKey === 'video') {
                        const u = {}
                        if (value.video.url && value.video.url.length > 0) {
                            u['视频地址'] = value.video.url.map(item => ({id: item.key, '地址': item.value}))
                        }
                        data['视频'] = u
                    } else if (objKey === 'device' && !_.isEmpty(value.device)) {
                        let d = {}
                        const driver = model && model.model && model.model.device && model.model.device.driver
                        if (driver && driversSchema[driver]) {
                            const properties = driversSchema[driver].device.properties
                            d = convertObjectKey(value.device, properties)
                        }
                        data['设备配置'] = d
                    } else if (objKey === 'computed') {
                        let c = {}
                        // const computedSchema = getNodeSchema({ name: 'computed' })

                        const computedSchema = global.schemaCache[projectID]['computed']
                        if (computedSchema) {
                            const properties = computedSchema.properties
                            c = convertObjectKey(value.computed, properties)
                        }
                        data['计算节点'] = c
                    } else if (objKey === 'warning') {
                        let w = {}
                        // const warningSchema = getNodeSchema({ name: 'warning' })
                        const warningSchema = global.schemaCache[projectID]['warning']
                        if (warningSchema) {
                            const properties = warningSchema.properties
                            w = convertObjectKey(value.warning, properties)
                        }
                        data['报警信息'] = w
                    } else if (objKey === 'custom') {
                        let c = {}
                        if (value.custom && value.custom.schema) {
                            c = value.custom.schema
                        }
                        data['属性设置'] = c
                    } else if (objKey === 'cells' && !_.isEmpty(value.cells)) {
                        let d = {};
                        let smallD = {};
                        let componentNameList = [];
                        let componentName = '';
                        let dName = '';

                        Object.keys(value[objKey]).forEach(objCellKey => {
                            if (objCellKey !== 'root') {
                                smallD = {};
                                componentName = '';
                                componentNameList = objCellKey.split('_');
                                componentNameList.forEach((obj, index) => {
                                    if (index !== componentNameList.length - 1) {
                                        if (componentName !== '') {
                                            if (componentName === 'data') {
                                                componentName += '_' + obj
                                            } else {
                                                componentName += '.' + obj
                                            }
                                        } else {
                                            componentName = obj
                                        }
                                    }
                                });
                                //componentName = componentNameList[0];
                                if (componentName && cellSchema[componentName]) {
                                    const properties = cellSchema[componentName]
                                    // console.log('value[objKey][objCellKey]:', value[objKey][objCellKey])
                                    // console.log('properties:', properties)
                                    if (properties.paramSchema && _.isFunction(properties.paramSchema)) {
                                        properties.paramSchema = properties.paramSchema()
                                    }

                                    dName = componentNameList.slice(-1).toString();

                                    if (properties.paramSchema && properties.paramSchema.properties && value[objKey][objCellKey].params) {
                                        smallD = convertCellsObjectKey(value[objKey][objCellKey].params, properties.paramSchema.properties)
                                    }
                                    if (value[objKey][objCellKey].params && value[objKey][objCellKey].params.name) {
                                        dName = value[objKey][objCellKey].params.name
                                        componentKeyNameMap[properties.title + "_" + componentNameList.slice(-1).toString()] = dName
                                    }
                                    d[properties.title + "_" + componentNameList.slice(-1).toString()] = smallD
                                } else {
                                    console.log('未追踪该画面组件', componentName)
                                }
                            }
                        });
                        if (Object.keys(d).length !== 0) {
                            data['画面组件'] = d
                        } else {
                            console.log('此次修改中没有被追踪的画面组件')
                        }
                    } else if (objKey === 'hasChild'
                        || objKey === 'modifyTime'
                        || objKey === 'timeoutTime'
                        || objKey === 'connectTime'
                        || objKey === 'createTime'
                        || objKey === 'offlined' // ？？？未知字段
                        || objKey === 'drivers'  // ？？？未知字段
                    ) {
                        //  忽略属性
                    } else {
                        data[objKey] = value[objKey]
                    }
                })
            } else {
                data = value
            }

            return data
        }

        //格式化为一句话
        const formatDiffToWord = (wholeLog, addLogMap, editLogMap, removeLogMap) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = ''
            if (diffWordInDashboard) {
                data = diffWordInDashboard + ";"
            } else {
                data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + ":";
            }
            data = data + "\n"
            if (addLogMap.length !== 0) {
                Object.keys(addLogMap).forEach(objKey => {
                    if (!addLogMap[objKey]) {
                        data = data + `<span style="color: green">添加</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!addLogMap[objKey].名称 && _.isPlainObject(addLogMap[objKey]) || addLogMap[objKey] instanceof Array) {
                    //     addLogMap[objKey] = JSON.stringify(addLogMap[objKey])
                    // }
                    let lastWordList = objKey.split('>');
                    let lastWord = '';
                    if (lastWordList.length !== 0) {
                        lastWord = lastWordList[lastWordList.length - 1]
                    }
                    if (lastWordList.length > 1) {
                        if ((componentKeyNameMap[lastWordList[lastWordList.length - 1]]) || (lastWordList[lastWordList.length - 1].includes('_'))) {
                            let comRandomList = lastWordList[lastWordList.length - 1].split('_');
                            if (comRandomList.length !== 0) {
                                lastWord = comRandomList[0]
                            }
                        }
                        if (addLogMap[objKey].名称) {
                            data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");";
                            data = data + "\n"
                        } else {
                            if (componentKeyNameMap[lastWordList[lastWordList.length - 1]] && componentKeyNameMap[lastWordList[lastWordList.length - 1]] !== '') {
                                data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ")(种类:" + lastWord + ",名称:" + componentKeyNameMap[lastWordList[lastWordList.length - 1]] + ");";
                            } else {
                                let objKeyList = objKey.split('>');
                                if (objKeyList[objKeyList.length - 1].includes('_')) {
                                    data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ")(种类:" + lastWord + ");";
                                } else {
                                    data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ");";
                                }
                            }
                            data = data + "\n"
                        }
                    } else if (lastWordList.length === 1) {
                        Object.keys(addLogMap[objKey]).forEach(comKey => {
                            if ((componentKeyNameMap[comKey]) || (comKey.includes('_'))) {
                                let comRandomList = comKey.split('_');
                                if (comRandomList.length !== 0) {
                                    lastWord = comRandomList[0]
                                    if (addLogMap[objKey][comKey] && addLogMap[objKey][comKey].名称) {
                                        data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey][comKey].名称 + "的" + lastWord + "(位置:" + objKey + ");";
                                        data = data + "\n"
                                    } else {
                                        if (componentKeyNameMap[comKey] && componentKeyNameMap[comKey] !== '') {
                                            data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey][comKey]) + "(位置:" + objKey + ">" + comKey + ")(种类:" + lastWord + ",名称:" + componentKeyNameMap[comKey] + ");";
                                        } else {
                                            data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey][comKey]) + "(位置:" + objKey + ">" + comKey + ")(种类:" + lastWord + ");";
                                        }
                                        data = data + "\n"
                                    }
                                }
                            }
                        })
                    }
                })
            }
            if (editLogMap.length !== 0) {
                Object.keys(editLogMap).forEach(objKey => {
                    if (!editLogMap[objKey]) {
                        return
                    }
                    let lastWordList = editLogMap[objKey].split('>');
                    let lastWord = '';
                    let originWord = '';
                    let componentCategory = '';
                    if (lastWordList.length > 1) {
                        originWord = lastWordList[0];
                        lastWord = lastWordList[lastWordList.length - 1];
                    }

                    let lastWordKeyList = objKey.split('>');
                    if (lastWordKeyList.length > 1) {
                        if (componentKeyNameMap[lastWordKeyList[lastWordKeyList.length - 1]] || (lastWordKeyList[lastWordKeyList.length - 1].includes('_'))) {
                            let comRandomList = lastWordKeyList[lastWordKeyList.length - 1].split('_');
                            if (comRandomList.length !== 0) {
                                componentCategory = comRandomList[0]
                            }
                        }
                        if (componentKeyNameMap[lastWordKeyList[lastWordKeyList.length - 1]] && componentKeyNameMap[lastWordKeyList[lastWordKeyList.length - 1]] !== '') {
                            data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ")(种类:" + componentCategory + ",名称:" + componentKeyNameMap[lastWordKeyList[lastWordKeyList.length - 1]] + ");"
                        } else {
                            let objKeyList = objKey.split('>');
                            if (objKeyList[objKeyList.length - 1].includes('_')) {
                                data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ")(种类:" + componentCategory + ");";
                            } else {
                                data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ");";
                            }
                        }
                        data = data + "\n"
                    } else if (lastWordKeyList.length === 1) {
                        Object.keys(editLogMap[objKey]).forEach(comKey => {
                            if ((componentKeyNameMap[comKey]) || (comKey.includes('_'))) {
                                let comRandomList = comKey.split('_');
                                if (comRandomList.length !== 0) {
                                    lastWord = comRandomList[0]
                                    if (componentKeyNameMap[comKey] && componentKeyNameMap[comKey] !== '') {
                                        data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ">" + comKey + ")(种类:" + componentCategory + ",名称:" + componentKeyNameMap[comKey] + ");"
                                    } else {
                                        data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ">" + comKey + ")(种类:" + componentCategory + ");";
                                    }
                                    data = data + "\n"
                                }
                            }
                        })
                    }
                })
            }
            if (removeLogMap.length !== 0) {
                Object.keys(removeLogMap).forEach(objKey => {
                    if (!removeLogMap[objKey]) {
                        data = data + `<span style="color: red">删除</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!removeLogMap[objKey].名称 && _.isPlainObject(removeLogMap[objKey]) || removeLogMap[objKey] instanceof Array) {
                    //     removeLogMap[objKey] = JSON.parse(JSON.stringify(removeLogMap[objKey]))
                    // }

                    let lastWordList = objKey.split('>');
                    let lastWord = '';
                    if (lastWordList.length !== 0) {
                        lastWord = lastWordList[lastWordList.length - 1]
                    }
                    if (lastWordList.length > 1) {
                        if (componentKeyNameMap[lastWordList[lastWordList.length - 1]] || (lastWordList[lastWordList.length - 1].includes('_'))) {
                            let comRandomList = lastWordList[lastWordList.length - 1].split('_');
                            if (comRandomList.length !== 0) {
                                lastWord = comRandomList[0]
                            }
                        }
                        if (removeLogMap[objKey].名称) {
                            data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                            data = data + "\n"
                        } else {
                            if (componentKeyNameMap[lastWordList[lastWordList.length - 1]] && componentKeyNameMap[lastWordList[lastWordList.length - 1]] !== '') {
                                data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ")(种类:" + lastWord + ",名称:" + componentKeyNameMap[lastWordList[lastWordList.length - 1]] + ");";
                            } else {
                                let objKeyList = objKey.split('>');
                                if (objKeyList[objKeyList.length - 1].includes('_')) {
                                    data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ")(种类:" + lastWord + ");";
                                } else {
                                    data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ");";
                                }
                            }
                            data = data + "\n"
                        }
                    } else if (lastWordList.length === 1) {
                        Object.keys(removeLogMap[objKey]).forEach(comKey => {
                            if ((componentKeyNameMap[comKey]) || (comKey.includes('_'))) {
                                let comRandomList = comKey.split('_');
                                if (comRandomList.length !== 0) {
                                    lastWord = comRandomList[0]
                                    if (removeLogMap[objKey][comKey] && removeLogMap[objKey][comKey].名称) {
                                        data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey][comKey].名称 + "的" + lastWord + "(位置:" + objKey + ");";
                                        data = data + "\n"
                                    } else {
                                        if (componentKeyNameMap[comKey] && componentKeyNameMap[comKey] !== '') {
                                            data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey][comKey]) + "(位置:" + objKey + ">" + comKey + ")(种类:" + lastWord + ",名称:" + componentKeyNameMap[comKey] + ");";
                                        } else {
                                            data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey][comKey]) + "(位置:" + objKey + ">" + comKey + ")(种类:" + lastWord + ");";
                                        }
                                        data = data + "\n"
                                    }
                                }
                            }
                        })
                    }

                })
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };

        let addLogMap = {};
        let editLogMap = {};
        let removeLogMap = {};

        let walkJsonPath = '';
        let splitWalk = [];
        let splitWalkLength = 0;


        let testOriginData = {}
        let testNewData = {}

        if (wholeLog.data && wholeLog.data.data) {
            if (!_.isPlainObject(wholeLog.data.data)) {
                testNewData = JSON.parse(wholeLog.data.data)
            } else {
                testNewData = wholeLog.data.data
            }
        }

        if (wholeLog.oldData && wholeLog.oldData.data) {
            if (!_.isPlainObject(wholeLog.data.data)) {
                testOriginData = JSON.parse(wholeLog.oldData.data)
            } else {
                testOriginData = wholeLog.oldData.data
            }
        }
        wholeLog.data = testNewData
        wholeLog.oldData = testOriginData

        let originStr = wholeLog.oldData
        let newStr = wholeLog.data

        let newChineseStr = originToAvailable(newStr, wholeLog, projectID);

        if (Object.keys(newChineseStr).length === 0) {
            if (diffWordInDashboard) {
                let updateMap = {'diff': ''}
                updateMap = {'diff': diffWordInDashboard + ';'};
                try {
                    let tokenProject = await _this.token(projectID)
                    _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
                } catch (err) {
                    console.error('updateLog err:', err)
                }
                console.log('updateLog newChineseStr cell', Object.keys(newChineseStr).length, newChineseStr)
                return
            } else {
                console.log("newChineseStr global缓存pending testDashboardCellsSchema")
                return
            }
        }

        let originChineseStr = originToAvailable(originStr, wholeLog, projectID);

        console.log("newChineseStr cell:", JSON.stringify(newChineseStr))

        console.log("originChineseStr cell:", JSON.stringify(originChineseStr))

        //提取出两个json中不同的部分
        let deltChinese = jsondiffpatch.diff(originChineseStr, newChineseStr);

        if (!deltChinese) {
            let updateMap = {'diff': ''}
            if (diffWordInDashboard) {
                updateMap = {'diff': diffWordInDashboard + ';'};
            }
            try {
                let tokenProject = await _this.token(projectID)
                _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
            } catch (err) {
                console.error('updateLog err:', err)
            }
            return
        }
        console.log("deltChinese cell:", JSON.stringify(deltChinese))

        recursionMergePath(deltChinese, walkJsonPath)
        // console.log(recursionMergePath(deltChinese, walkJsonPath));

        // console.log('walkJsonPath Chinese:', walkJsonPath);

        console.log('addLogMap Chinese cell:', addLogMap);

        console.log('editLogMap Chinese cell:', editLogMap);

        console.log('removeLogMap Chinese cell:', removeLogMap);

        console.log('formatDiffToWord cell:');
        let diffWord = formatDiffToWord(wholeLog, addLogMap, editLogMap, removeLogMap)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };

    async testExtSchema(wholeLog, projectID) {
        let _this = this
        //生成addLogMap，editLogMap，removeLogMap的key，val
        const recursionMergePath = (obj, walkJsonPath) => {
            return Object.keys(obj).map(k => {
                // console.log("k:", k, "v:", obj[k])

                if (isNaN(k) && k !== '_t') {
                    let testNumber = k.replace('_', "");
                    if (isNaN(testNumber)) {
                        if (walkJsonPath === '') {
                            walkJsonPath = k
                        } else {
                            walkJsonPath = walkJsonPath + '>' + k
                        }
                    }
                }

                if (_.isPlainObject(obj[k])) {
                    recursionMergePath(obj[k], walkJsonPath)
                }

                if (obj[k] instanceof Array) {
                    switch (obj[k].length) {
                        case 1:
                            addLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                        case 2:
                            editLogMap[walkJsonPath] = obj[k][0] + ">" + obj[k][1];
                            splitWalk = walkJsonPath.split('>');
                            // console.log('splitWalk:',splitWalk)
                            splitWalkLength = splitWalk.length - 1;
                            // console.log('splitWalkLength:',splitWalkLength)
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                    // console.log('splitWalkKey:',k,'splitWalkVal:',splitWalk[k],'k !== splitWalkLength:',k !== splitWalkLength)
                                    // console.log(Object.prototype.toString.call(k))
                                    // console.log(Object.prototype.toString.call(splitWalkLength))
                                }
                            });
                            // console.log('splitwalkJsonPath:', walkJsonPath)
                            break;
                        case 3:
                            removeLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                    }
                }


                // const value = obj[k]
                // let title = properties[k] && properties[k].title
                // // console.log(value,title,properties)
                // if (!title && k === 'settings') title = '驱动设置'
                // if (!title || !properties) return
                // if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                //     const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                //     return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                // }
                // if (_.isPlainObject(value) && k !== 'logic') {
                //     let objProperties = properties && properties[k] && properties[k].properties
                //     if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                //         objProperties = properties && properties[k] && properties[k].items.properties
                //     }
                //     return {[title]: convertObjectKey(value, objProperties)}
                // } else {
                //     return {[title]: value}
                // }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // originToAvailable中的递归方法
        const convertObjectKey = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                let title = properties[k] && properties[k].title
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) return
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertObjectKey(value, objProperties)}
                } else {
                    return {[title]: value}
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // 原始数据转换为可读性数据(转中文map)
        const originToAvailable = (value, model, projectID) => {
            let driversSchema = {}
            let data = {}
            if (!global.schemaCache[projectID]) {
                console.log('global.schemaCache[projectID]未加载完成')
                return {}
            }

            //处理可选的固定组件字段如device,computed
            if (value && _.isPlainObject(value)) {
                if (value._settings) {
                    Object.keys(value._settings).forEach(objKey => {
                        // 资产
                        if (objKey === 'department') {
                            let u = []
                            if (value._settings.department && value._settings.department.length > 0) {
                                u = value._settings.department.map(item => item.name).join(',')
                            }
                            data['所属部门'] = u
                        } else if (objKey === 'parent') {
                            let p = []
                            if (value._settings.parent && value._settings.parent.length > 0) {
                                p = value._settings.parent.map(item => item.name).join(',')
                            }
                            data['所属资产'] = p
                        } else if (objKey === 'name') {
                            data['名称'] = value._settings.name
                        } else if (objKey === 'remark') {
                            data['备注'] = value._settings.remark
                        } else if (objKey === 'creator') {
                            data['创建人'] = value._settings.creator
                        } else if (objKey === 'focus') {
                            data['关注'] = value._settings.focus ? '是' : '否'
                        } else if (objKey === 'online') {
                            data['在线'] = value._settings.online ? '在线' : '掉线'
                        } else if (objKey === 'off') {
                            data['断电'] = value._settings.off ? '上电' : '断电'
                        } else if (objKey === 'disable') {
                            data['禁用'] = value._settings.disable ? '是' : '否'
                        } else if (objKey === 'uid') {
                            data['编号'] = value._settings.uid
                        } else if (objKey === 'gis') {
                            data['地理信息'] = value._settings.gis
                        } else if (objKey === 'model') {
                            data['模型'] = value._tableTitle
                        } else if (objKey === 'dashboard') {
                            data['画面'] = value._settings.dashboard.did
                        } else if (objKey === 'video') {
                            const u = {}
                            if (value._settings.video.url && value._settings.video.url.length > 0) {
                                u['视频地址'] = value._settings.video.url.map(item => ({
                                    id: item.key,
                                    '地址': item.value
                                }))
                            }
                            data['视频'] = u
                        } else if (objKey === 'device' && !_.isEmpty(value._settings.device)) {
                            let d = {}
                            const driver = model && model.model && model.model.device && model.model.device.driver
                            if (driver && driversSchema[driver]) {
                                const properties = driversSchema[driver].device.properties
                                d = convertObjectKey(value._settings.device, properties)
                            }
                            data['设备配置'] = d
                        } else if (objKey === 'computed') {
                            let c = {}
                            // const computedSchema = getNodeSchema({ name: 'computed' })

                            const computedSchema = global.schemaCache[projectID]['computed']
                            if (computedSchema) {
                                const properties = computedSchema.properties
                                c = convertObjectKey(value._settings.computed, properties)
                            }
                            data['计算节点'] = c
                        } else if (objKey === 'warning') {
                            let w = {}
                            // const warningSchema = getNodeSchema({ name: 'warning' })
                            const warningSchema = global.schemaCache[projectID]['warning']
                            if (warningSchema) {
                                const properties = warningSchema.properties
                                w = convertObjectKey(value._settings.warning, properties)
                            }
                            data['报警信息'] = w
                        } else if (objKey === 'hasChild'
                            || objKey === 'modifyTime'
                            || objKey === 'timeoutTime'
                            || objKey === 'connectTime'
                            || objKey === 'createTime'
                            || objKey === 'offlined' // ？？？未知字段
                            || objKey === 'drivers'  // ？？？未知字段
                        ) {
                            //  忽略属性
                        } else if (objKey === "warning.restrainWarning") {
                            data["报警信息"] = {"抑制报警": value._settings[objKey]}
                        } else {
                            data['_settings.' + objKey] = value._settings[objKey]
                        }
                    })
                }
            } else {
                data = value
            }


            if (value && _.isPlainObject(value)) {
                for (var k in value) {
                    const schemaCache = global.schemaCache[projectID][model.extTableName]
                    if (schemaCache && schemaCache.properties) {
                        for (var i in schemaCache.properties) {
                            if (i == k) {
                                if (schemaCache.properties[i] && schemaCache.properties[i].title) {
                                    data[schemaCache.properties[i].title] = value[k]
                                }
                            }
                        }
                    } else {
                        data[objKey] = value[objKey]
                    }
                }
            } else {
                data = value
            }
            return data
        }

        //格式化为一句话
        const formatDiffToWord = (wholeLog, addLogMap, editLogMap, removeLogMap) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + ":";
            data = data + "\n"
            if (addLogMap.length !== 0) {
                Object.keys(addLogMap).forEach(objKey => {
                    if (!addLogMap[objKey]) {
                        data = data + `<span style="color: green">添加</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!addLogMap[objKey].名称 && _.isPlainObject(addLogMap[objKey]) || addLogMap[objKey] instanceof Array) {
                    //     addLogMap[objKey] = JSON.stringify(addLogMap[objKey])
                    // }
                    if (addLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = ''
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (editLogMap.length !== 0) {
                Object.keys(editLogMap).forEach(objKey => {
                    if (!editLogMap[objKey]) {
                        return
                    }
                    let lastWordList = editLogMap[objKey].split('>');
                    let lastWord = '';
                    let originWord = '';
                    if (lastWordList.length > 1) {
                        originWord = lastWordList[0]
                        lastWord = lastWordList[lastWordList.length - 1]
                    }
                    data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ");"
                    data = data + "\n"
                })
            }
            if (removeLogMap.length !== 0) {
                Object.keys(removeLogMap).forEach(objKey => {
                    if (!removeLogMap[objKey]) {
                        data = data + `<span style="color: red">删除</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!removeLogMap[objKey].名称 && _.isPlainObject(removeLogMap[objKey]) || removeLogMap[objKey] instanceof Array) {
                    //     removeLogMap[objKey] = JSON.stringify(removeLogMap[objKey])
                    // }
                    if (removeLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = '';
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };

        let addLogMap = {};
        let editLogMap = {};
        let removeLogMap = {};

        let walkJsonPath = '';
        let splitWalk = [];
        let splitWalkLength = 0;


        // let wholeLog = {
        //     user: {
        //         "id": "22222",
        //         "name": "张三"
        //     },
        //     time: "2020-07-06T09:57:40+08:00",
        //     model: { "id": "5c934d3b839a5547624eaebf", "name": "采油井#" },
        //     oldData: {
        //         "_id": "5efd81b4bdb0ba9e0175298f", "STROKE": 1111, "FREQUENCY": 111, "DYNA_ID": "1", "CHECK_DATE": "2020-07-01T06:41:50.000Z",
        //         "schema": { "record": "测试记录" }
        //     },
        //     data: {
        //         "_id": "5efd81b4bdb0ba9e0175298f", "STROKE": 222, "FREQUENCY": 111, "DYNA_ID": "1", "CHECK_DATE": "2020-07-01T06:41:50.000Z",
        //         "schema": { "record": "测试记录1111" }
        //     },
        //     logHandleType: 'ext',
        //     extTableName: 'gongtu'
        // };

        let originStr = wholeLog.oldData
        let newStr = wholeLog.data

        let newChineseStr = originToAvailable(newStr, wholeLog, projectID);

        if (Object.keys(newChineseStr).length === 0) {
            console.log("newChineseStr global缓存pending")
            return
        }

        let originChineseStr = originToAvailable(originStr, wholeLog, projectID);

        console.log("newChineseStr:", JSON.stringify(newChineseStr))

        console.log("originChineseStr:", JSON.stringify(originChineseStr))

        //提取出两个json中不同的部分
        let deltChinese = jsondiffpatch.diff(originChineseStr, newChineseStr);

        if (!deltChinese) {
            let updateMap = {'diff': ''}
            try {
                let tokenProject = await _this.token(projectID)
                _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
            } catch (err) {
                console.error('updateLog err:', err)
            }
            return
        }

        console.log("deltChinese:", JSON.stringify(deltChinese))

        recursionMergePath(deltChinese, walkJsonPath)
        // console.log(recursionMergePath(deltChinese, walkJsonPath));

        // console.log('walkJsonPath Chinese:', walkJsonPath);

        console.log('addLogMap Chinese:', addLogMap);

        console.log('editLogMap Chinese:', editLogMap);

        console.log('removeLogMap Chinese:', removeLogMap);

        console.log('formatDiffToWord:');
        let diffWord = formatDiffToWord(wholeLog, addLogMap, editLogMap, removeLogMap)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };

    async testTableSchema(wholeLog, projectID) {
        let _this = this
        //生成addLogMap，editLogMap，removeLogMap的key，val
        const recursionMergePath = (obj, walkJsonPath) => {
            return Object.keys(obj).map(k => {
                // console.log("k:", k, "v:", obj[k])

                if (isNaN(k) && k !== '_t') {
                    let testNumber = k.replace('_', "");
                    if (isNaN(testNumber)) {
                        if (walkJsonPath === '') {
                            walkJsonPath = k
                        } else {
                            walkJsonPath = walkJsonPath + '>' + k
                        }
                    }
                }

                if (_.isPlainObject(obj[k])) {
                    recursionMergePath(obj[k], walkJsonPath)
                }

                if (obj[k] instanceof Array) {
                    switch (obj[k].length) {
                        case 1:
                            addLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                        case 2:
                            editLogMap[walkJsonPath] = obj[k][0] + ">" + obj[k][1];
                            splitWalk = walkJsonPath.split('>');
                            // console.log('splitWalk:',splitWalk)
                            splitWalkLength = splitWalk.length - 1;
                            // console.log('splitWalkLength:',splitWalkLength)
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                    // console.log('splitWalkKey:',k,'splitWalkVal:',splitWalk[k],'k !== splitWalkLength:',k !== splitWalkLength)
                                    // console.log(Object.prototype.toString.call(k))
                                    // console.log(Object.prototype.toString.call(splitWalkLength))
                                }
                            });
                            // console.log('splitwalkJsonPath:', walkJsonPath)
                            break;
                        case 3:
                            removeLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                    }
                }


                // const value = obj[k]
                // let title = properties[k] && properties[k].title
                // // console.log(value,title,properties)
                // if (!title && k === 'settings') title = '驱动设置'
                // if (!title || !properties) return
                // if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                //     const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                //     return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                // }
                // if (_.isPlainObject(value) && k !== 'logic') {
                //     let objProperties = properties && properties[k] && properties[k].properties
                //     if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                //         objProperties = properties && properties[k] && properties[k].items.properties
                //     }
                //     return {[title]: convertObjectKey(value, objProperties)}
                // } else {
                //     return {[title]: value}
                // }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // originToAvailable中的递归方法
        const convertObjectKey = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                let title = properties[k] && properties[k].title
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) return
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertObjectKey(value, objProperties)}
                } else {
                    return {[title]: value}
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // 原始数据转换为可读性数据(转中文map)
        const originToAvailable = (value, model, projectID) => {
            let driversSchema = {}

            let data = {}
            try {
                // driversSchema = app.get('drivers')
                if (!global.schemaCache[projectID]) {
                    console.log('global.schemaCache[projectID]未加载完成')
                    return {}
                }
                driversSchema = global.schemaCache[projectID]["device"]
            } catch (error) {
                console.log('加载device模块失败！')
            }
            // const nodeSchema = app.get('models')['Node']
            // const m = convertObjectKey(value, nodeSchema.properties)
            if (value && _.isPlainObject(value)) {
                Object.keys(value).forEach(objKey => {
                    // 资产
                    if (objKey === 'department') {
                        let u = []
                        if (value.department && value.department.length > 0) {
                            u = value.department.map(item => item.name).join(',')
                        }
                        data['所属部门'] = u
                    } else if (objKey === 'parent') {
                        let p = []
                        if (value.parent && value.parent.length > 0) {
                            p = value.parent.map(item => item.name).join(',')
                        }
                        data['所属资产'] = p
                    } else if (objKey === 'name') {
                        data['名称'] = value.name
                    } else if (objKey === 'remark') {
                        data['备注'] = value.remark
                    } else if (objKey === 'creator') {
                        data['创建人'] = value.creator
                    } else if (objKey === 'focus') {
                        data['关注'] = value.focus ? '是' : '否'
                    } else if (objKey === 'online') {
                        data['在线'] = value.online ? '在线' : '掉线'
                    } else if (objKey === 'off') {
                        data['断电'] = value.off ? '上电' : '断电'
                    } else if (objKey === 'disable') {
                        data['禁用'] = value.disable ? '是' : '否'
                    } else if (objKey === 'uid') {
                        data['编号'] = value.uid
                    } else if (objKey === 'gis') {
                        data['地理信息'] = value.gis
                    } else if (objKey === 'model') {
                        data['模型'] = value.model.name
                    } else if (objKey === 'dashboard') {
                        data['画面'] = value.dashboard.did
                    } else if (objKey === 'video') {
                        const u = {}
                        if (value.video.url && value.video.url.length > 0) {
                            u['视频地址'] = value.video.url.map(item => ({id: item.key, '地址': item.value}))
                        }
                        data['视频'] = u
                    } else if (objKey === 'device' && !_.isEmpty(value.device)) {
                        let d = {}
                        // const driver = model && model.model && model.model.device && model.model.device.driver
                        // if (driver && driversSchema[driver]) {
                        //     const properties = driversSchema[driver].device.properties
                        //     d = convertObjectKey(value.device, properties)
                        // }
                        if (value.device) {
                            d = value.device
                        }
                        data['设备配置'] = d
                    } else if (objKey === 'computed') {
                        let c = {}
                        // const computedSchema = getNodeSchema({ name: 'computed' })

                        const computedSchema = global.schemaCache[projectID]['computed']
                        if (computedSchema) {
                            const properties = computedSchema.properties
                            c = convertObjectKey(value.computed, properties)
                        }
                        data['计算节点'] = c
                    } else if (objKey === 'warning') {
                        let w = {}
                        // const warningSchema = getNodeSchema({ name: 'warning' })
                        const warningSchema = global.schemaCache[projectID]['warning']
                        if (warningSchema) {
                            const properties = warningSchema.properties
                            w = convertObjectKey(value.warning, properties)
                        }
                        data['报警信息'] = w
                    } else if (objKey === 'custom') {
                        let c = {}
                        if (value.custom && value.custom.schema) {
                            c = value.custom.schema
                        }
                        data['属性设置'] = c
                    } else if (objKey === 'hasChild'
                        || objKey === 'modifyTime'
                        || objKey === 'timeoutTime'
                        || objKey === 'connectTime'
                        || objKey === 'createTime'
                        || objKey === 'offlined' // ？？？未知字段
                        || objKey === 'drivers'  // ？？？未知字段
                    ) {
                        //  忽略属性
                    } else {
                        data[objKey] = value[objKey]
                    }
                })
            } else {
                data = value
            }

            return data
        }

        //格式化为一句话
        const formatDiffToWord = (wholeLog, addLogMap, editLogMap, removeLogMap) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + ":";
            data = data + "\n"
            if (addLogMap.length !== 0) {
                Object.keys(addLogMap).forEach(objKey => {
                    if (!addLogMap[objKey]) {
                        data = data + `<span style="color: green">添加</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!addLogMap[objKey].名称 && _.isPlainObject(addLogMap[objKey]) || addLogMap[objKey] instanceof Array) {
                    //     addLogMap[objKey] = JSON.stringify(addLogMap[objKey])
                    // }
                    if (addLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = ''
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (editLogMap.length !== 0) {
                Object.keys(editLogMap).forEach(objKey => {
                    if (!editLogMap[objKey]) {
                        return
                    }
                    let lastWordList = editLogMap[objKey].split('>');
                    let lastWord = '';
                    let originWord = '';
                    if (lastWordList.length > 1) {
                        originWord = lastWordList[0]
                        lastWord = lastWordList[lastWordList.length - 1]
                    }
                    data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ");"
                    data = data + "\n"
                })
            }
            if (removeLogMap.length !== 0) {
                Object.keys(removeLogMap).forEach(objKey => {
                    if (!removeLogMap[objKey]) {
                        data = data + `<span style="color: red">删除</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!removeLogMap[objKey].名称 && _.isPlainObject(removeLogMap[objKey]) || removeLogMap[objKey] instanceof Array) {
                    //     removeLogMap[objKey] = JSON.stringify(removeLogMap[objKey])
                    // }
                    if (removeLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = '';
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };

        let addLogMap = {};
        let editLogMap = {};
        let removeLogMap = {};

        let walkJsonPath = '';
        let splitWalk = [];
        let splitWalkLength = 0;


        // let wholeLog = {
        //     user: {
        //         "id": "22222",
        //         "name": "张三"
        //     },
        //     time: "2020-07-06T09:57:40+08:00",
        //     model: { "id": "5c934d3b839a5547624eaebf", "name": "采油井#" },
        //     oldData: {
        //         "_id": "5efd81b4bdb0ba9e0175298f", "STROKE": 1111, "FREQUENCY": 111, "DYNA_ID": "1", "CHECK_DATE": "2020-07-01T06:41:50.000Z",
        //         "schema": { "record": "测试记录" }
        //     },
        //     data: {
        //         "_id": "5efd81b4bdb0ba9e0175298f", "STROKE": 222, "FREQUENCY": 111, "DYNA_ID": "1", "CHECK_DATE": "2020-07-01T06:41:50.000Z",
        //         "schema": { "record": "测试记录1111" }
        //     },
        //     logHandleType: 'ext',
        //     extTableName: 'gongtu'
        // };

        let originStr = wholeLog.oldData
        let newStr = wholeLog.data

        let newChineseStr = originToAvailable(newStr, wholeLog, projectID);

        if (Object.keys(newChineseStr).length === 0) {
            console.log("newChineseStr global缓存pending")
            return
        }

        let originChineseStr = originToAvailable(originStr, wholeLog, projectID);

        console.log("newChineseStr:", JSON.stringify(newChineseStr))

        console.log("originChineseStr:", JSON.stringify(originChineseStr))

        //提取出两个json中不同的部分
        let deltChinese = jsondiffpatch.diff(originChineseStr, newChineseStr);

        if (!deltChinese) {
            let updateMap = {'diff': ''}
            try {
                let tokenProject = await _this.token(projectID)
                _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
            } catch (err) {
                console.error('updateLog err:', err)
            }
            return
        }

        console.log("deltChinese:", JSON.stringify(deltChinese))

        recursionMergePath(deltChinese, walkJsonPath)
        // console.log(recursionMergePath(deltChinese, walkJsonPath));

        // console.log('walkJsonPath Chinese:', walkJsonPath);

        console.log('addLogMap Chinese:', addLogMap);

        console.log('editLogMap Chinese:', editLogMap);

        console.log('removeLogMap Chinese:', removeLogMap);

        console.log('formatDiffToWord:');
        let diffWord = formatDiffToWord(wholeLog, addLogMap, editLogMap, removeLogMap)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };

    async testEventSchema(wholeLog, projectID) {
        let _this = this
        //生成addLogMap，editLogMap，removeLogMap的key，val
        const recursionMergePath = (obj, walkJsonPath) => {
            return Object.keys(obj).map(k => {
                // console.log("k:", k, "v:", obj[k])

                if (isNaN(k) && k !== '_t') {
                    let testNumber = k.replace('_', "");
                    if (isNaN(testNumber)) {
                        if (walkJsonPath === '') {
                            walkJsonPath = k
                        } else {
                            walkJsonPath = walkJsonPath + '>' + k
                        }
                    }
                }

                if (_.isPlainObject(obj[k])) {
                    recursionMergePath(obj[k], walkJsonPath)
                }

                if (obj[k] instanceof Array) {
                    switch (obj[k].length) {
                        case 1:
                            addLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                        case 2:
                            editLogMap[walkJsonPath] = obj[k][0] + ">" + obj[k][1];
                            splitWalk = walkJsonPath.split('>');
                            // console.log('splitWalk:',splitWalk)
                            splitWalkLength = splitWalk.length - 1;
                            // console.log('splitWalkLength:',splitWalkLength)
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                    // console.log('splitWalkKey:',k,'splitWalkVal:',splitWalk[k],'k !== splitWalkLength:',k !== splitWalkLength)
                                    // console.log(Object.prototype.toString.call(k))
                                    // console.log(Object.prototype.toString.call(splitWalkLength))
                                }
                            });
                            // console.log('splitwalkJsonPath:', walkJsonPath)
                            break;
                        case 3:
                            removeLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                    }
                }


                // const value = obj[k]
                // let title = properties[k] && properties[k].title
                // // console.log(value,title,properties)
                // if (!title && k === 'settings') title = '驱动设置'
                // if (!title || !properties) return
                // if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                //     const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                //     return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                // }
                // if (_.isPlainObject(value) && k !== 'logic') {
                //     let objProperties = properties && properties[k] && properties[k].properties
                //     if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                //         objProperties = properties && properties[k] && properties[k].items.properties
                //     }
                //     return {[title]: convertObjectKey(value, objProperties)}
                // } else {
                //     return {[title]: value}
                // }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // originToAvailable中的递归方法
        const convertObjectKey = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                let title = properties[k] && properties[k].title
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) return
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertObjectKey(value, objProperties)}
                } else {
                    return {[title]: value}
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // 原始数据转换为可读性数据(转中文map)
        const originToAvailable = (value, model, projectID) => {
            let driversSchema = {}

            let data = {}
            try {
                // driversSchema = app.get('drivers')
                if (!global.schemaCache[projectID]) {
                    console.log('global.schemaCache[projectID]未加载完成')
                    return {}
                }
                driversSchema = global.schemaCache[projectID]["device"]
            } catch (error) {
                console.log('加载device模块失败！')
            }
            // const nodeSchema = app.get('models')['Node']
            // const m = convertObjectKey(value, nodeSchema.properties)
            if (value && _.isPlainObject(value)) {
                Object.keys(value).forEach(objKey => {
                    // 资产
                    if (objKey === 'department') {
                        let u = []
                        if (value.department && value.department.length > 0) {
                            u = value.department.map(item => item.name).join(',')
                        }
                        data['所属部门'] = u
                    } else if (objKey === 'parent') {
                        let p = []
                        if (value.parent && value.parent.length > 0) {
                            p = value.parent.map(item => item.name).join(',')
                        }
                        data['所属资产'] = p
                    } else if (objKey === 'name') {
                        data['名称'] = value.name
                    } else if (objKey === 'remark') {
                        data['备注'] = value.remark
                    } else if (objKey === 'creator') {
                        data['创建人'] = value.creator
                    } else if (objKey === 'focus') {
                        data['关注'] = value.focus ? '是' : '否'
                    } else if (objKey === 'online') {
                        data['在线'] = value.online ? '在线' : '掉线'
                    } else if (objKey === 'off') {
                        data['断电'] = value.off ? '上电' : '断电'
                    } else if (objKey === 'disable') {
                        data['禁用'] = value.disable ? '是' : '否'
                    } else if (objKey === 'uid') {
                        data['编号'] = value.uid
                    } else if (objKey === 'gis') {
                        data['地理信息'] = value.gis
                    } else if (objKey === 'model') {
                        data['模型'] = value.model.name
                    } else if (objKey === 'dashboard') {
                        data['画面'] = value.dashboard.did
                    } else if (objKey === 'video') {
                        const u = {}
                        if (value.video.url && value.video.url.length > 0) {
                            u['视频地址'] = value.video.url.map(item => ({id: item.key, '地址': item.value}))
                        }
                        data['视频'] = u
                    } else if (objKey === 'device' && !_.isEmpty(value.device)) {
                        let d = {}
                        const driver = model && model.model && model.model.device && model.model.device.driver
                        if (driver && driversSchema[driver]) {
                            const properties = driversSchema[driver].device.properties
                            d = convertObjectKey(value.device, properties)
                        }
                        data['设备配置'] = d
                    } else if (objKey === 'computed') {
                        let c = {}
                        // const computedSchema = getNodeSchema({ name: 'computed' })

                        const computedSchema = global.schemaCache[projectID]['computed']
                        if (computedSchema) {
                            const properties = computedSchema.properties
                            c = convertObjectKey(value.computed, properties)
                        }
                        data['计算节点'] = c
                    } else if (objKey === 'warning') {
                        let w = {}
                        // const warningSchema = getNodeSchema({ name: 'warning' })
                        const warningSchema = global.schemaCache[projectID]['warning']
                        if (warningSchema) {
                            const properties = warningSchema.properties
                            w = convertObjectKey(value.warning, properties)
                        }
                        data['报警信息'] = w
                    } else if (objKey === 'custom') {
                        let c = {}
                        if (value.custom && value.custom.schema) {
                            c = value.custom.schema
                        }
                        data['属性设置'] = c
                    } else if (objKey === 'hasChild'
                        || objKey === 'modifyTime'
                        || objKey === 'timeoutTime'
                        || objKey === 'connectTime'
                        || objKey === 'createTime'
                        || objKey === 'offlined' // ？？？未知字段
                        || objKey === 'drivers'  // ？？？未知字段
                    ) {
                        //  忽略属性
                    } else {
                        data[objKey] = value[objKey]
                    }
                })
            } else {
                data = value
            }

            return data
        }

        //格式化为一句话
        const formatDiffToWord = (wholeLog, addLogMap, editLogMap, removeLogMap) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + ":";
            data = data + "\n"
            if (addLogMap.length !== 0) {
                Object.keys(addLogMap).forEach(objKey => {
                    if (!addLogMap[objKey]) {
                        data = data + `<span style="color: green">添加</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!addLogMap[objKey].名称 && _.isPlainObject(addLogMap[objKey]) || addLogMap[objKey] instanceof Array) {
                    //     addLogMap[objKey] = JSON.stringify(addLogMap[objKey])
                    // }
                    if (addLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = ''
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (editLogMap.length !== 0) {
                Object.keys(editLogMap).forEach(objKey => {
                    if (!editLogMap[objKey]) {
                        return
                    }
                    let lastWordList = editLogMap[objKey].split('>');
                    let lastWord = '';
                    let originWord = '';
                    if (lastWordList.length > 1) {
                        originWord = lastWordList[0]
                        lastWord = lastWordList[lastWordList.length - 1]
                    }
                    data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ");"
                    data = data + "\n"
                })
            }
            if (removeLogMap.length !== 0) {
                Object.keys(removeLogMap).forEach(objKey => {
                    if (!removeLogMap[objKey]) {
                        data = data + `<span style="color: red">删除</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!removeLogMap[objKey].名称 && _.isPlainObject(removeLogMap[objKey]) || removeLogMap[objKey] instanceof Array) {
                    //     removeLogMap[objKey] = JSON.stringify(removeLogMap[objKey])
                    // }
                    if (removeLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = '';
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };

        let addLogMap = {};
        let editLogMap = {};
        let removeLogMap = {};

        let walkJsonPath = '';
        let splitWalk = [];
        let splitWalkLength = 0;


        // let wholeLog = {
        //     user: {
        //         "id": "22222",
        //         "name": "张三"
        //     },
        //     time: "2020-07-06T09:57:40+08:00",
        //     model: {"id": "5c934d3b839a5547624eaebf", "name": "采油井#"},
        //     oldData: {
        //         "computed": {
        //             "tags": [
        //                 {
        //                     "id": "YXZT",
        //                     "mapping": {
        //                         "id": "YXZT",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "运行状态",
        //                     "policy": "save",
        //                     "type": "mapping",
        //                 },
        //                 {
        //                     "id": "YW",
        //                     "mapping": {
        //                         "id": "YW",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "油温",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //                 {
        //                     "id": "YW2",
        //                     "mapping": {
        //                         "id": "YW2",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "油温2",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //             ]
        //         },
        //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站"},
        //         "custom": {"record": "测试记录"},
        //         "warning": {
        //             "rules": [
        //                 {
        //                     "alert": true,
        //                     "description": "注气焖井油温>200",
        //                     "disable": true,
        //                     "extraTags": [
        //                         {
        //                             "id": "lat"
        //                         }
        //                     ],
        //                     "handle": false,
        //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
        //                     "interval": 30,
        //                     "level": "中",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "==": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     200
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
        //                     "warningname": "注气报警"
        //                 },
        //                 {
        //                     "alert": true,
        //                     "description": "非注气状态报警",
        //                     "disable": true,
        //                     "handle": true,
        //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "!=": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     180
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "超限报警",
        //                     "warningname": "非注气状态"
        //                 }
        //             ]
        //         }
        //     },
        //     data: {
        //         "computed": {
        //             "tags": [
        //                 {
        //                     "id": "YXZT",
        //                     "mapping": {
        //                         "id": "YXZT",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "运行状态1",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //                 {
        //                     "id": "IA",
        //                     "mapping": {
        //                         "id": "IA",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "A相电流",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 }
        //             ],
        //         },
        //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站1"},
        //         "custom": {"record": "测试记录1", "select-4F10": "测试选择器111"},
        //         "warning": {
        //             "rules": [
        //                 {
        //                     "alert": true,
        //                     "description": "注气焖井油温>200",
        //                     "disable": true,
        //                     "extraTags": [
        //                         {
        //                             "id": "lat"
        //                         }
        //                     ],
        //                     "handle": false,
        //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
        //                     "interval": 30,
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "==": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     200
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
        //                     "warningname": "注气报警"
        //                 },
        //                 {
        //                     "alert": true,
        //                     "description": "非注气状态报警",
        //                     "disable": true,
        //                     "handle": true,
        //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "!=": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     180
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "超限报警",
        //                     "warningname": "非注气状态"
        //                 }
        //             ]
        //         }
        //     }
        // };

        let originStr = wholeLog.oldData
        let newStr = wholeLog.data

        let newChineseStr = originToAvailable(newStr, wholeLog, projectID);

        if (Object.keys(newChineseStr).length === 0) {
            console.log("newChineseStr global缓存pending")
            return
        }

        let originChineseStr = originToAvailable(originStr, wholeLog, projectID);

        console.log("newChineseStr:", JSON.stringify(newChineseStr))

        console.log("originChineseStr:", JSON.stringify(originChineseStr))

        //提取出两个json中不同的部分
        let deltChinese = jsondiffpatch.diff(originChineseStr, newChineseStr);

        if (!deltChinese) {
            let updateMap = {'diff': ''}
            try {
                let tokenProject = await _this.token(projectID)
                _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
            } catch (err) {
                console.error('updateLog err:', err)
            }
            return
        }
        console.log("deltChinese:", JSON.stringify(deltChinese))

        recursionMergePath(deltChinese, walkJsonPath)
        // console.log(recursionMergePath(deltChinese, walkJsonPath));

        // console.log('walkJsonPath Chinese:', walkJsonPath);

        console.log('addLogMap Chinese:', addLogMap);

        console.log('editLogMap Chinese:', editLogMap);

        console.log('removeLogMap Chinese:', removeLogMap);

        console.log('formatDiffToWord:');
        let diffWord = formatDiffToWord(wholeLog, addLogMap, editLogMap, removeLogMap)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };

    async testEventHandlerSchema(wholeLog, projectID) {
        let _this = this
        //生成addLogMap，editLogMap，removeLogMap的key，val
        const recursionMergePath = (obj, walkJsonPath) => {
            return Object.keys(obj).map(k => {
                // console.log("k:", k, "v:", obj[k])

                if (isNaN(k) && k !== '_t') {
                    let testNumber = k.replace('_', "");
                    if (isNaN(testNumber)) {
                        if (walkJsonPath === '') {
                            walkJsonPath = k
                        } else {
                            walkJsonPath = walkJsonPath + '>' + k
                        }
                    }
                }

                if (_.isPlainObject(obj[k])) {
                    recursionMergePath(obj[k], walkJsonPath)
                }

                if (obj[k] instanceof Array) {
                    switch (obj[k].length) {
                        case 1:
                            addLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                        case 2:
                            editLogMap[walkJsonPath] = obj[k][0] + ">" + obj[k][1];
                            splitWalk = walkJsonPath.split('>');
                            // console.log('splitWalk:',splitWalk)
                            splitWalkLength = splitWalk.length - 1;
                            // console.log('splitWalkLength:',splitWalkLength)
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                    // console.log('splitWalkKey:',k,'splitWalkVal:',splitWalk[k],'k !== splitWalkLength:',k !== splitWalkLength)
                                    // console.log(Object.prototype.toString.call(k))
                                    // console.log(Object.prototype.toString.call(splitWalkLength))
                                }
                            });
                            // console.log('splitwalkJsonPath:', walkJsonPath)
                            break;
                        case 3:
                            removeLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                    }
                }


                // const value = obj[k]
                // let title = properties[k] && properties[k].title
                // // console.log(value,title,properties)
                // if (!title && k === 'settings') title = '驱动设置'
                // if (!title || !properties) return
                // if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                //     const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                //     return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                // }
                // if (_.isPlainObject(value) && k !== 'logic') {
                //     let objProperties = properties && properties[k] && properties[k].properties
                //     if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                //         objProperties = properties && properties[k] && properties[k].items.properties
                //     }
                //     return {[title]: convertObjectKey(value, objProperties)}
                // } else {
                //     return {[title]: value}
                // }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // originToAvailable中的递归方法
        const convertObjectKey = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                let title = properties[k] && properties[k].title
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) return
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertObjectKey(value, objProperties)}
                } else {
                    return {[title]: value}
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // 原始数据转换为可读性数据(转中文map)
        const originToAvailable = (value, model, projectID) => {
            let driversSchema = {}

            let data = {}
            try {
                // driversSchema = app.get('drivers')
                if (!global.schemaCache[projectID]) {
                    console.log('global.schemaCache[projectID]未加载完成')
                    return {}
                }
                driversSchema = global.schemaCache[projectID]["device"]
            } catch (error) {
                console.log('加载device模块失败！')
            }
            // const nodeSchema = app.get('models')['Node']
            // const m = convertObjectKey(value, nodeSchema.properties)
            if (value && _.isPlainObject(value)) {
                Object.keys(value).forEach(objKey => {
                    // 资产
                    if (objKey === 'department') {
                        let u = []
                        if (value.department && value.department.length > 0) {
                            u = value.department.map(item => item.name).join(',')
                        }
                        data['所属部门'] = u
                    } else if (objKey === 'parent') {
                        let p = []
                        if (value.parent && value.parent.length > 0) {
                            p = value.parent.map(item => item.name).join(',')
                        }
                        data['所属资产'] = p
                    } else if (objKey === 'handlername') {
                        data['名称'] = value.handlername
                    } else if (objKey === 'remark') {
                        data['备注'] = value.remark
                    } else if (objKey === 'creator') {
                        data['创建人'] = value.creator
                    } else if (objKey === 'focus') {
                        data['关注'] = value.focus ? '是' : '否'
                    } else if (objKey === 'online') {
                        data['在线'] = value.online ? '在线' : '掉线'
                    } else if (objKey === 'off') {
                        data['断电'] = value.off ? '上电' : '断电'
                    } else if (objKey === 'disable') {
                        data['禁用'] = value.disable ? '是' : '否'
                    } else if (objKey === 'uid') {
                        data['编号'] = value.uid
                    } else if (objKey === 'gis') {
                        data['地理信息'] = value.gis
                    } else if (objKey === 'model') {
                        data['模型'] = value.model.name
                    } else if (objKey === 'dashboard') {
                        data['画面'] = value.dashboard.did
                    } else if (objKey === 'video') {
                        const u = {}
                        if (value.video.url && value.video.url.length > 0) {
                            u['视频地址'] = value.video.url.map(item => ({id: item.key, '地址': item.value}))
                        }
                        data['视频'] = u
                    } else if (objKey === 'device' && !_.isEmpty(value.device)) {
                        let d = {}
                        const driver = model && model.model && model.model.device && model.model.device.driver
                        if (driver && driversSchema[driver]) {
                            const properties = driversSchema[driver].device.properties
                            d = convertObjectKey(value.device, properties)
                        }
                        data['设备配置'] = d
                    } else if (objKey === 'computed') {
                        let c = {}
                        // const computedSchema = getNodeSchema({ name: 'computed' })

                        const computedSchema = global.schemaCache[projectID]['computed']
                        if (computedSchema) {
                            const properties = computedSchema.properties
                            c = convertObjectKey(value.computed, properties)
                        }
                        data['计算节点'] = c
                    } else if (objKey === 'warning') {
                        let w = {}
                        // const warningSchema = getNodeSchema({ name: 'warning' })
                        const warningSchema = global.schemaCache[projectID]['warning']
                        if (warningSchema) {
                            const properties = warningSchema.properties
                            w = convertObjectKey(value.warning, properties)
                        }
                        data['报警信息'] = w
                    } else if (objKey === 'custom') {
                        let c = {}
                        if (value.custom && value.custom.schema) {
                            c = value.custom.schema
                        }
                        data['属性设置'] = c
                    } else if (objKey === 'hasChild'
                        || objKey === 'modifyTime'
                        || objKey === 'timeoutTime'
                        || objKey === 'connectTime'
                        || objKey === 'createTime'
                        || objKey === 'offlined' // ？？？未知字段
                        || objKey === 'drivers'  // ？？？未知字段
                    ) {
                        //  忽略属性
                    } else {
                        data[objKey] = value[objKey]
                    }
                })
            } else {
                data = value
            }

            return data
        }

        //格式化为一句话
        const formatDiffToWord = (wholeLog, addLogMap, editLogMap, removeLogMap) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + ":";
            data = data + "\n"
            if (addLogMap.length !== 0) {
                Object.keys(addLogMap).forEach(objKey => {
                    if (!addLogMap[objKey]) {
                        data = data + `<span style="color: green">添加</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!addLogMap[objKey].名称 && _.isPlainObject(addLogMap[objKey]) || addLogMap[objKey] instanceof Array) {
                    //     addLogMap[objKey] = JSON.stringify(addLogMap[objKey])
                    // }
                    if (addLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = ''
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (editLogMap.length !== 0) {
                Object.keys(editLogMap).forEach(objKey => {
                    if (!editLogMap[objKey]) {
                        return
                    }
                    let lastWordList = editLogMap[objKey].split('>');
                    let lastWord = '';
                    let originWord = '';
                    if (lastWordList.length > 1) {
                        originWord = lastWordList[0]
                        lastWord = lastWordList[lastWordList.length - 1]
                    }
                    data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ");"
                    data = data + "\n"
                })
            }
            if (removeLogMap.length !== 0) {
                Object.keys(removeLogMap).forEach(objKey => {
                    if (!removeLogMap[objKey]) {
                        data = data + `<span style="color: red">删除</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!removeLogMap[objKey].名称 && _.isPlainObject(removeLogMap[objKey]) || removeLogMap[objKey] instanceof Array) {
                    //     removeLogMap[objKey] = JSON.stringify(removeLogMap[objKey])
                    // }
                    if (removeLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = '';
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };

        let addLogMap = {};
        let editLogMap = {};
        let removeLogMap = {};

        let walkJsonPath = '';
        let splitWalk = [];
        let splitWalkLength = 0;


        let originStr = wholeLog.oldData
        let newStr = wholeLog.data

        let newChineseStr = originToAvailable(newStr, wholeLog, projectID);

        if (Object.keys(newChineseStr).length === 0) {
            console.log("newChineseStr global缓存pending")
            return
        }

        let originChineseStr = originToAvailable(originStr, wholeLog, projectID);

        console.log("newChineseStr:", JSON.stringify(newChineseStr))

        console.log("originChineseStr:", JSON.stringify(originChineseStr))

        //提取出两个json中不同的部分
        let deltChinese = jsondiffpatch.diff(originChineseStr, newChineseStr);

        if (!deltChinese) {
            let updateMap = {'diff': ''}
            try {
                let tokenProject = await _this.token(projectID)
                _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
            } catch (err) {
                console.error('updateLog err:', err)
            }
            return
        }
        console.log("deltChinese:", JSON.stringify(deltChinese))

        recursionMergePath(deltChinese, walkJsonPath)
        // console.log(recursionMergePath(deltChinese, walkJsonPath));

        // console.log('walkJsonPath Chinese:', walkJsonPath);

        console.log('addLogMap Chinese:', addLogMap);

        console.log('editLogMap Chinese:', editLogMap);

        console.log('removeLogMap Chinese:', removeLogMap);

        console.log('formatDiffToWord:');
        let diffWord = formatDiffToWord(wholeLog, addLogMap, editLogMap, removeLogMap)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };

    async testReportSchema(wholeLog, projectID) {
        let _this = this
        //生成addLogMap，editLogMap，removeLogMap的key，val
        const recursionMergePath = (obj, walkJsonPath) => {
            return Object.keys(obj).map(k => {
                // console.log("k:", k, "v:", obj[k])

                if (isNaN(k) && k !== '_t') {
                    let testNumber = k.replace('_', "");
                    if (isNaN(testNumber)) {
                        if (walkJsonPath === '') {
                            walkJsonPath = k
                        } else {
                            walkJsonPath = walkJsonPath + '>' + k
                        }
                    }
                }

                if (_.isPlainObject(obj[k])) {
                    recursionMergePath(obj[k], walkJsonPath)
                }

                if (obj[k] instanceof Array) {
                    switch (obj[k].length) {
                        case 1:
                            addLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                        case 2:
                            editLogMap[walkJsonPath] = obj[k][0] + ">" + obj[k][1];
                            splitWalk = walkJsonPath.split('>');
                            // console.log('splitWalk:',splitWalk)
                            splitWalkLength = splitWalk.length - 1;
                            // console.log('splitWalkLength:',splitWalkLength)
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                    // console.log('splitWalkKey:',k,'splitWalkVal:',splitWalk[k],'k !== splitWalkLength:',k !== splitWalkLength)
                                    // console.log(Object.prototype.toString.call(k))
                                    // console.log(Object.prototype.toString.call(splitWalkLength))
                                }
                            });
                            // console.log('splitwalkJsonPath:', walkJsonPath)
                            break;
                        case 3:
                            removeLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                    }
                }


                // const value = obj[k]
                // let title = properties[k] && properties[k].title
                // // console.log(value,title,properties)
                // if (!title && k === 'settings') title = '驱动设置'
                // if (!title || !properties) return
                // if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                //     const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                //     return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                // }
                // if (_.isPlainObject(value) && k !== 'logic') {
                //     let objProperties = properties && properties[k] && properties[k].properties
                //     if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                //         objProperties = properties && properties[k] && properties[k].items.properties
                //     }
                //     return {[title]: convertObjectKey(value, objProperties)}
                // } else {
                //     return {[title]: value}
                // }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // originToAvailable中的递归方法
        const convertObjectKey = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                let title = properties[k] && properties[k].title
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) return
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertObjectKey(value, objProperties)}
                } else {
                    return {[title]: value}
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // 原始数据转换为可读性数据(转中文map)
        const originToAvailable = (value, model, projectID) => {

            let data = {}
            // const nodeSchema = app.get('models')['Node']
            // const m = convertObjectKey(value, nodeSchema.properties)
            if (value && _.isPlainObject(value)) {
                Object.keys(value).forEach(objKey => {
                    // 资产
                    if (objKey === 'name') {
                        data['名称'] = value.name
                    } else if (objKey === 'canEdit') {
                        data['查看可编辑'] = value.canEdit
                    } else if (objKey === 'catalog') {
                        if (value.catalog && value.catalog.name) {
                            data['所属目录'] = value.catalog.name
                        }
                    } else if (objKey === 'exportTime') {
                        data['导出时间'] = value.exportTime
                    } else if (objKey === 'exportUser') {
                        if (value.exportUser && value.exportUser.name) {
                            data['导出用户'] = value.exportUser.name
                        }
                    } else if (objKey === 'focus') {
                        data['关注'] = value.focus
                    } else if (objKey === 'interval') {
                        data['报表周期'] = value.interval
                    } else if (objKey === 'model') {
                        if (value.model && value.model.name) {
                            data['模型'] = value.model.name
                        }
                    } else if (objKey === 'node') {
                        if (value.node && value.node.name) {
                            data['资产'] = value.node.name
                        }
                    } else if (objKey === 'reportType') {
                        data['报表类型'] = value.reportType
                    } else if (objKey === 'type') {
                        data['统计类型'] = value.type
                    } else if (objKey === 'startTime') {
                        data['起始时间'] = value.startTime
                    } else if (objKey === 'printTime') {
                        data['打印时间'] = value.printTime
                    } else if (objKey === 'printUser') {
                        if (value.printUser && value.printUser.name) {
                            data['打印用户'] = value.printUser.name
                        }
                    } else if (objKey === 'interval') {
                        data['周期间隔'] = value.interval
                    } else if (objKey === 'table') {
                        data['表格'] = value.table
                    } else if (objKey === 'data') {
                        data['副本表格'] = value.data
                    }
                })
            } else {
                data = value
            }

            return data
        };

        //格式化为一句话
        const formatDiffToWord = (wholeLog, addLogMap, editLogMap, removeLogMap) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + ":";
            data = data + "\n"
            if (addLogMap.length !== 0) {
                Object.keys(addLogMap).forEach(objKey => {
                    if (!addLogMap[objKey]) {
                        data = data + `<span style="color: green">添加</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!addLogMap[objKey].名称 && _.isPlainObject(addLogMap[objKey]) || addLogMap[objKey] instanceof Array) {
                    //     addLogMap[objKey] = JSON.stringify(addLogMap[objKey])
                    // }
                    if (addLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = ''
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (editLogMap.length !== 0) {
                Object.keys(editLogMap).forEach(objKey => {
                    if (!editLogMap[objKey]) {
                        return
                    }
                    let lastWordList = editLogMap[objKey].split('>');
                    let lastWord = '';
                    let originWord = '';
                    if (lastWordList.length > 1) {
                        originWord = lastWordList[0]
                        lastWord = lastWordList[lastWordList.length - 1]
                    }
                    data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ");"
                    data = data + "\n"
                })
            }
            if (removeLogMap.length !== 0) {
                Object.keys(removeLogMap).forEach(objKey => {
                    if (!removeLogMap[objKey]) {
                        data = data + `<span style="color: red">删除</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!removeLogMap[objKey].名称 && _.isPlainObject(removeLogMap[objKey]) || removeLogMap[objKey] instanceof Array) {
                    //     removeLogMap[objKey] = JSON.stringify(removeLogMap[objKey])
                    // }
                    if (removeLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = '';
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };

        let addLogMap = {};
        let editLogMap = {};
        let removeLogMap = {};

        let walkJsonPath = '';
        let splitWalk = [];
        let splitWalkLength = 0;


        let originStr = wholeLog.oldData
        let newStr = wholeLog.data

        let newChineseStr = originToAvailable(newStr, wholeLog, projectID);

        if (Object.keys(newChineseStr).length === 0) {
            console.log("newChineseStr global缓存pending")
            return
        }

        let originChineseStr = originToAvailable(originStr, wholeLog, projectID);

        console.log("newChineseStr:", JSON.stringify(newChineseStr))

        console.log("originChineseStr:", JSON.stringify(originChineseStr))

        //提取出两个json中不同的部分
        let deltChinese = jsondiffpatch.diff(originChineseStr, newChineseStr);

        if (!deltChinese) {
            let updateMap = {'diff': ''}
            try {
                let tokenProject = await _this.token(projectID)
                _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
            } catch (err) {
                console.error('updateLog err:', err)
            }
            return
        }
        console.log("deltChinese:", JSON.stringify(deltChinese))

        recursionMergePath(deltChinese, walkJsonPath)

        // console.log('walkJsonPath Chinese:', walkJsonPath);

        console.log('addLogMap Chinese:', addLogMap);

        console.log('editLogMap Chinese:', editLogMap);

        console.log('removeLogMap Chinese:', removeLogMap);

        console.log('formatDiffToWord:');
        let diffWord = formatDiffToWord(wholeLog, addLogMap, editLogMap, removeLogMap)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };

    async testWarnRemarkSchema(wholeLog, projectID) {
        let _this = this
        //生成addLogMap，editLogMap，removeLogMap的key，val
        const recursionMergePath = (obj, walkJsonPath) => {
            return Object.keys(obj).map(k => {
                // console.log("k:", k, "v:", obj[k])

                if (isNaN(k) && k !== '_t') {
                    let testNumber = k.replace('_', "");
                    if (isNaN(testNumber)) {
                        if (walkJsonPath === '') {
                            walkJsonPath = k
                        } else {
                            walkJsonPath = walkJsonPath + '>' + k
                        }
                    }
                }

                if (_.isPlainObject(obj[k])) {
                    recursionMergePath(obj[k], walkJsonPath)
                }

                if (obj[k] instanceof Array) {
                    switch (obj[k].length) {
                        case 1:
                            addLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                        case 2:
                            editLogMap[walkJsonPath] = obj[k][0] + ">" + obj[k][1];
                            splitWalk = walkJsonPath.split('>');
                            // console.log('splitWalk:',splitWalk)
                            splitWalkLength = splitWalk.length - 1;
                            // console.log('splitWalkLength:',splitWalkLength)
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                    // console.log('splitWalkKey:',k,'splitWalkVal:',splitWalk[k],'k !== splitWalkLength:',k !== splitWalkLength)
                                    // console.log(Object.prototype.toString.call(k))
                                    // console.log(Object.prototype.toString.call(splitWalkLength))
                                }
                            });
                            // console.log('splitwalkJsonPath:', walkJsonPath)
                            break;
                        case 3:
                            removeLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                    }
                }


                // const value = obj[k]
                // let title = properties[k] && properties[k].title
                // // console.log(value,title,properties)
                // if (!title && k === 'settings') title = '驱动设置'
                // if (!title || !properties) return
                // if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                //     const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                //     return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                // }
                // if (_.isPlainObject(value) && k !== 'logic') {
                //     let objProperties = properties && properties[k] && properties[k].properties
                //     if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                //         objProperties = properties && properties[k] && properties[k].items.properties
                //     }
                //     return {[title]: convertObjectKey(value, objProperties)}
                // } else {
                //     return {[title]: value}
                // }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // originToAvailable中的递归方法
        const convertObjectKey = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                let title = properties[k] && properties[k].title
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) return
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertObjectKey(value, objProperties)}
                } else {
                    return {[title]: value}
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // 原始数据转换为可读性数据(转中文map)
        const originToAvailable = (value, model, projectID) => {

            let data = {}
            // const nodeSchema = app.get('models')['Node']
            // const m = convertObjectKey(value, nodeSchema.properties)
            if (value && _.isPlainObject(value)) {
                Object.keys(value).forEach(objKey => {
                    // 资产
                    if (objKey === 'remark') {
                        data['备注'] = value.remark
                    }
                    if (objKey === 'status') {
                        data['确认状态'] = value.status
                    }
                    if (objKey === 'processed') {
                        data['处理状态'] = value.processed
                    }
                })
            } else {
                data = value
            }

            return data
        };

        //格式化为一句话
        const formatDiffToWord = (wholeLog, addLogMap, editLogMap, removeLogMap) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + ":";
            data = data + "\n"
            if (addLogMap.length !== 0) {
                Object.keys(addLogMap).forEach(objKey => {
                    if (!addLogMap[objKey]) {
                        data = data + `<span style="color: green">添加</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!addLogMap[objKey].名称 && _.isPlainObject(addLogMap[objKey]) || addLogMap[objKey] instanceof Array) {
                    //     addLogMap[objKey] = JSON.stringify(addLogMap[objKey])
                    // }
                    if (addLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = ''
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (editLogMap.length !== 0) {
                Object.keys(editLogMap).forEach(objKey => {
                    if (!editLogMap[objKey]) {
                        return
                    }
                    let lastWordList = editLogMap[objKey].split('>');
                    let lastWord = '';
                    let originWord = '';
                    if (lastWordList.length > 1) {
                        originWord = lastWordList[0]
                        lastWord = lastWordList[lastWordList.length - 1]
                    }
                    data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ");"
                    data = data + "\n"
                })
            }
            if (removeLogMap.length !== 0) {
                Object.keys(removeLogMap).forEach(objKey => {
                    if (!removeLogMap[objKey]) {
                        data = data + `<span style="color: red">删除</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!removeLogMap[objKey].名称 && _.isPlainObject(removeLogMap[objKey]) || removeLogMap[objKey] instanceof Array) {
                    //     removeLogMap[objKey] = JSON.stringify(removeLogMap[objKey])
                    // }
                    if (removeLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = '';
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };

        let addLogMap = {};
        let editLogMap = {};
        let removeLogMap = {};

        let walkJsonPath = '';
        let splitWalk = [];
        let splitWalkLength = 0;


        // let wholeLog = {
        //     user: {
        //         "id": "22222",
        //         "name": "张三"
        //     },
        //     time: "2020-07-06T09:57:40+08:00",
        //     model: {"id": "5c934d3b839a5547624eaebf", "name": "采油井#"},
        //     oldData: {
        //         "computed": {
        //             "tags": [
        //                 {
        //                     "id": "YXZT",
        //                     "mapping": {
        //                         "id": "YXZT",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "运行状态",
        //                     "policy": "save",
        //                     "type": "mapping",
        //                 },
        //                 {
        //                     "id": "YW",
        //                     "mapping": {
        //                         "id": "YW",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "油温",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //                 {
        //                     "id": "YW2",
        //                     "mapping": {
        //                         "id": "YW2",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "油温2",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //             ]
        //         },
        //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站"},
        //         "custom": {"record": "测试记录"},
        //         "warning": {
        //             "rules": [
        //                 {
        //                     "alert": true,
        //                     "description": "注气焖井油温>200",
        //                     "disable": true,
        //                     "extraTags": [
        //                         {
        //                             "id": "lat"
        //                         }
        //                     ],
        //                     "handle": false,
        //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
        //                     "interval": 30,
        //                     "level": "中",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "==": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     200
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
        //                     "warningname": "注气报警"
        //                 },
        //                 {
        //                     "alert": true,
        //                     "description": "非注气状态报警",
        //                     "disable": true,
        //                     "handle": true,
        //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "!=": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     180
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "超限报警",
        //                     "warningname": "非注气状态"
        //                 }
        //             ]
        //         }
        //     },
        //     data: {
        //         "computed": {
        //             "tags": [
        //                 {
        //                     "id": "YXZT",
        //                     "mapping": {
        //                         "id": "YXZT",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "运行状态1",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //                 {
        //                     "id": "IA",
        //                     "mapping": {
        //                         "id": "IA",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "A相电流",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 }
        //             ],
        //         },
        //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站1"},
        //         "custom": {"record": "测试记录1", "select-4F10": "测试选择器111"},
        //         "warning": {
        //             "rules": [
        //                 {
        //                     "alert": true,
        //                     "description": "注气焖井油温>200",
        //                     "disable": true,
        //                     "extraTags": [
        //                         {
        //                             "id": "lat"
        //                         }
        //                     ],
        //                     "handle": false,
        //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
        //                     "interval": 30,
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "==": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     200
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
        //                     "warningname": "注气报警"
        //                 },
        //                 {
        //                     "alert": true,
        //                     "description": "非注气状态报警",
        //                     "disable": true,
        //                     "handle": true,
        //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "!=": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     180
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "超限报警",
        //                     "warningname": "非注气状态"
        //                 }
        //             ]
        //         }
        //     }
        // };

        let originStr = wholeLog.oldData
        let newStr = wholeLog.data

        let newChineseStr = originToAvailable(newStr, wholeLog, projectID);

        if (Object.keys(newChineseStr).length === 0) {
            console.log("newChineseStr global缓存pending")
            return
        }

        let originChineseStr = originToAvailable(originStr, wholeLog, projectID);

        console.log("newChineseStr:", JSON.stringify(newChineseStr))

        console.log("originChineseStr:", JSON.stringify(originChineseStr))

        //提取出两个json中不同的部分
        let deltChinese = jsondiffpatch.diff(originChineseStr, newChineseStr);

        if (!deltChinese) {
            let updateMap = {'diff': ''}
            try {
                let tokenProject = await _this.token(projectID)
                _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
            } catch (err) {
                console.error('updateLog err:', err)
            }
            return
        }
        console.log("deltChinese:", JSON.stringify(deltChinese))

        recursionMergePath(deltChinese, walkJsonPath)

        // console.log('walkJsonPath Chinese:', walkJsonPath);

        console.log('addLogMap Chinese:', addLogMap);

        console.log('editLogMap Chinese:', editLogMap);

        console.log('removeLogMap Chinese:', removeLogMap);

        console.log('formatDiffToWord:');
        let diffWord = formatDiffToWord(wholeLog, addLogMap, editLogMap, removeLogMap)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };

    async testWarnConfirm(wholeLog, projectID) {
        let _this = this

        //格式化为一句话
        const formatDiffToWord = (wholeLog) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + "进行了报警确认，报警信息为:";
            data = data + "\n"
            if (wholeLog.warning) {
                let fieldString = '';
                if (wholeLog.warning.fields) {
                    wholeLog.warning.fields.forEach(obj => {
                        if (obj.name && (obj.value || obj.value === 0)) {
                            fieldString += fieldString + ' ' + obj.name + ':' + obj.value;
                        }
                    });
                }
                data = data + `报警类型:` + wholeLog.warning.type + ";报警描述:" + wholeLog.warning.desc + ";报警级别:" + wholeLog.warning.level + ";报警数值:(" + fieldString + " );";
                data = data + "\n";
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };


        let diffWord = formatDiffToWord(wholeLog)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };

    async testWarnProcess(wholeLog, projectID) {
        let _this = this

        //格式化为一句话
        const formatDiffToWord = (wholeLog) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + "进行了报警处理，报警信息为:";
            data = data + "\n"
            if (wholeLog.warning) {
                let fieldString = '';
                if (wholeLog.warning.fields) {
                    wholeLog.warning.fields.forEach(obj => {
                        if (obj.name && (obj.value || obj.value === 0)) {
                            fieldString += fieldString + ' ' + obj.name + ':' + obj.value;
                        }
                    });
                }
                data = data + `报警类型:` + wholeLog.warning.type + ";报警描述:" + wholeLog.warning.desc + ";报警级别:" + wholeLog.warning.level + ";报警数值:(" + fieldString + " );";
                data = data + "\n";
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };


        let diffWord = formatDiffToWord(wholeLog)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };

    async testUserSchema(wholeLog, projectID) {
        let _this = this
        //生成addLogMap，editLogMap，removeLogMap的key，val
        const recursionMergePath = (obj, walkJsonPath) => {
            return Object.keys(obj).map(k => {
                // console.log("k:", k, "v:", obj[k])

                if (isNaN(k) && k !== '_t') {
                    let testNumber = k.replace('_', "");
                    if (isNaN(testNumber)) {
                        if (walkJsonPath === '') {
                            walkJsonPath = k
                        } else {
                            walkJsonPath = walkJsonPath + '>' + k
                        }
                    }
                }

                if (_.isPlainObject(obj[k])) {
                    recursionMergePath(obj[k], walkJsonPath)
                }

                if (obj[k] instanceof Array) {
                    switch (obj[k].length) {
                        case 1:
                            addLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                        case 2:
                            editLogMap[walkJsonPath] = obj[k][0] + ">" + obj[k][1];
                            splitWalk = walkJsonPath.split('>');
                            // console.log('splitWalk:',splitWalk)
                            splitWalkLength = splitWalk.length - 1;
                            // console.log('splitWalkLength:',splitWalkLength)
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                    // console.log('splitWalkKey:',k,'splitWalkVal:',splitWalk[k],'k !== splitWalkLength:',k !== splitWalkLength)
                                    // console.log(Object.prototype.toString.call(k))
                                    // console.log(Object.prototype.toString.call(splitWalkLength))
                                }
                            });
                            // console.log('splitwalkJsonPath:', walkJsonPath)
                            break;
                        case 3:
                            removeLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                    }
                }


                // const value = obj[k]
                // let title = properties[k] && properties[k].title
                // // console.log(value,title,properties)
                // if (!title && k === 'settings') title = '驱动设置'
                // if (!title || !properties) return
                // if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                //     const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                //     return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                // }
                // if (_.isPlainObject(value) && k !== 'logic') {
                //     let objProperties = properties && properties[k] && properties[k].properties
                //     if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                //         objProperties = properties && properties[k] && properties[k].items.properties
                //     }
                //     return {[title]: convertObjectKey(value, objProperties)}
                // } else {
                //     return {[title]: value}
                // }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // originToAvailable中的递归方法
        const convertObjectKey = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                let title = properties[k] && properties[k].title
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) return
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertObjectKey(value, objProperties)}
                } else {
                    return {[title]: value}
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // 原始数据转换为可读性数据(转中文map)
        const originToAvailable = (value, model, projectID) => {

            let data = {}
            // const nodeSchema = app.get('models')['Node']
            // const m = convertObjectKey(value, nodeSchema.properties)
            if (value && _.isPlainObject(value)) {
                Object.keys(value).forEach(objKey => {
                    // 资产
                    if (objKey === 'name') {
                        data['名称'] = value.name
                    } else if (objKey === 'password') {
                        data['密码'] = value.password
                    } else if (objKey === 'roles') {
                        let u = []
                        if (value.roles && value.roles.length > 0) {
                            u = value.roles.map(item => item.name).join(',')
                        }
                        data['角色'] = u
                    } else if (objKey === 'department') {
                        let u = []
                        if (value.department && value.department.length > 0) {
                            u = value.department.map(item => item.name).join(',')
                        }
                        data['所属部门'] = u
                    } else if (objKey === 'node') {
                        let u = []
                        if (value.node && value.node.length > 0) {
                            u = value.node.map(item => item.name).join(',')
                        }
                        data['资产'] = u
                    } else if (objKey === 'email') {
                        data['邮箱'] = value.email
                    } else if (objKey === 'remark') {
                        data['备注'] = value.remark
                    } else if (objKey === 'openid') {
                        data['微信用户标识'] = value.openid
                    } else if (objKey === 'dduserid') {
                        data['钉钉用户标识'] = value.dduserid
                    } else if (objKey === 'phone') {
                        data['手机号'] = value.phone
                    } else if (objKey === 'industry') {
                        data['行业配置'] = value.industry
                    } else if (objKey === 'phone') {
                        data['手机号'] = value.phone
                    } else if (objKey === 'permission') {
                        data['权限'] = value.permission
                    } else if (objKey === 'extra') {
                        data['额外属性'] = value.extra
                    } else if (objKey === 'onlyNode') {
                        data['仅管理个人资产'] = value.onlyNode ? '是' : '否'
                    } else if (objKey === 'expireUse') {
                        data['有效时长'] = value.expireUse
                    } else if (objKey === 'dashboard') {
                        data['画面'] = value.dashboard
                    } else if (objKey === 'startTime') {
                        data['有效期起始时间'] = value.dashboard
                    } else if (objKey === 'noChildDept') {
                        data['不查看子部门报警数据'] = value.noChildDept ? '是' : '否'
                    } else if (objKey === 'disabled') {
                        data['禁用'] = value.disabled ? '是' : '否'
                    } else if (objKey === 'binddingtalk') {
                        data['绑定钉钉'] = value.binddingtalk ? '是' : '否'
                    } else if (objKey === 'pageSetting') {
                        data['页配置'] = value.pageSetting
                    } else if (objKey === 'mainmenu') {
                        data['主页菜单配置'] = value.mainmenu
                    } else if (objKey === 'shareUserType') {
                        data['分享画面类型'] = value.shareUserType
                    } else if (objKey === 'timeoutTimeUse') {
                        data['分享画面超时时间'] = value.timeoutTimeUse
                    } else if (objKey === 'warningFilter') {
                        data['报警分组配置'] = value.warningFilter
                    }
                })
            } else {
                data = value
            }

            return data
        };

        //格式化为一句话
        const formatDiffToWord = (wholeLog, addLogMap, editLogMap, removeLogMap) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + ":";
            data = data + "\n"
            if (addLogMap.length !== 0) {
                Object.keys(addLogMap).forEach(objKey => {
                    if (!addLogMap[objKey]) {
                        data = data + `<span style="color: green">添加</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!addLogMap[objKey].名称 && _.isPlainObject(addLogMap[objKey]) || addLogMap[objKey] instanceof Array) {
                    //     addLogMap[objKey] = JSON.stringify(addLogMap[objKey])
                    // }
                    if (addLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = ''
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (editLogMap.length !== 0) {
                Object.keys(editLogMap).forEach(objKey => {
                    if (!editLogMap[objKey]) {
                        return
                    }
                    let lastWordList = editLogMap[objKey].split('>');
                    let lastWord = '';
                    let originWord = '';
                    if (lastWordList.length > 1) {
                        originWord = lastWordList[0]
                        lastWord = lastWordList[lastWordList.length - 1]
                    }
                    data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ");"
                    data = data + "\n"
                })
            }
            if (removeLogMap.length !== 0) {
                Object.keys(removeLogMap).forEach(objKey => {
                    if (!removeLogMap[objKey]) {
                        data = data + `<span style="color: red">删除</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!removeLogMap[objKey].名称 && _.isPlainObject(removeLogMap[objKey]) || removeLogMap[objKey] instanceof Array) {
                    //     removeLogMap[objKey] = JSON.stringify(removeLogMap[objKey])
                    // }
                    if (removeLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = '';
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };

        let addLogMap = {};
        let editLogMap = {};
        let removeLogMap = {};

        let walkJsonPath = '';
        let splitWalk = [];
        let splitWalkLength = 0;


        // let wholeLog = {
        //     user: {
        //         "id": "22222",
        //         "name": "张三"
        //     },
        //     time: "2020-07-06T09:57:40+08:00",
        //     model: {"id": "5c934d3b839a5547624eaebf", "name": "采油井#"},
        //     oldData: {
        //         "computed": {
        //             "tags": [
        //                 {
        //                     "id": "YXZT",
        //                     "mapping": {
        //                         "id": "YXZT",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "运行状态",
        //                     "policy": "save",
        //                     "type": "mapping",
        //                 },
        //                 {
        //                     "id": "YW",
        //                     "mapping": {
        //                         "id": "YW",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "油温",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //                 {
        //                     "id": "YW2",
        //                     "mapping": {
        //                         "id": "YW2",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "油温2",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //             ]
        //         },
        //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站"},
        //         "custom": {"record": "测试记录"},
        //         "warning": {
        //             "rules": [
        //                 {
        //                     "alert": true,
        //                     "description": "注气焖井油温>200",
        //                     "disable": true,
        //                     "extraTags": [
        //                         {
        //                             "id": "lat"
        //                         }
        //                     ],
        //                     "handle": false,
        //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
        //                     "interval": 30,
        //                     "level": "中",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "==": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     200
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
        //                     "warningname": "注气报警"
        //                 },
        //                 {
        //                     "alert": true,
        //                     "description": "非注气状态报警",
        //                     "disable": true,
        //                     "handle": true,
        //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "!=": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     180
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "超限报警",
        //                     "warningname": "非注气状态"
        //                 }
        //             ]
        //         }
        //     },
        //     data: {
        //         "computed": {
        //             "tags": [
        //                 {
        //                     "id": "YXZT",
        //                     "mapping": {
        //                         "id": "YXZT",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "运行状态1",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //                 {
        //                     "id": "IA",
        //                     "mapping": {
        //                         "id": "IA",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "A相电流",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 }
        //             ],
        //         },
        //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站1"},
        //         "custom": {"record": "测试记录1", "select-4F10": "测试选择器111"},
        //         "warning": {
        //             "rules": [
        //                 {
        //                     "alert": true,
        //                     "description": "注气焖井油温>200",
        //                     "disable": true,
        //                     "extraTags": [
        //                         {
        //                             "id": "lat"
        //                         }
        //                     ],
        //                     "handle": false,
        //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
        //                     "interval": 30,
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "==": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     200
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
        //                     "warningname": "注气报警"
        //                 },
        //                 {
        //                     "alert": true,
        //                     "description": "非注气状态报警",
        //                     "disable": true,
        //                     "handle": true,
        //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "!=": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     180
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "超限报警",
        //                     "warningname": "非注气状态"
        //                 }
        //             ]
        //         }
        //     }
        // };

        let originStr = wholeLog.oldData
        let newStr = wholeLog.data

        let newChineseStr = originToAvailable(newStr, wholeLog, projectID);

        if (Object.keys(newChineseStr).length === 0) {
            console.log("newChineseStr global缓存pending")
            return
        }

        let originChineseStr = originToAvailable(originStr, wholeLog, projectID);

        console.log("newChineseStr:", JSON.stringify(newChineseStr))

        console.log("originChineseStr:", JSON.stringify(originChineseStr))

        //提取出两个json中不同的部分
        let deltChinese = jsondiffpatch.diff(originChineseStr, newChineseStr);

        if (!deltChinese) {
            let updateMap = {'diff': ''}
            try {
                let tokenProject = await _this.token(projectID)
                _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
            } catch (err) {
                console.error('updateLog err:', err)
            }
            return
        }
        console.log("deltChinese:", JSON.stringify(deltChinese))

        recursionMergePath(deltChinese, walkJsonPath)

        // console.log('walkJsonPath Chinese:', walkJsonPath);

        console.log('addLogMap Chinese:', addLogMap);

        console.log('editLogMap Chinese:', editLogMap);

        console.log('removeLogMap Chinese:', removeLogMap);

        console.log('formatDiffToWord:');
        let diffWord = formatDiffToWord(wholeLog, addLogMap, editLogMap, removeLogMap)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };

    async testOauthAppSchema(wholeLog, projectID) {
        let _this = this
        //生成addLogMap，editLogMap，removeLogMap的key，val
        const recursionMergePath = (obj, walkJsonPath) => {
            return Object.keys(obj).map(k => {
                // console.log("k:", k, "v:", obj[k])

                if (isNaN(k) && k !== '_t') {
                    let testNumber = k.replace('_', "");
                    if (isNaN(testNumber)) {
                        if (walkJsonPath === '') {
                            walkJsonPath = k
                        } else {
                            walkJsonPath = walkJsonPath + '>' + k
                        }
                    }
                }

                if (_.isPlainObject(obj[k])) {
                    recursionMergePath(obj[k], walkJsonPath)
                }

                if (obj[k] instanceof Array) {
                    switch (obj[k].length) {
                        case 1:
                            addLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                        case 2:
                            editLogMap[walkJsonPath] = obj[k][0] + ">" + obj[k][1];
                            splitWalk = walkJsonPath.split('>');
                            // console.log('splitWalk:',splitWalk)
                            splitWalkLength = splitWalk.length - 1;
                            // console.log('splitWalkLength:',splitWalkLength)
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                    // console.log('splitWalkKey:',k,'splitWalkVal:',splitWalk[k],'k !== splitWalkLength:',k !== splitWalkLength)
                                    // console.log(Object.prototype.toString.call(k))
                                    // console.log(Object.prototype.toString.call(splitWalkLength))
                                }
                            });
                            // console.log('splitwalkJsonPath:', walkJsonPath)
                            break;
                        case 3:
                            removeLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                    }
                }


                // const value = obj[k]
                // let title = properties[k] && properties[k].title
                // // console.log(value,title,properties)
                // if (!title && k === 'settings') title = '驱动设置'
                // if (!title || !properties) return
                // if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                //     const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                //     return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                // }
                // if (_.isPlainObject(value) && k !== 'logic') {
                //     let objProperties = properties && properties[k] && properties[k].properties
                //     if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                //         objProperties = properties && properties[k] && properties[k].items.properties
                //     }
                //     return {[title]: convertObjectKey(value, objProperties)}
                // } else {
                //     return {[title]: value}
                // }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // originToAvailable中的递归方法
        const convertObjectKey = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                let title = properties[k] && properties[k].title
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) return
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertObjectKey(value, objProperties)}
                } else {
                    return {[title]: value}
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // 原始数据转换为可读性数据(转中文map)
        const originToAvailable = (value, model, projectID) => {

            let data = {}
            // const nodeSchema = app.get('models')['Node']
            // const m = convertObjectKey(value, nodeSchema.properties)
            if (value && _.isPlainObject(value)) {
                Object.keys(value).forEach(objKey => {
                    // 资产
                    if (objKey === 'name') {
                        data['名称'] = value.name
                    } else if (objKey === 'clientid') {
                        data['客户端id'] = value.clientid
                    } else if (objKey === 'clientsecret') {
                        data['客户端密钥'] = value.clientsecret
                    } else if (objKey === 'domain') {
                        data['回调接⼝的url的前缀'] = value.domain
                    } else if (objKey === 'redirect_uri') {
                        data['回调接⼝'] = value.redirect_uri
                    } else if (objKey === 'platform_uri') {
                        data['平台地址'] = value.platform_uri
                    }
                })
            } else {
                data = value
            }

            return data
        };

        //格式化为一句话
        const formatDiffToWord = (wholeLog, addLogMap, editLogMap, removeLogMap) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + ":";
            data = data + "\n"
            if (addLogMap.length !== 0) {
                Object.keys(addLogMap).forEach(objKey => {
                    if (!addLogMap[objKey]) {
                        data = data + `<span style="color: green">添加</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!addLogMap[objKey].名称 && _.isPlainObject(addLogMap[objKey]) || addLogMap[objKey] instanceof Array) {
                    //     addLogMap[objKey] = JSON.stringify(addLogMap[objKey])
                    // }
                    if (addLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = ''
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (editLogMap.length !== 0) {
                Object.keys(editLogMap).forEach(objKey => {
                    if (!editLogMap[objKey]) {
                        return
                    }
                    let lastWordList = editLogMap[objKey].split('>');
                    let lastWord = '';
                    let originWord = '';
                    if (lastWordList.length > 1) {
                        originWord = lastWordList[0]
                        lastWord = lastWordList[lastWordList.length - 1]
                    }
                    data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ");"
                    data = data + "\n"
                })
            }
            if (removeLogMap.length !== 0) {
                Object.keys(removeLogMap).forEach(objKey => {
                    if (!removeLogMap[objKey]) {
                        data = data + `<span style="color: red">删除</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!removeLogMap[objKey].名称 && _.isPlainObject(removeLogMap[objKey]) || removeLogMap[objKey] instanceof Array) {
                    //     removeLogMap[objKey] = JSON.stringify(removeLogMap[objKey])
                    // }
                    if (removeLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = '';
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };

        let addLogMap = {};
        let editLogMap = {};
        let removeLogMap = {};

        let walkJsonPath = '';
        let splitWalk = [];
        let splitWalkLength = 0;


        // let wholeLog = {
        //     user: {
        //         "id": "22222",
        //         "name": "张三"
        //     },
        //     time: "2020-07-06T09:57:40+08:00",
        //     model: {"id": "5c934d3b839a5547624eaebf", "name": "采油井#"},
        //     oldData: {
        //         "computed": {
        //             "tags": [
        //                 {
        //                     "id": "YXZT",
        //                     "mapping": {
        //                         "id": "YXZT",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "运行状态",
        //                     "policy": "save",
        //                     "type": "mapping",
        //                 },
        //                 {
        //                     "id": "YW",
        //                     "mapping": {
        //                         "id": "YW",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "油温",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //                 {
        //                     "id": "YW2",
        //                     "mapping": {
        //                         "id": "YW2",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "油温2",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //             ]
        //         },
        //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站"},
        //         "custom": {"record": "测试记录"},
        //         "warning": {
        //             "rules": [
        //                 {
        //                     "alert": true,
        //                     "description": "注气焖井油温>200",
        //                     "disable": true,
        //                     "extraTags": [
        //                         {
        //                             "id": "lat"
        //                         }
        //                     ],
        //                     "handle": false,
        //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
        //                     "interval": 30,
        //                     "level": "中",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "==": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     200
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
        //                     "warningname": "注气报警"
        //                 },
        //                 {
        //                     "alert": true,
        //                     "description": "非注气状态报警",
        //                     "disable": true,
        //                     "handle": true,
        //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "!=": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     180
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "超限报警",
        //                     "warningname": "非注气状态"
        //                 }
        //             ]
        //         }
        //     },
        //     data: {
        //         "computed": {
        //             "tags": [
        //                 {
        //                     "id": "YXZT",
        //                     "mapping": {
        //                         "id": "YXZT",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "运行状态1",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //                 {
        //                     "id": "IA",
        //                     "mapping": {
        //                         "id": "IA",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "A相电流",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 }
        //             ],
        //         },
        //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站1"},
        //         "custom": {"record": "测试记录1", "select-4F10": "测试选择器111"},
        //         "warning": {
        //             "rules": [
        //                 {
        //                     "alert": true,
        //                     "description": "注气焖井油温>200",
        //                     "disable": true,
        //                     "extraTags": [
        //                         {
        //                             "id": "lat"
        //                         }
        //                     ],
        //                     "handle": false,
        //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
        //                     "interval": 30,
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "==": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     200
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
        //                     "warningname": "注气报警"
        //                 },
        //                 {
        //                     "alert": true,
        //                     "description": "非注气状态报警",
        //                     "disable": true,
        //                     "handle": true,
        //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "!=": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     180
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "超限报警",
        //                     "warningname": "非注气状态"
        //                 }
        //             ]
        //         }
        //     }
        // };

        let originStr = wholeLog.oldData
        let newStr = wholeLog.data

        let newChineseStr = originToAvailable(newStr, wholeLog, projectID);

        if (Object.keys(newChineseStr).length === 0) {
            console.log("newChineseStr global缓存pending")
            return
        }

        let originChineseStr = originToAvailable(originStr, wholeLog, projectID);

        console.log("newChineseStr:", JSON.stringify(newChineseStr))

        console.log("originChineseStr:", JSON.stringify(originChineseStr))

        //提取出两个json中不同的部分
        let deltChinese = jsondiffpatch.diff(originChineseStr, newChineseStr);

        if (!deltChinese) {
            let updateMap = {'diff': ''}
            try {
                let tokenProject = await _this.token(projectID)
                _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
            } catch (err) {
                console.error('updateLog err:', err)
            }
            return
        }
        console.log("deltChinese:", JSON.stringify(deltChinese))

        recursionMergePath(deltChinese, walkJsonPath)

        // console.log('walkJsonPath Chinese:', walkJsonPath);

        console.log('addLogMap Chinese:', addLogMap);

        console.log('editLogMap Chinese:', editLogMap);

        console.log('removeLogMap Chinese:', removeLogMap);

        console.log('formatDiffToWord:');
        let diffWord = formatDiffToWord(wholeLog, addLogMap, editLogMap, removeLogMap)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };

    async testDeptSchema(wholeLog, projectID) {
        let _this = this
        //生成addLogMap，editLogMap，removeLogMap的key，val
        const recursionMergePath = (obj, walkJsonPath) => {
            return Object.keys(obj).map(k => {
                // console.log("k:", k, "v:", obj[k])

                if (isNaN(k) && k !== '_t') {
                    let testNumber = k.replace('_', "");
                    if (isNaN(testNumber)) {
                        if (walkJsonPath === '') {
                            walkJsonPath = k
                        } else {
                            walkJsonPath = walkJsonPath + '>' + k
                        }
                    }
                }

                if (_.isPlainObject(obj[k])) {
                    recursionMergePath(obj[k], walkJsonPath)
                }

                if (obj[k] instanceof Array) {
                    switch (obj[k].length) {
                        case 1:
                            addLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                        case 2:
                            editLogMap[walkJsonPath] = obj[k][0] + ">" + obj[k][1];
                            splitWalk = walkJsonPath.split('>');
                            // console.log('splitWalk:',splitWalk)
                            splitWalkLength = splitWalk.length - 1;
                            // console.log('splitWalkLength:',splitWalkLength)
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                    // console.log('splitWalkKey:',k,'splitWalkVal:',splitWalk[k],'k !== splitWalkLength:',k !== splitWalkLength)
                                    // console.log(Object.prototype.toString.call(k))
                                    // console.log(Object.prototype.toString.call(splitWalkLength))
                                }
                            });
                            // console.log('splitwalkJsonPath:', walkJsonPath)
                            break;
                        case 3:
                            removeLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                    }
                }


                // const value = obj[k]
                // let title = properties[k] && properties[k].title
                // // console.log(value,title,properties)
                // if (!title && k === 'settings') title = '驱动设置'
                // if (!title || !properties) return
                // if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                //     const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                //     return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                // }
                // if (_.isPlainObject(value) && k !== 'logic') {
                //     let objProperties = properties && properties[k] && properties[k].properties
                //     if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                //         objProperties = properties && properties[k] && properties[k].items.properties
                //     }
                //     return {[title]: convertObjectKey(value, objProperties)}
                // } else {
                //     return {[title]: value}
                // }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // originToAvailable中的递归方法
        const convertObjectKey = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                let title = properties[k] && properties[k].title
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) return
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertObjectKey(value, objProperties)}
                } else {
                    return {[title]: value}
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // 原始数据转换为可读性数据(转中文map)
        const originToAvailable = (value, model, projectID) => {

            let data = {}
            // const nodeSchema = app.get('models')['Node']
            // const m = convertObjectKey(value, nodeSchema.properties)
            if (value && _.isPlainObject(value)) {
                Object.keys(value).forEach(objKey => {
                    // 资产
                    if (objKey === 'name') {
                        data['名称'] = value.name
                    } else if (objKey === 'fullName') {
                        data['部门全名'] = value.fullName
                    } else if (objKey === 'node') {
                        let u = []
                        if (value.node && value.node.length > 0) {
                            u = value.node.map(item => item.name).join(',')
                        }
                        data['资产'] = u
                    } else if (objKey === 'users') {
                        let u = []
                        if (value.users && value.users.length > 0) {
                            u = value.users.map(item => item.name).join(',')
                        }
                        data['用户'] = u
                    } else if (objKey === 'parent') {
                        if (value.parent && value.parent.name) {
                            data['所属部门'] = value.parent.name
                        }
                    } else if (objKey === 'remark') {
                        data['备注'] = value.remark
                    }
                })
            } else {
                data = value
            }

            return data
        };

        //格式化为一句话
        const formatDiffToWord = (wholeLog, addLogMap, editLogMap, removeLogMap) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + ":";
            data = data + "\n"
            if (addLogMap.length !== 0) {
                Object.keys(addLogMap).forEach(objKey => {
                    if (!addLogMap[objKey]) {
                        data = data + `<span style="color: green">添加</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!addLogMap[objKey].名称 && _.isPlainObject(addLogMap[objKey]) || addLogMap[objKey] instanceof Array) {
                    //     addLogMap[objKey] = JSON.stringify(addLogMap[objKey])
                    // }
                    if (addLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = ''
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (editLogMap.length !== 0) {
                Object.keys(editLogMap).forEach(objKey => {
                    if (!editLogMap[objKey]) {
                        return
                    }
                    let lastWordList = editLogMap[objKey].split('>');
                    let lastWord = '';
                    let originWord = '';
                    if (lastWordList.length > 1) {
                        originWord = lastWordList[0]
                        lastWord = lastWordList[lastWordList.length - 1]
                    }
                    data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ");"
                    data = data + "\n"
                })
            }
            if (removeLogMap.length !== 0) {
                Object.keys(removeLogMap).forEach(objKey => {
                    if (!removeLogMap[objKey]) {
                        data = data + `<span style="color: red">删除</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!removeLogMap[objKey].名称 && _.isPlainObject(removeLogMap[objKey]) || removeLogMap[objKey] instanceof Array) {
                    //     removeLogMap[objKey] = JSON.stringify(removeLogMap[objKey])
                    // }
                    if (removeLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = '';
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };

        let addLogMap = {};
        let editLogMap = {};
        let removeLogMap = {};

        let walkJsonPath = '';
        let splitWalk = [];
        let splitWalkLength = 0;


        let originStr = wholeLog.oldData
        let newStr = wholeLog.data

        let newChineseStr = originToAvailable(newStr, wholeLog, projectID);

        if (Object.keys(newChineseStr).length === 0) {
            console.log("newChineseStr global缓存pending")
            return
        }

        let originChineseStr = originToAvailable(originStr, wholeLog, projectID);

        console.log("newChineseStr:", JSON.stringify(newChineseStr))

        console.log("originChineseStr:", JSON.stringify(originChineseStr))

        //提取出两个json中不同的部分
        let deltChinese = jsondiffpatch.diff(originChineseStr, newChineseStr);

        if (!deltChinese) {
            let updateMap = {'diff': ''}
            try {
                let tokenProject = await _this.token(projectID)
                _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
            } catch (err) {
                console.error('updateLog err:', err)
            }
            return
        }
        console.log("deltChinese:", JSON.stringify(deltChinese))

        recursionMergePath(deltChinese, walkJsonPath)

        // console.log('walkJsonPath Chinese:', walkJsonPath);

        console.log('addLogMap Chinese:', addLogMap);

        console.log('editLogMap Chinese:', editLogMap);

        console.log('removeLogMap Chinese:', removeLogMap);

        console.log('formatDiffToWord:');
        let diffWord = formatDiffToWord(wholeLog, addLogMap, editLogMap, removeLogMap)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };

    async testSyncSchema(wholeLog, projectID) {
        let _this = this
        //生成addLogMap，editLogMap，removeLogMap的key，val
        const recursionMergePath = (obj, walkJsonPath) => {
            return Object.keys(obj).map(k => {
                // console.log("k:", k, "v:", obj[k])

                if (isNaN(k) && k !== '_t') {
                    let testNumber = k.replace('_', "");
                    if (isNaN(testNumber)) {
                        if (walkJsonPath === '') {
                            walkJsonPath = k
                        } else {
                            walkJsonPath = walkJsonPath + '>' + k
                        }
                    }
                }

                if (_.isPlainObject(obj[k])) {
                    recursionMergePath(obj[k], walkJsonPath)
                }

                if (obj[k] instanceof Array) {
                    switch (obj[k].length) {
                        case 1:
                            addLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                        case 2:
                            editLogMap[walkJsonPath] = obj[k][0] + ">" + obj[k][1];
                            splitWalk = walkJsonPath.split('>');
                            // console.log('splitWalk:',splitWalk)
                            splitWalkLength = splitWalk.length - 1;
                            // console.log('splitWalkLength:',splitWalkLength)
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                    // console.log('splitWalkKey:',k,'splitWalkVal:',splitWalk[k],'k !== splitWalkLength:',k !== splitWalkLength)
                                    // console.log(Object.prototype.toString.call(k))
                                    // console.log(Object.prototype.toString.call(splitWalkLength))
                                }
                            });
                            // console.log('splitwalkJsonPath:', walkJsonPath)
                            break;
                        case 3:
                            removeLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                    }
                }


                // const value = obj[k]
                // let title = properties[k] && properties[k].title
                // // console.log(value,title,properties)
                // if (!title && k === 'settings') title = '驱动设置'
                // if (!title || !properties) return
                // if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                //     const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                //     return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                // }
                // if (_.isPlainObject(value) && k !== 'logic') {
                //     let objProperties = properties && properties[k] && properties[k].properties
                //     if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                //         objProperties = properties && properties[k] && properties[k].items.properties
                //     }
                //     return {[title]: convertObjectKey(value, objProperties)}
                // } else {
                //     return {[title]: value}
                // }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // originToAvailable中的递归方法
        const convertObjectKey = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                let title = properties[k] && properties[k].title
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) return
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertObjectKey(value, objProperties)}
                } else {
                    return {[title]: value}
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // 原始数据转换为可读性数据(转中文map)
        const originToAvailable = (value, model, projectID) => {

            let data = {}
            // const nodeSchema = app.get('models')['Node']
            // const m = convertObjectKey(value, nodeSchema.properties)
            if (value && _.isPlainObject(value)) {
                Object.keys(value).forEach(objKey => {
                    // 资产
                    if (objKey === 'name') {
                        data['名称'] = value.name
                    } else if (objKey === 'settings') {
                        if (value.settings) {
                            data['系统配置'] = value.settings
                        }
                    }
                })
            } else {
                data = value
            }

            return data
        };

        //格式化为一句话
        const formatDiffToWord = (wholeLog, addLogMap, editLogMap, removeLogMap) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + ":";
            data = data + "\n"
            if (addLogMap.length !== 0) {
                Object.keys(addLogMap).forEach(objKey => {
                    if (!addLogMap[objKey]) {
                        data = data + `<span style="color: green">添加</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!addLogMap[objKey].名称 && _.isPlainObject(addLogMap[objKey]) || addLogMap[objKey] instanceof Array) {
                    //     addLogMap[objKey] = JSON.stringify(addLogMap[objKey])
                    // }
                    if (addLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = ''
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (editLogMap.length !== 0) {
                Object.keys(editLogMap).forEach(objKey => {
                    if (!editLogMap[objKey]) {
                        return
                    }
                    let lastWordList = editLogMap[objKey].split('>');
                    let lastWord = '';
                    let originWord = '';
                    if (lastWordList.length > 1) {
                        originWord = lastWordList[0]
                        lastWord = lastWordList[lastWordList.length - 1]
                    }
                    data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ");"
                    data = data + "\n"
                })
            }
            if (removeLogMap.length !== 0) {
                Object.keys(removeLogMap).forEach(objKey => {
                    if (!removeLogMap[objKey]) {
                        data = data + `<span style="color: red">删除</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!removeLogMap[objKey].名称 && _.isPlainObject(removeLogMap[objKey]) || removeLogMap[objKey] instanceof Array) {
                    //     removeLogMap[objKey] = JSON.stringify(removeLogMap[objKey])
                    // }
                    if (removeLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = '';
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };

        let addLogMap = {};
        let editLogMap = {};
        let removeLogMap = {};

        let walkJsonPath = '';
        let splitWalk = [];
        let splitWalkLength = 0;


        let originStr = wholeLog.oldData
        let newStr = wholeLog.data

        let newChineseStr = originToAvailable(newStr, wholeLog, projectID);

        if (Object.keys(newChineseStr).length === 0) {
            console.log("newChineseStr global缓存pending")
            return
        }

        let originChineseStr = originToAvailable(originStr, wholeLog, projectID);

        console.log("newChineseStr:", JSON.stringify(newChineseStr))

        console.log("originChineseStr:", JSON.stringify(originChineseStr))

        //提取出两个json中不同的部分
        let deltChinese = jsondiffpatch.diff(originChineseStr, newChineseStr);

        if (!deltChinese) {
            let updateMap = {'diff': ''}
            try {
                let tokenProject = await _this.token(projectID)
                _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
            } catch (err) {
                console.error('updateLog err:', err)
            }
            return
        }
        console.log("deltChinese:", JSON.stringify(deltChinese))

        recursionMergePath(deltChinese, walkJsonPath)

        // console.log('walkJsonPath Chinese:', walkJsonPath);

        console.log('addLogMap Chinese:', addLogMap);

        console.log('editLogMap Chinese:', editLogMap);

        console.log('removeLogMap Chinese:', removeLogMap);

        console.log('formatDiffToWord:');
        let diffWord = formatDiffToWord(wholeLog, addLogMap, editLogMap, removeLogMap)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };

    async testWarningArchiveSchema(wholeLog, projectID) {
        let _this = this
        //生成addLogMap，editLogMap，removeLogMap的key，val
        const recursionMergePath = (obj, walkJsonPath) => {
            return Object.keys(obj).map(k => {
                // console.log("k:", k, "v:", obj[k])

                if (isNaN(k) && k !== '_t') {
                    let testNumber = k.replace('_', "");
                    if (isNaN(testNumber)) {
                        if (walkJsonPath === '') {
                            walkJsonPath = k
                        } else {
                            walkJsonPath = walkJsonPath + '>' + k
                        }
                    }
                }

                if (_.isPlainObject(obj[k])) {
                    recursionMergePath(obj[k], walkJsonPath)
                }

                if (obj[k] instanceof Array) {
                    switch (obj[k].length) {
                        case 1:
                            addLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                        case 2:
                            editLogMap[walkJsonPath] = obj[k][0] + ">" + obj[k][1];
                            splitWalk = walkJsonPath.split('>');
                            // console.log('splitWalk:',splitWalk)
                            splitWalkLength = splitWalk.length - 1;
                            // console.log('splitWalkLength:',splitWalkLength)
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                    // console.log('splitWalkKey:',k,'splitWalkVal:',splitWalk[k],'k !== splitWalkLength:',k !== splitWalkLength)
                                    // console.log(Object.prototype.toString.call(k))
                                    // console.log(Object.prototype.toString.call(splitWalkLength))
                                }
                            });
                            // console.log('splitwalkJsonPath:', walkJsonPath)
                            break;
                        case 3:
                            removeLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                    }
                }


                // const value = obj[k]
                // let title = properties[k] && properties[k].title
                // // console.log(value,title,properties)
                // if (!title && k === 'settings') title = '驱动设置'
                // if (!title || !properties) return
                // if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                //     const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                //     return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                // }
                // if (_.isPlainObject(value) && k !== 'logic') {
                //     let objProperties = properties && properties[k] && properties[k].properties
                //     if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                //         objProperties = properties && properties[k] && properties[k].items.properties
                //     }
                //     return {[title]: convertObjectKey(value, objProperties)}
                // } else {
                //     return {[title]: value}
                // }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // originToAvailable中的递归方法
        const convertObjectKey = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                let title = properties[k] && properties[k].title
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) return
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertObjectKey(value, objProperties)}
                } else {
                    return {[title]: value}
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // 原始数据转换为可读性数据(转中文map)
        const originToAvailable = (value, model, projectID) => {

            let data = {}
            // const nodeSchema = app.get('models')['Node']
            // const m = convertObjectKey(value, nodeSchema.properties)
            if (value && _.isPlainObject(value)) {
                Object.keys(value).forEach(objKey => {
                    // 资产
                    if (objKey === 'name') {
                        data['名称'] = value.name
                    } else if (objKey === 'range') {
                        data['归档范围类型'] = value.range
                    } else if (objKey === 'type') {
                        let u = []
                        if (value.type && value.type.length > 0) {
                            u = value.type.map(item => item.name).join(',')
                        }
                        data['归档报警类型'] = u
                    } else if (objKey === 'interval') {
                        data['归档周期'] = value.interval
                    } else if (objKey === 'disable') {
                        data['禁用'] = value.disable
                    } else if (objKey === 'status') {
                        data['确认状态'] = value.status
                    } else if (objKey === 'processed') {
                        data['处理状态'] = value.processed
                    } else if (objKey === 'department') {
                        let u = []
                        if (value.department && value.department.length > 0) {
                            u = value.department.map(item => item.name).join(',')
                        }
                        data['归档报警部门'] = u
                    } else if (objKey === 'model') {
                        let u = []
                        if (value.model && value.model.length > 0) {
                            u = value.model.map(item => item.name).join(',')
                        }
                        data['归档报警模型'] = u
                    }
                })
            } else {
                data = value
            }

            return data
        };

        //格式化为一句话
        const formatDiffToWord = (wholeLog, addLogMap, editLogMap, removeLogMap) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + ":";
            data = data + "\n"
            if (addLogMap.length !== 0) {
                Object.keys(addLogMap).forEach(objKey => {
                    if (!addLogMap[objKey]) {
                        data = data + `<span style="color: green">添加</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!addLogMap[objKey].名称 && _.isPlainObject(addLogMap[objKey]) || addLogMap[objKey] instanceof Array) {
                    //     addLogMap[objKey] = JSON.stringify(addLogMap[objKey])
                    // }
                    if (addLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = ''
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (editLogMap.length !== 0) {
                Object.keys(editLogMap).forEach(objKey => {
                    if (!editLogMap[objKey]) {
                        return
                    }
                    let lastWordList = editLogMap[objKey].split('>');
                    let lastWord = '';
                    let originWord = '';
                    if (lastWordList.length > 1) {
                        originWord = lastWordList[0]
                        lastWord = lastWordList[lastWordList.length - 1]
                    }
                    data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ");"
                    data = data + "\n"
                })
            }
            if (removeLogMap.length !== 0) {
                Object.keys(removeLogMap).forEach(objKey => {
                    if (!removeLogMap[objKey]) {
                        data = data + `<span style="color: red">删除</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!removeLogMap[objKey].名称 && _.isPlainObject(removeLogMap[objKey]) || removeLogMap[objKey] instanceof Array) {
                    //     removeLogMap[objKey] = JSON.stringify(removeLogMap[objKey])
                    // }
                    if (removeLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = '';
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };

        let addLogMap = {};
        let editLogMap = {};
        let removeLogMap = {};

        let walkJsonPath = '';
        let splitWalk = [];
        let splitWalkLength = 0;


        // let wholeLog = {
        //     user: {
        //         "id": "22222",
        //         "name": "张三"
        //     },
        //     time: "2020-07-06T09:57:40+08:00",
        //     model: {"id": "5c934d3b839a5547624eaebf", "name": "采油井#"},
        //     oldData: {
        //         "computed": {
        //             "tags": [
        //                 {
        //                     "id": "YXZT",
        //                     "mapping": {
        //                         "id": "YXZT",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "运行状态",
        //                     "policy": "save",
        //                     "type": "mapping",
        //                 },
        //                 {
        //                     "id": "YW",
        //                     "mapping": {
        //                         "id": "YW",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "油温",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //                 {
        //                     "id": "YW2",
        //                     "mapping": {
        //                         "id": "YW2",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "油温2",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //             ]
        //         },
        //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站"},
        //         "custom": {"record": "测试记录"},
        //         "warning": {
        //             "rules": [
        //                 {
        //                     "alert": true,
        //                     "description": "注气焖井油温>200",
        //                     "disable": true,
        //                     "extraTags": [
        //                         {
        //                             "id": "lat"
        //                         }
        //                     ],
        //                     "handle": false,
        //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
        //                     "interval": 30,
        //                     "level": "中",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "==": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     200
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
        //                     "warningname": "注气报警"
        //                 },
        //                 {
        //                     "alert": true,
        //                     "description": "非注气状态报警",
        //                     "disable": true,
        //                     "handle": true,
        //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "!=": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     180
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "超限报警",
        //                     "warningname": "非注气状态"
        //                 }
        //             ]
        //         }
        //     },
        //     data: {
        //         "computed": {
        //             "tags": [
        //                 {
        //                     "id": "YXZT",
        //                     "mapping": {
        //                         "id": "YXZT",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "运行状态1",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //                 {
        //                     "id": "IA",
        //                     "mapping": {
        //                         "id": "IA",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "A相电流",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 }
        //             ],
        //         },
        //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站1"},
        //         "custom": {"record": "测试记录1", "select-4F10": "测试选择器111"},
        //         "warning": {
        //             "rules": [
        //                 {
        //                     "alert": true,
        //                     "description": "注气焖井油温>200",
        //                     "disable": true,
        //                     "extraTags": [
        //                         {
        //                             "id": "lat"
        //                         }
        //                     ],
        //                     "handle": false,
        //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
        //                     "interval": 30,
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "==": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     200
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
        //                     "warningname": "注气报警"
        //                 },
        //                 {
        //                     "alert": true,
        //                     "description": "非注气状态报警",
        //                     "disable": true,
        //                     "handle": true,
        //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "!=": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     180
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "超限报警",
        //                     "warningname": "非注气状态"
        //                 }
        //             ]
        //         }
        //     }
        // };

        let originStr = wholeLog.oldData
        let newStr = wholeLog.data

        let newChineseStr = originToAvailable(newStr, wholeLog, projectID);

        if (Object.keys(newChineseStr).length === 0) {
            console.log("newChineseStr global缓存pending")
            return
        }

        let originChineseStr = originToAvailable(originStr, wholeLog, projectID);

        console.log("newChineseStr:", JSON.stringify(newChineseStr))

        console.log("originChineseStr:", JSON.stringify(originChineseStr))

        //提取出两个json中不同的部分
        let deltChinese = jsondiffpatch.diff(originChineseStr, newChineseStr);

        if (!deltChinese) {
            let updateMap = {'diff': ''}
            try {
                let tokenProject = await _this.token(projectID)
                _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
            } catch (err) {
                console.error('updateLog err:', err)
            }
            return
        }
        console.log("deltChinese:", JSON.stringify(deltChinese))

        recursionMergePath(deltChinese, walkJsonPath)

        // console.log('walkJsonPath Chinese:', walkJsonPath);

        console.log('addLogMap Chinese:', addLogMap);

        console.log('editLogMap Chinese:', editLogMap);

        console.log('removeLogMap Chinese:', removeLogMap);

        console.log('formatDiffToWord:');
        let diffWord = formatDiffToWord(wholeLog, addLogMap, editLogMap, removeLogMap)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };

    async testWarningCleanSchema(wholeLog, projectID) {
        let _this = this
        //生成addLogMap，editLogMap，removeLogMap的key，val
        const recursionMergePath = (obj, walkJsonPath) => {
            return Object.keys(obj).map(k => {
                // console.log("k:", k, "v:", obj[k])

                if (isNaN(k) && k !== '_t') {
                    let testNumber = k.replace('_', "");
                    if (isNaN(testNumber)) {
                        if (walkJsonPath === '') {
                            walkJsonPath = k
                        } else {
                            walkJsonPath = walkJsonPath + '>' + k
                        }
                    }
                }

                if (_.isPlainObject(obj[k])) {
                    recursionMergePath(obj[k], walkJsonPath)
                }

                if (obj[k] instanceof Array) {
                    switch (obj[k].length) {
                        case 1:
                            addLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                        case 2:
                            editLogMap[walkJsonPath] = obj[k][0] + ">" + obj[k][1];
                            splitWalk = walkJsonPath.split('>');
                            // console.log('splitWalk:',splitWalk)
                            splitWalkLength = splitWalk.length - 1;
                            // console.log('splitWalkLength:',splitWalkLength)
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                    // console.log('splitWalkKey:',k,'splitWalkVal:',splitWalk[k],'k !== splitWalkLength:',k !== splitWalkLength)
                                    // console.log(Object.prototype.toString.call(k))
                                    // console.log(Object.prototype.toString.call(splitWalkLength))
                                }
                            });
                            // console.log('splitwalkJsonPath:', walkJsonPath)
                            break;
                        case 3:
                            removeLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                    }
                }


                // const value = obj[k]
                // let title = properties[k] && properties[k].title
                // // console.log(value,title,properties)
                // if (!title && k === 'settings') title = '驱动设置'
                // if (!title || !properties) return
                // if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                //     const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                //     return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                // }
                // if (_.isPlainObject(value) && k !== 'logic') {
                //     let objProperties = properties && properties[k] && properties[k].properties
                //     if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                //         objProperties = properties && properties[k] && properties[k].items.properties
                //     }
                //     return {[title]: convertObjectKey(value, objProperties)}
                // } else {
                //     return {[title]: value}
                // }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // originToAvailable中的递归方法
        const convertObjectKey = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                let title = properties[k] && properties[k].title
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) return
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertObjectKey(value, objProperties)}
                } else {
                    return {[title]: value}
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // 原始数据转换为可读性数据(转中文map)
        const originToAvailable = (value, model, projectID) => {

            let data = {}
            // const nodeSchema = app.get('models')['Node']
            // const m = convertObjectKey(value, nodeSchema.properties)
            if (value && _.isPlainObject(value)) {
                Object.keys(value).forEach(objKey => {
                    // 资产
                    if (objKey === 'name') {
                        data['名称'] = value.name
                    } else if (objKey === 'range') {
                        data['归档范围类型'] = value.range
                    } else if (objKey === 'type') {
                        let u = []
                        if (value.type && value.type.length > 0) {
                            u = value.type.map(item => item.name).join(',')
                        }
                        data['归档报警类型'] = u
                    } else if (objKey === 'interval') {
                        data['归档周期'] = value.interval
                    } else if (objKey === 'hasArchive') {
                        data['删除归档报警'] = value.hasArchive
                    } else if (objKey === 'disable') {
                        data['禁用'] = value.disable
                    } else if (objKey === 'status') {
                        data['确认状态'] = value.status
                    } else if (objKey === 'processed') {
                        data['处理状态'] = value.processed
                    } else if (objKey === 'department') {
                        let u = []
                        if (value.department && value.department.length > 0) {
                            u = value.department.map(item => item.name).join(',')
                        }
                        data['归档报警部门'] = u
                    } else if (objKey === 'model') {
                        let u = []
                        if (value.model && value.model.length > 0) {
                            u = value.model.map(item => item.name).join(',')
                        }
                        data['归档报警模型'] = u
                    }
                })
            } else {
                data = value
            }

            return data
        };

        //格式化为一句话
        const formatDiffToWord = (wholeLog, addLogMap, editLogMap, removeLogMap) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + ":";
            data = data + "\n"
            if (addLogMap.length !== 0) {
                Object.keys(addLogMap).forEach(objKey => {
                    if (!addLogMap[objKey]) {
                        data = data + `<span style="color: green">添加</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!addLogMap[objKey].名称 && _.isPlainObject(addLogMap[objKey]) || addLogMap[objKey] instanceof Array) {
                    //     addLogMap[objKey] = JSON.stringify(addLogMap[objKey])
                    // }
                    if (addLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = ''
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (editLogMap.length !== 0) {
                Object.keys(editLogMap).forEach(objKey => {
                    if (!editLogMap[objKey]) {
                        return
                    }
                    let lastWordList = editLogMap[objKey].split('>');
                    let lastWord = '';
                    let originWord = '';
                    if (lastWordList.length > 1) {
                        originWord = lastWordList[0]
                        lastWord = lastWordList[lastWordList.length - 1]
                    }
                    data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ");"
                    data = data + "\n"
                })
            }
            if (removeLogMap.length !== 0) {
                Object.keys(removeLogMap).forEach(objKey => {
                    if (!removeLogMap[objKey]) {
                        data = data + `<span style="color: red">删除</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!removeLogMap[objKey].名称 && _.isPlainObject(removeLogMap[objKey]) || removeLogMap[objKey] instanceof Array) {
                    //     removeLogMap[objKey] = JSON.stringify(removeLogMap[objKey])
                    // }
                    if (removeLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = '';
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };

        let addLogMap = {};
        let editLogMap = {};
        let removeLogMap = {};

        let walkJsonPath = '';
        let splitWalk = [];
        let splitWalkLength = 0;


        // let wholeLog = {
        //     user: {
        //         "id": "22222",
        //         "name": "张三"
        //     },
        //     time: "2020-07-06T09:57:40+08:00",
        //     model: {"id": "5c934d3b839a5547624eaebf", "name": "采油井#"},
        //     oldData: {
        //         "computed": {
        //             "tags": [
        //                 {
        //                     "id": "YXZT",
        //                     "mapping": {
        //                         "id": "YXZT",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "运行状态",
        //                     "policy": "save",
        //                     "type": "mapping",
        //                 },
        //                 {
        //                     "id": "YW",
        //                     "mapping": {
        //                         "id": "YW",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "油温",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //                 {
        //                     "id": "YW2",
        //                     "mapping": {
        //                         "id": "YW2",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "油温2",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //             ]
        //         },
        //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站"},
        //         "custom": {"record": "测试记录"},
        //         "warning": {
        //             "rules": [
        //                 {
        //                     "alert": true,
        //                     "description": "注气焖井油温>200",
        //                     "disable": true,
        //                     "extraTags": [
        //                         {
        //                             "id": "lat"
        //                         }
        //                     ],
        //                     "handle": false,
        //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
        //                     "interval": 30,
        //                     "level": "中",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "==": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     200
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
        //                     "warningname": "注气报警"
        //                 },
        //                 {
        //                     "alert": true,
        //                     "description": "非注气状态报警",
        //                     "disable": true,
        //                     "handle": true,
        //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "!=": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     180
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "超限报警",
        //                     "warningname": "非注气状态"
        //                 }
        //             ]
        //         }
        //     },
        //     data: {
        //         "computed": {
        //             "tags": [
        //                 {
        //                     "id": "YXZT",
        //                     "mapping": {
        //                         "id": "YXZT",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "运行状态1",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //                 {
        //                     "id": "IA",
        //                     "mapping": {
        //                         "id": "IA",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "A相电流",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 }
        //             ],
        //         },
        //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站1"},
        //         "custom": {"record": "测试记录1", "select-4F10": "测试选择器111"},
        //         "warning": {
        //             "rules": [
        //                 {
        //                     "alert": true,
        //                     "description": "注气焖井油温>200",
        //                     "disable": true,
        //                     "extraTags": [
        //                         {
        //                             "id": "lat"
        //                         }
        //                     ],
        //                     "handle": false,
        //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
        //                     "interval": 30,
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "==": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     200
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
        //                     "warningname": "注气报警"
        //                 },
        //                 {
        //                     "alert": true,
        //                     "description": "非注气状态报警",
        //                     "disable": true,
        //                     "handle": true,
        //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "!=": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     180
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "超限报警",
        //                     "warningname": "非注气状态"
        //                 }
        //             ]
        //         }
        //     }
        // };

        let originStr = wholeLog.oldData
        let newStr = wholeLog.data

        let newChineseStr = originToAvailable(newStr, wholeLog, projectID);

        if (Object.keys(newChineseStr).length === 0) {
            console.log("newChineseStr global缓存pending")
            return
        }

        let originChineseStr = originToAvailable(originStr, wholeLog, projectID);

        console.log("newChineseStr:", JSON.stringify(newChineseStr))

        console.log("originChineseStr:", JSON.stringify(originChineseStr))

        //提取出两个json中不同的部分
        let deltChinese = jsondiffpatch.diff(originChineseStr, newChineseStr);

        if (!deltChinese) {
            let updateMap = {'diff': ''}
            try {
                let tokenProject = await _this.token(projectID)
                _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
            } catch (err) {
                console.error('updateLog err:', err)
            }
            return
        }
        console.log("deltChinese:", JSON.stringify(deltChinese))

        recursionMergePath(deltChinese, walkJsonPath)

        // console.log('walkJsonPath Chinese:', walkJsonPath);

        console.log('addLogMap Chinese:', addLogMap);

        console.log('editLogMap Chinese:', editLogMap);

        console.log('removeLogMap Chinese:', removeLogMap);

        console.log('formatDiffToWord:');
        let diffWord = formatDiffToWord(wholeLog, addLogMap, editLogMap, removeLogMap)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };

    async testFlowSchema(wholeLog, projectID) {
        let _this = this
        //生成addLogMap，editLogMap，removeLogMap的key，val
        const recursionMergePath = (obj, walkJsonPath) => {
            return Object.keys(obj).map(k => {
                // console.log("k:", k, "v:", obj[k])

                if (isNaN(k) && k !== '_t') {
                    let testNumber = k.replace('_', "");
                    if (isNaN(testNumber)) {
                        if (walkJsonPath === '') {
                            walkJsonPath = k
                        } else {
                            walkJsonPath = walkJsonPath + '>' + k
                        }
                    }
                }

                if (_.isPlainObject(obj[k])) {
                    recursionMergePath(obj[k], walkJsonPath)
                }

                if (obj[k] instanceof Array) {
                    switch (obj[k].length) {
                        case 1:
                            addLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                        case 2:
                            editLogMap[walkJsonPath] = obj[k][0] + ">" + obj[k][1];
                            splitWalk = walkJsonPath.split('>');
                            // console.log('splitWalk:',splitWalk)
                            splitWalkLength = splitWalk.length - 1;
                            // console.log('splitWalkLength:',splitWalkLength)
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                    // console.log('splitWalkKey:',k,'splitWalkVal:',splitWalk[k],'k !== splitWalkLength:',k !== splitWalkLength)
                                    // console.log(Object.prototype.toString.call(k))
                                    // console.log(Object.prototype.toString.call(splitWalkLength))
                                }
                            });
                            // console.log('splitwalkJsonPath:', walkJsonPath)
                            break;
                        case 3:
                            removeLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                    }
                }


                // const value = obj[k]
                // let title = properties[k] && properties[k].title
                // // console.log(value,title,properties)
                // if (!title && k === 'settings') title = '驱动设置'
                // if (!title || !properties) return
                // if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                //     const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                //     return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                // }
                // if (_.isPlainObject(value) && k !== 'logic') {
                //     let objProperties = properties && properties[k] && properties[k].properties
                //     if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                //         objProperties = properties && properties[k] && properties[k].items.properties
                //     }
                //     return {[title]: convertObjectKey(value, objProperties)}
                // } else {
                //     return {[title]: value}
                // }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // originToAvailable中的递归方法
        const convertObjectKey = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                let title = properties[k] && properties[k].title
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) return
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertObjectKey(value, objProperties)}
                } else {
                    return {[title]: value}
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // 原始数据转换为可读性数据(转中文map)
        const originToAvailable = (value, model, projectID) => {

            let data = {}
            // const nodeSchema = app.get('models')['Node']
            // const m = convertObjectKey(value, nodeSchema.properties)
            if (value && _.isPlainObject(value)) {
                Object.keys(value).forEach(objKey => {
                    // 资产
                    if (objKey === 'name') {
                        data['名称'] = value.name
                    } else if (objKey === 'settings') {
                        if (value.settings) {
                            data['流程配置'] = value.settings
                        }
                    } else if (objKey === 'type') {
                        data['流程类型'] = value.type
                    } else if (objKey === 'invalid') {
                        data['失效'] = value.invalid
                    } else if (objKey === 'disable') {
                        data['禁用'] = value.disable
                    } else if (objKey === 'startTime') {
                        data['开始时间'] = value.startTime
                    } else if (objKey === 'endTime') {
                        data['结束时间'] = value.endTime
                    } else if (objKey === 'extra') {
                        data['额外属性'] = value.extra
                    }
                })
            } else {
                data = value
            }

            return data
        };

        //格式化为一句话
        const formatDiffToWord = (wholeLog, addLogMap, editLogMap, removeLogMap) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + ":";
            data = data + "\n"
            if (addLogMap.length !== 0) {
                Object.keys(addLogMap).forEach(objKey => {
                    if (!addLogMap[objKey]) {
                        data = data + `<span style="color: green">添加</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!addLogMap[objKey].名称 && _.isPlainObject(addLogMap[objKey]) || addLogMap[objKey] instanceof Array) {
                    //     addLogMap[objKey] = JSON.stringify(addLogMap[objKey])
                    // }
                    if (addLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = ''
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (editLogMap.length !== 0) {
                Object.keys(editLogMap).forEach(objKey => {
                    if (!editLogMap[objKey]) {
                        return
                    }
                    let lastWordList = editLogMap[objKey].split('>');
                    let lastWord = '';
                    let originWord = '';
                    if (lastWordList.length > 1) {
                        originWord = lastWordList[0]
                        lastWord = lastWordList[lastWordList.length - 1]
                    }
                    data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ");"
                    data = data + "\n"
                })
            }
            if (removeLogMap.length !== 0) {
                Object.keys(removeLogMap).forEach(objKey => {
                    if (!removeLogMap[objKey]) {
                        data = data + `<span style="color: red">删除</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!removeLogMap[objKey].名称 && _.isPlainObject(removeLogMap[objKey]) || removeLogMap[objKey] instanceof Array) {
                    //     removeLogMap[objKey] = JSON.stringify(removeLogMap[objKey])
                    // }
                    if (removeLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = '';
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };

        let addLogMap = {};
        let editLogMap = {};
        let removeLogMap = {};

        let walkJsonPath = '';
        let splitWalk = [];
        let splitWalkLength = 0;


        // let wholeLog = {
        //     user: {
        //         "id": "22222",
        //         "name": "张三"
        //     },
        //     time: "2020-07-06T09:57:40+08:00",
        //     model: {"id": "5c934d3b839a5547624eaebf", "name": "采油井#"},
        //     oldData: {
        //         "computed": {
        //             "tags": [
        //                 {
        //                     "id": "YXZT",
        //                     "mapping": {
        //                         "id": "YXZT",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "运行状态",
        //                     "policy": "save",
        //                     "type": "mapping",
        //                 },
        //                 {
        //                     "id": "YW",
        //                     "mapping": {
        //                         "id": "YW",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "油温",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //                 {
        //                     "id": "YW2",
        //                     "mapping": {
        //                         "id": "YW2",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "油温2",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //             ]
        //         },
        //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站"},
        //         "custom": {"record": "测试记录"},
        //         "warning": {
        //             "rules": [
        //                 {
        //                     "alert": true,
        //                     "description": "注气焖井油温>200",
        //                     "disable": true,
        //                     "extraTags": [
        //                         {
        //                             "id": "lat"
        //                         }
        //                     ],
        //                     "handle": false,
        //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
        //                     "interval": 30,
        //                     "level": "中",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "==": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     200
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
        //                     "warningname": "注气报警"
        //                 },
        //                 {
        //                     "alert": true,
        //                     "description": "非注气状态报警",
        //                     "disable": true,
        //                     "handle": true,
        //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "!=": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     180
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "超限报警",
        //                     "warningname": "非注气状态"
        //                 }
        //             ]
        //         }
        //     },
        //     data: {
        //         "computed": {
        //             "tags": [
        //                 {
        //                     "id": "YXZT",
        //                     "mapping": {
        //                         "id": "YXZT",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "运行状态1",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //                 {
        //                     "id": "IA",
        //                     "mapping": {
        //                         "id": "IA",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "A相电流",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 }
        //             ],
        //         },
        //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站1"},
        //         "custom": {"record": "测试记录1", "select-4F10": "测试选择器111"},
        //         "warning": {
        //             "rules": [
        //                 {
        //                     "alert": true,
        //                     "description": "注气焖井油温>200",
        //                     "disable": true,
        //                     "extraTags": [
        //                         {
        //                             "id": "lat"
        //                         }
        //                     ],
        //                     "handle": false,
        //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
        //                     "interval": 30,
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "==": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     200
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
        //                     "warningname": "注气报警"
        //                 },
        //                 {
        //                     "alert": true,
        //                     "description": "非注气状态报警",
        //                     "disable": true,
        //                     "handle": true,
        //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "!=": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     180
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "超限报警",
        //                     "warningname": "非注气状态"
        //                 }
        //             ]
        //         }
        //     }
        // };

        let originStr = wholeLog.oldData
        let newStr = wholeLog.data

        let newChineseStr = originToAvailable(newStr, wholeLog, projectID);

        if (Object.keys(newChineseStr).length === 0) {
            console.log("newChineseStr global缓存pending")
            return
        }

        let originChineseStr = originToAvailable(originStr, wholeLog, projectID);

        console.log("newChineseStr:", JSON.stringify(newChineseStr))

        console.log("originChineseStr:", JSON.stringify(originChineseStr))

        //提取出两个json中不同的部分
        let deltChinese = jsondiffpatch.diff(originChineseStr, newChineseStr);

        if (!deltChinese) {
            let updateMap = {'diff': ''}
            try {
                let tokenProject = await _this.token(projectID)
                _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
            } catch (err) {
                console.error('updateLog err:', err)
            }
            return
        }
        console.log("deltChinese:", JSON.stringify(deltChinese))

        recursionMergePath(deltChinese, walkJsonPath)

        // console.log('walkJsonPath Chinese:', walkJsonPath);

        console.log('addLogMap Chinese:', addLogMap);

        console.log('editLogMap Chinese:', editLogMap);

        console.log('removeLogMap Chinese:', removeLogMap);

        console.log('formatDiffToWord:');
        let diffWord = formatDiffToWord(wholeLog, addLogMap, editLogMap, removeLogMap)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };

    async testFlowTaskSchema(wholeLog, projectID) {
        let _this = this
        //生成addLogMap，editLogMap，removeLogMap的key，val
        const recursionMergePath = (obj, walkJsonPath) => {
            return Object.keys(obj).map(k => {
                // console.log("k:", k, "v:", obj[k])

                if (isNaN(k) && k !== '_t') {
                    let testNumber = k.replace('_', "");
                    if (isNaN(testNumber)) {
                        if (walkJsonPath === '') {
                            walkJsonPath = k
                        } else {
                            walkJsonPath = walkJsonPath + '>' + k
                        }
                    }
                }

                if (_.isPlainObject(obj[k])) {
                    recursionMergePath(obj[k], walkJsonPath)
                }

                if (obj[k] instanceof Array) {
                    switch (obj[k].length) {
                        case 1:
                            addLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                        case 2:
                            editLogMap[walkJsonPath] = obj[k][0] + ">" + obj[k][1];
                            splitWalk = walkJsonPath.split('>');
                            // console.log('splitWalk:',splitWalk)
                            splitWalkLength = splitWalk.length - 1;
                            // console.log('splitWalkLength:',splitWalkLength)
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                    // console.log('splitWalkKey:',k,'splitWalkVal:',splitWalk[k],'k !== splitWalkLength:',k !== splitWalkLength)
                                    // console.log(Object.prototype.toString.call(k))
                                    // console.log(Object.prototype.toString.call(splitWalkLength))
                                }
                            });
                            // console.log('splitwalkJsonPath:', walkJsonPath)
                            break;
                        case 3:
                            removeLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                    }
                }


                // const value = obj[k]
                // let title = properties[k] && properties[k].title
                // // console.log(value,title,properties)
                // if (!title && k === 'settings') title = '驱动设置'
                // if (!title || !properties) return
                // if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                //     const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                //     return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                // }
                // if (_.isPlainObject(value) && k !== 'logic') {
                //     let objProperties = properties && properties[k] && properties[k].properties
                //     if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                //         objProperties = properties && properties[k] && properties[k].items.properties
                //     }
                //     return {[title]: convertObjectKey(value, objProperties)}
                // } else {
                //     return {[title]: value}
                // }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // originToAvailable中的递归方法
        const convertObjectKey = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                let title = properties[k] && properties[k].title
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) return
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertObjectKey(value, objProperties)}
                } else {
                    return {[title]: value}
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // 原始数据转换为可读性数据(转中文map)
        const originToAvailable = (value, model, projectID) => {

            let data = {}
            // const nodeSchema = app.get('models')['Node']
            // const m = convertObjectKey(value, nodeSchema.properties)
            if (value && _.isPlainObject(value)) {
                Object.keys(value).forEach(objKey => {
                    // 资产
                    if (objKey === 'jobKey') {
                        data['流程处理任务标识'] = value.jobKey
                    } else if (objKey === 'value') {
                        if (value.value) {
                            data['流程任务值'] = value.value
                        }
                    } else if (objKey === 'variables') {
                        if (value.variables) {
                            data['流程传输变量'] = value.variables
                        }
                    } else if (objKey === 'data') {
                        if (value.data) {
                            data['数据'] = value.data
                        }
                    } else if (objKey === 'type') {
                        data['流程类型'] = value.type
                    } else if (objKey === 'process') {
                        data['流程状态'] = value.process
                    } else if (objKey === 'elementId') {
                        data['流程节点标识'] = value.elementId
                    } else if (objKey === 'bpmnProcessId') {
                        data['bpmn唯一标识'] = value.bpmnProcessId
                    } else if (objKey === 'processinstancekey') {
                        data['流程实例'] = value.processinstancekey
                    } else if (objKey === 'processTime') {
                        data['处理时间'] = value.processTime
                    }
                })
            } else {
                data = value
            }

            return data
        };

        //格式化为一句话
        const formatDiffToWord = (wholeLog, addLogMap, editLogMap, removeLogMap) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + ":";
            data = data + "\n"
            if (addLogMap.length !== 0) {
                Object.keys(addLogMap).forEach(objKey => {
                    if (!addLogMap[objKey]) {
                        data = data + `<span style="color: green">添加</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!addLogMap[objKey].名称 && _.isPlainObject(addLogMap[objKey]) || addLogMap[objKey] instanceof Array) {
                    //     addLogMap[objKey] = JSON.stringify(addLogMap[objKey])
                    // }
                    if (addLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = ''
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (editLogMap.length !== 0) {
                Object.keys(editLogMap).forEach(objKey => {
                    if (!editLogMap[objKey]) {
                        return
                    }
                    let lastWordList = editLogMap[objKey].split('>');
                    let lastWord = '';
                    let originWord = '';
                    if (lastWordList.length > 1) {
                        originWord = lastWordList[0]
                        lastWord = lastWordList[lastWordList.length - 1]
                    }
                    data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ");"
                    data = data + "\n"
                })
            }
            if (removeLogMap.length !== 0) {
                Object.keys(removeLogMap).forEach(objKey => {
                    if (!removeLogMap[objKey]) {
                        data = data + `<span style="color: red">删除</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!removeLogMap[objKey].名称 && _.isPlainObject(removeLogMap[objKey]) || removeLogMap[objKey] instanceof Array) {
                    //     removeLogMap[objKey] = JSON.stringify(removeLogMap[objKey])
                    // }
                    if (removeLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = '';
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };

        let addLogMap = {};
        let editLogMap = {};
        let removeLogMap = {};

        let walkJsonPath = '';
        let splitWalk = [];
        let splitWalkLength = 0;


        // let wholeLog = {
        //     user: {
        //         "id": "22222",
        //         "name": "张三"
        //     },
        //     time: "2020-07-06T09:57:40+08:00",
        //     model: {"id": "5c934d3b839a5547624eaebf", "name": "采油井#"},
        //     oldData: {
        //         "computed": {
        //             "tags": [
        //                 {
        //                     "id": "YXZT",
        //                     "mapping": {
        //                         "id": "YXZT",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "运行状态",
        //                     "policy": "save",
        //                     "type": "mapping",
        //                 },
        //                 {
        //                     "id": "YW",
        //                     "mapping": {
        //                         "id": "YW",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "油温",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //                 {
        //                     "id": "YW2",
        //                     "mapping": {
        //                         "id": "YW2",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "油温2",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //             ]
        //         },
        //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站"},
        //         "custom": {"record": "测试记录"},
        //         "warning": {
        //             "rules": [
        //                 {
        //                     "alert": true,
        //                     "description": "注气焖井油温>200",
        //                     "disable": true,
        //                     "extraTags": [
        //                         {
        //                             "id": "lat"
        //                         }
        //                     ],
        //                     "handle": false,
        //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
        //                     "interval": 30,
        //                     "level": "中",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "==": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     200
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
        //                     "warningname": "注气报警"
        //                 },
        //                 {
        //                     "alert": true,
        //                     "description": "非注气状态报警",
        //                     "disable": true,
        //                     "handle": true,
        //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "!=": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     180
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "超限报警",
        //                     "warningname": "非注气状态"
        //                 }
        //             ]
        //         }
        //     },
        //     data: {
        //         "computed": {
        //             "tags": [
        //                 {
        //                     "id": "YXZT",
        //                     "mapping": {
        //                         "id": "YXZT",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "运行状态1",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 },
        //                 {
        //                     "id": "IA",
        //                     "mapping": {
        //                         "id": "IA",
        //                         "model": {
        //                             "id": "5d0215ef978032d7c9acaf62",
        //                             "name": "采油井4G"
        //                         },
        //                         "name": "运行状态",
        //                         "type": ""
        //                     },
        //                     "name": "A相电流",
        //                     "policy": "save",
        //                     "unit": "12656",
        //                 }
        //             ],
        //         },
        //         "model": {"id": "5c934c87839a5547624eae7c", "name": "接转站1"},
        //         "custom": {"record": "测试记录1", "select-4F10": "测试选择器111"},
        //         "warning": {
        //             "rules": [
        //                 {
        //                     "alert": true,
        //                     "description": "注气焖井油温>200",
        //                     "disable": true,
        //                     "extraTags": [
        //                         {
        //                             "id": "lat"
        //                         }
        //                     ],
        //                     "handle": false,
        //                     "id": "936194a7-ac61-4b87-9ddc-31e82cd31417",
        //                     "interval": 30,
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "==": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     200
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "b6840f5e-f547-4452-8e32-290887a43ff0",
        //                     "warningname": "注气报警"
        //                 },
        //                 {
        //                     "alert": true,
        //                     "description": "非注气状态报警",
        //                     "disable": true,
        //                     "handle": true,
        //                     "id": "7b56aab1-d730-4dce-b849-01744afa50b7",
        //                     "level": "高",
        //                     "logic": {
        //                         "and": [
        //                             {
        //                                 "!=": [
        //                                     {
        //                                         "var": [
        //                                             "node.custom.select-CF3B"
        //                                         ]
        //                                     },
        //                                     "2"
        //                                 ]
        //                             },
        //                             {
        //                                 ">": [
        //                                     {
        //                                         "var": [
        //                                             "var.YW"
        //                                         ]
        //                                     },
        //                                     180
        //                                 ]
        //                             }
        //                         ]
        //                     },
        //                     "type": "超限报警",
        //                     "warningname": "非注气状态"
        //                 }
        //             ]
        //         }
        //     }
        // };

        let originStr = wholeLog.oldData
        let newStr = wholeLog.data

        let newChineseStr = originToAvailable(newStr, wholeLog, projectID);

        if (Object.keys(newChineseStr).length === 0) {
            console.log("newChineseStr global缓存pending")
            return
        }

        let originChineseStr = originToAvailable(originStr, wholeLog, projectID);

        console.log("newChineseStr:", JSON.stringify(newChineseStr))

        console.log("originChineseStr:", JSON.stringify(originChineseStr))

        //提取出两个json中不同的部分
        let deltChinese = jsondiffpatch.diff(originChineseStr, newChineseStr);

        if (!deltChinese) {
            let updateMap = {'diff': ''}
            try {
                let tokenProject = await _this.token(projectID)
                _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
            } catch (err) {
                console.error('updateLog err:', err)
            }
            return
        }
        console.log("deltChinese:", JSON.stringify(deltChinese))

        recursionMergePath(deltChinese, walkJsonPath)

        // console.log('walkJsonPath Chinese:', walkJsonPath);

        console.log('addLogMap Chinese:', addLogMap);

        console.log('editLogMap Chinese:', editLogMap);

        console.log('removeLogMap Chinese:', removeLogMap);

        console.log('formatDiffToWord:');
        let diffWord = formatDiffToWord(wholeLog, addLogMap, editLogMap, removeLogMap)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };

    async testRoleSchema(wholeLog, projectID) {
        let _this = this
        //生成addLogMap，editLogMap，removeLogMap的key，val
        const recursionMergePath = (obj, walkJsonPath) => {
            return Object.keys(obj).map(k => {
                // console.log("k:", k, "v:", obj[k])

                if (isNaN(k) && k !== '_t') {
                    let testNumber = k.replace('_', "");
                    if (isNaN(testNumber)) {
                        if (walkJsonPath === '') {
                            walkJsonPath = k
                        } else {
                            walkJsonPath = walkJsonPath + '>' + k
                        }
                    }
                }

                if (_.isPlainObject(obj[k])) {
                    recursionMergePath(obj[k], walkJsonPath)
                }

                if (obj[k] instanceof Array) {
                    switch (obj[k].length) {
                        case 1:
                            addLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                        case 2:
                            editLogMap[walkJsonPath] = obj[k][0] + ">" + obj[k][1];
                            splitWalk = walkJsonPath.split('>');
                            // console.log('splitWalk:',splitWalk)
                            splitWalkLength = splitWalk.length - 1;
                            // console.log('splitWalkLength:',splitWalkLength)
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                    // console.log('splitWalkKey:',k,'splitWalkVal:',splitWalk[k],'k !== splitWalkLength:',k !== splitWalkLength)
                                    // console.log(Object.prototype.toString.call(k))
                                    // console.log(Object.prototype.toString.call(splitWalkLength))
                                }
                            });
                            // console.log('splitwalkJsonPath:', walkJsonPath)
                            break;
                        case 3:
                            removeLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                    }
                }


                // const value = obj[k]
                // let title = properties[k] && properties[k].title
                // // console.log(value,title,properties)
                // if (!title && k === 'settings') title = '驱动设置'
                // if (!title || !properties) return
                // if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                //     const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                //     return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                // }
                // if (_.isPlainObject(value) && k !== 'logic') {
                //     let objProperties = properties && properties[k] && properties[k].properties
                //     if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                //         objProperties = properties && properties[k] && properties[k].items.properties
                //     }
                //     return {[title]: convertObjectKey(value, objProperties)}
                // } else {
                //     return {[title]: value}
                // }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // originToAvailable中的递归方法
        const convertObjectKey = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                let title = properties[k] && properties[k].title
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) return
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertObjectKey(value, objProperties)}
                } else {
                    return {[title]: value}
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // 原始数据转换为可读性数据(转中文map)
        const originToAvailable = (value, model, projectID) => {

            let data = {}
            // const nodeSchema = app.get('models')['Node']
            // const m = convertObjectKey(value, nodeSchema.properties)
            if (value && _.isPlainObject(value)) {
                Object.keys(value).forEach(objKey => {
                    // 资产
                    if (objKey === 'name') {
                        data['名称'] = value.name
                    } else if (objKey === 'description') {
                        data['角色描述'] = value.description
                    } else if (objKey === 'permission') {
                        if (value.permission) {
                            data['角色管理的权限'] = value.permission
                        }
                    } else if (objKey === 'users') {
                        let u = []
                        if (value.users && value.users.length > 0) {
                            u = value.users.map(item => item.name).join(',')
                        }
                        data['用户'] = u
                    } else if (objKey === 'dashboard') {
                        if (value.dashboard) {
                            data['画面'] = value.dashboard
                        }
                    } else if (objKey === 'disabled') {
                        data['禁用'] = value.disabled
                    }
                })
            } else {
                data = value
            }

            return data
        };

        //格式化为一句话
        const formatDiffToWord = (wholeLog, addLogMap, editLogMap, removeLogMap) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + ":";
            data = data + "\n"
            if (addLogMap.length !== 0) {
                Object.keys(addLogMap).forEach(objKey => {
                    if (!addLogMap[objKey]) {
                        data = data + `<span style="color: green">添加</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!addLogMap[objKey].名称 && _.isPlainObject(addLogMap[objKey]) || addLogMap[objKey] instanceof Array) {
                    //     addLogMap[objKey] = JSON.stringify(addLogMap[objKey])
                    // }
                    if (addLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = ''
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (editLogMap.length !== 0) {
                Object.keys(editLogMap).forEach(objKey => {
                    if (!editLogMap[objKey]) {
                        return
                    }
                    let lastWordList = editLogMap[objKey].split('>');
                    let lastWord = '';
                    let originWord = '';
                    if (lastWordList.length > 1) {
                        originWord = lastWordList[0]
                        lastWord = lastWordList[lastWordList.length - 1]
                    }
                    data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ");"
                    data = data + "\n"
                })
            }
            if (removeLogMap.length !== 0) {
                Object.keys(removeLogMap).forEach(objKey => {
                    if (!removeLogMap[objKey]) {
                        data = data + `<span style="color: red">删除</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!removeLogMap[objKey].名称 && _.isPlainObject(removeLogMap[objKey]) || removeLogMap[objKey] instanceof Array) {
                    //     removeLogMap[objKey] = JSON.stringify(removeLogMap[objKey])
                    // }
                    if (removeLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = '';
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };

        let addLogMap = {};
        let editLogMap = {};
        let removeLogMap = {};

        let walkJsonPath = '';
        let splitWalk = [];
        let splitWalkLength = 0;


        let originStr = wholeLog.oldData
        let newStr = wholeLog.data

        let newChineseStr = originToAvailable(newStr, wholeLog, projectID);

        if (Object.keys(newChineseStr).length === 0) {
            console.log("newChineseStr global缓存pending")
            return
        }

        let originChineseStr = originToAvailable(originStr, wholeLog, projectID);

        console.log("newChineseStr:", JSON.stringify(newChineseStr))

        console.log("originChineseStr:", JSON.stringify(originChineseStr))

        //提取出两个json中不同的部分
        let deltChinese = jsondiffpatch.diff(originChineseStr, newChineseStr);

        if (!deltChinese) {
            let updateMap = {'diff': ''}
            try {
                let tokenProject = await _this.token(projectID)
                _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
            } catch (err) {
                console.error('updateLog err:', err)
            }
            return
        }
        console.log("deltChinese:", JSON.stringify(deltChinese))

        recursionMergePath(deltChinese, walkJsonPath)

        // console.log('walkJsonPath Chinese:', walkJsonPath);

        console.log('addLogMap Chinese:', addLogMap);

        console.log('editLogMap Chinese:', editLogMap);

        console.log('removeLogMap Chinese:', removeLogMap);

        console.log('formatDiffToWord:');
        let diffWord = formatDiffToWord(wholeLog, addLogMap, editLogMap, removeLogMap)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };

    async testSystemVariableSchema(wholeLog, projectID) {
        let _this = this
        //生成addLogMap，editLogMap，removeLogMap的key，val
        const recursionMergePath = (obj, walkJsonPath) => {
            return Object.keys(obj).map(k => {
                // console.log("k:", k, "v:", obj[k])

                if (isNaN(k) && k !== '_t') {
                    let testNumber = k.replace('_', "");
                    if (isNaN(testNumber)) {
                        if (walkJsonPath === '') {
                            walkJsonPath = k
                        } else {
                            walkJsonPath = walkJsonPath + '>' + k
                        }
                    }
                }

                if (_.isPlainObject(obj[k])) {
                    recursionMergePath(obj[k], walkJsonPath)
                }

                if (obj[k] instanceof Array) {
                    switch (obj[k].length) {
                        case 1:
                            addLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                        case 2:
                            editLogMap[walkJsonPath] = obj[k][0] + ">" + obj[k][1];
                            splitWalk = walkJsonPath.split('>');
                            // console.log('splitWalk:',splitWalk)
                            splitWalkLength = splitWalk.length - 1;
                            // console.log('splitWalkLength:',splitWalkLength)
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                    // console.log('splitWalkKey:',k,'splitWalkVal:',splitWalk[k],'k !== splitWalkLength:',k !== splitWalkLength)
                                    // console.log(Object.prototype.toString.call(k))
                                    // console.log(Object.prototype.toString.call(splitWalkLength))
                                }
                            });
                            // console.log('splitwalkJsonPath:', walkJsonPath)
                            break;
                        case 3:
                            removeLogMap[walkJsonPath] = obj[k][0];
                            splitWalk = walkJsonPath.split('>');
                            splitWalkLength = splitWalk.length - 1;
                            walkJsonPath = '';
                            Object.keys(splitWalk).map(k => {
                                if (k !== splitWalkLength.toString()) {
                                    if (walkJsonPath === '') {
                                        walkJsonPath = splitWalk[k]
                                    } else {
                                        walkJsonPath = walkJsonPath + '>' + splitWalk[k]
                                    }
                                }
                            });
                            break;
                    }
                }


                // const value = obj[k]
                // let title = properties[k] && properties[k].title
                // // console.log(value,title,properties)
                // if (!title && k === 'settings') title = '驱动设置'
                // if (!title || !properties) return
                // if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                //     const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                //     return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                // }
                // if (_.isPlainObject(value) && k !== 'logic') {
                //     let objProperties = properties && properties[k] && properties[k].properties
                //     if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                //         objProperties = properties && properties[k] && properties[k].items.properties
                //     }
                //     return {[title]: convertObjectKey(value, objProperties)}
                // } else {
                //     return {[title]: value}
                // }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // originToAvailable中的递归方法
        const convertObjectKey = (obj, properties) => {
            return Object.keys(obj).map(k => {
                const value = obj[k]
                let title = properties[k] && properties[k].title
                // console.log(value,title,properties)
                if (!title && k === 'settings') title = '驱动设置'
                if (!title || !properties) return
                if (_.isArray(value) && k !== 'enum' && k !== 'enum_title') {
                    const arrProperties = properties[k].items && properties[k].items.type === 'object' ? properties[k].items.properties : null
                    return {[title]: value.map(item => convertObjectKey(item, arrProperties))}
                }
                if (_.isPlainObject(value) && k !== 'logic') {
                    let objProperties = properties && properties[k] && properties[k].properties
                    if (properties[k] && properties[k].items && properties[k].items.type === 'array') {
                        objProperties = properties && properties[k] && properties[k].items.properties
                    }
                    return {[title]: convertObjectKey(value, objProperties)}
                } else {
                    return {[title]: value}
                }
            }).reduce((prev, curr) => ({...prev, ...curr}), {})
        };

        // 原始数据转换为可读性数据(转中文map)
        const originToAvailable = (value, model, projectID) => {

            let data = {}
            // const nodeSchema = app.get('models')['Node']
            // const m = convertObjectKey(value, nodeSchema.properties)
            if (value && _.isPlainObject(value)) {
                Object.keys(value).forEach(objKey => {
                    // 资产
                    if (objKey === 'name') {
                        data['名称'] = value.name
                    } else if (objKey === 'type') {
                        data['类型'] = value.type
                    } else if (objKey === 'uid') {
                        data['编号'] = value.uid
                    } else if (objKey === 'value') {
                        data['值'] = value.value
                    }
                })
            } else {
                data = value
            }

            return data
        };

        //格式化为一句话
        const formatDiffToWord = (wholeLog, addLogMap, editLogMap, removeLogMap) => {
            let formatTime = wholeLog.time.replace(new RegExp('T', 'g'), " ");
            formatTime = formatTime.replace(new RegExp('\\+08:00', 'g'), "");
            formatTime = formatTime.replace(new RegExp('\\.\\d{3}', 'g'), "");
            let data = (wholeLog.user && wholeLog.user.name ? wholeLog.user.name : '未知用户') + '于' + formatTime + ":";
            data = data + "\n"
            if (addLogMap.length !== 0) {
                Object.keys(addLogMap).forEach(objKey => {
                    if (!addLogMap[objKey]) {
                        data = data + `<span style="color: green">添加</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!addLogMap[objKey].名称 && _.isPlainObject(addLogMap[objKey]) || addLogMap[objKey] instanceof Array) {
                    //     addLogMap[objKey] = JSON.stringify(addLogMap[objKey])
                    // }
                    if (addLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = ''
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: green">添加</span>:` + "名称为" + addLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: green">添加</span>:` + "值" + utilsMethod.limitStringLength(addLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (editLogMap.length !== 0) {
                Object.keys(editLogMap).forEach(objKey => {
                    if (!editLogMap[objKey]) {
                        return
                    }
                    let lastWordList = editLogMap[objKey].split('>');
                    let lastWord = '';
                    let originWord = '';
                    if (lastWordList.length > 1) {
                        originWord = lastWordList[0]
                        lastWord = lastWordList[lastWordList.length - 1]
                    }
                    data = data + `<span style="color: blue">编辑</span>:` + "值从" + utilsMethod.limitStringLength(originWord) + "变为" + utilsMethod.limitStringLength(lastWord) + "(位置:" + objKey + ");"
                    data = data + "\n"
                })
            }
            if (removeLogMap.length !== 0) {
                Object.keys(removeLogMap).forEach(objKey => {
                    if (!removeLogMap[objKey]) {
                        data = data + `<span style="color: red">删除</span>:` + "值null" + "(位置:" + objKey + ");";
                        data = data + "\n";
                        return
                    }
                    // if (!removeLogMap[objKey].名称 && _.isPlainObject(removeLogMap[objKey]) || removeLogMap[objKey] instanceof Array) {
                    //     removeLogMap[objKey] = JSON.stringify(removeLogMap[objKey])
                    // }
                    if (removeLogMap[objKey].名称) {
                        let lastWordList = objKey.split('>');
                        let lastWord = '';
                        if (lastWordList.length !== 0) {
                            lastWord = lastWordList[lastWordList.length - 1]
                        }
                        data = data + `<span style="color: red">删除</span>:` + "名称为" + removeLogMap[objKey].名称 + "的" + lastWord + "(位置:" + objKey + ");"
                        data = data + "\n"
                    } else {
                        data = data + `<span style="color: red">删除</span>:` + "值" + utilsMethod.limitStringLength(removeLogMap[objKey]) + "(位置:" + objKey + ");"
                        data = data + "\n"
                    }
                })
            }
            if (data.endsWith('\n')) {
                data = data.substring(0, data.length - 2);
            }
            return data
        };

        let addLogMap = {};
        let editLogMap = {};
        let removeLogMap = {};

        let walkJsonPath = '';
        let splitWalk = [];
        let splitWalkLength = 0;


        let originStr = wholeLog.oldData
        let newStr = wholeLog.data

        let newChineseStr = originToAvailable(newStr, wholeLog, projectID);

        if (Object.keys(newChineseStr).length === 0) {
            console.log("newChineseStr global缓存pending")
            return
        }

        let originChineseStr = originToAvailable(originStr, wholeLog, projectID);

        console.log("newChineseStr:", JSON.stringify(newChineseStr))

        console.log("originChineseStr:", JSON.stringify(originChineseStr))

        //提取出两个json中不同的部分
        let deltChinese = jsondiffpatch.diff(originChineseStr, newChineseStr);

        if (!deltChinese) {
            let updateMap = {'diff': ''}
            try {
                let tokenProject = await _this.token(projectID)
                _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
            } catch (err) {
                console.error('updateLog err:', err)
            }
            return
        }
        console.log("deltChinese:", JSON.stringify(deltChinese))

        recursionMergePath(deltChinese, walkJsonPath)

        // console.log('walkJsonPath Chinese:', walkJsonPath);

        console.log('addLogMap Chinese:', addLogMap);

        console.log('editLogMap Chinese:', editLogMap);

        console.log('removeLogMap Chinese:', removeLogMap);

        console.log('formatDiffToWord:');
        let diffWord = formatDiffToWord(wholeLog, addLogMap, editLogMap, removeLogMap)
        console.log(diffWord);
        let updateMap = {'diff': diffWord}
        try {
            let tokenProject = await _this.token(projectID)
            _this.updateLog(tokenProject, _this.config, updateMap, wholeLog.id, projectID)
        } catch (err) {
            console.error('updateLog err:', err)
        }
    };



    run() {
        let _this = this;

        this.getValue(keyToRead)
            .then((value) => {
                let parsedData = JSON.parse(value)
                if (parsedData.App.MQ.TYPE === 'rabbit') {
                    if (parsedData.App.MQ.RabbitMQ.Password.startsWith('ENC(') &&  parsedData.App.MQ.RabbitMQ.Password.endsWith(')')) {
                        let buffer = Buffer.from(parsedData.App.MQ.RabbitMQ.Password.slice(4,parsedData.App.MQ.RabbitMQ.Password.length-1), 'base64'); // 使用Base64编码的字符串
                        parsedData.App.MQ.RabbitMQ.Password = buffer.toString('utf8'); // 将Buffer转换为UTF-8字符串
                    }
                    let mqConfig = {host:parsedData.App.MQ.RabbitMQ.Host,port:parsedData.App.MQ.RabbitMQ.Port,username:parsedData.App.MQ.RabbitMQ.Username,password:parsedData.App.MQ.RabbitMQ.Password}
                    this.mqClient = new Rabbit(mqConfig)
                } else if (parsedData.App.MQ.TYPE === 'kafka') {
                    let mqConfig = {brokers:parsedData.App.MQ.KAFKA.Brokers,groupId:parsedData.App.MQ.KAFKA.GroupID}
                    this.mqClient = new KafkaJS(mqConfig)
                } else {
                    if (parsedData.App.MQ.MQTT.Password.startsWith('ENC(') &&  parsedData.App.MQ.MQTT.Password.endsWith(')')) {
                        let buffer = Buffer.from(parsedData.App.MQ.MQTT.Password.slice(4,parsedData.App.MQ.MQTT.Password.length-1), 'base64'); // 使用Base64编码的字符串
                        parsedData.App.MQ.MQTT.Password = buffer.toString('utf8');
                    }
                    let mqConfig = {host:parsedData.App.MQ.MQTT.Host,port:parsedData.App.MQ.MQTT.Port,username:parsedData.App.MQ.MQTT.Username,password:parsedData.App.MQ.MQTT.Password}
                    this.mqClient = new Mqtt(mqConfig)
                }
                let apiConfig = {}
                apiConfig.endpoint = config.api.endpoint
                apiConfig.type = parsedData.App.API.Type
                apiConfig.ak = parsedData.App.API.AK
                apiConfig.sk = parsedData.App.API.SK
                this.apiCli = new ApiClient(apiConfig);

                let _this = this;
                _this.mqttInit()
                utilsMethod.getSchema(_this.apiCli);

// console.log("schemaCache:",getSchema())

                setInterval(function () {
                    utilsMethod.getSchema(_this.apiCli)
                }, 1000 * 60);



            }).catch(error => console.log(error));

        // _this.token().then((token) => {
        //     // console.log('token:', token)
        //     _this.tokenStr = token
        //     _this.mqttInit()
        // }).catch(err => console.error(err))




        process.stdin.resume();
        let handler = function (signal) {
            console.log(`收到信号 ${signal}`);
            if (_this.mqClient) {
                _this.mqClient.close()
            }
            process.exit(0)
        }
        process.on('SIGINT', handler);
        process.on('SIGTERM', handler);
    }
}


module.exports = SchemaCacheGenerate